/* OdLib_Io - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.opendocument.io.odtable;


/**
 *
 * @author Vincent Calame
 */
public final class OdCellKey {

    private final short type;
    private final int currencyInteger;
    private final String styleName;

    private OdCellKey(short type, int currencyInteger, String styleName) {
        this.type = type;
        this.currencyInteger = currencyInteger;
        this.styleName = styleName;
    }

    public short getType() {
        return type;
    }

    public int getCurrencyInteger() {
        return currencyInteger;
    }

    public String getStyleName() {
        return styleName;
    }

    @Override
    public int hashCode() {
        int code = type + currencyInteger;
        if (styleName != null) {
            code = styleName.hashCode();
        }
        return code;
    }

    @Override
    public boolean equals(Object other) {
        if (other == null) {
            return false;
        }
        if (other == this) {
            return true;
        }
        if (other.getClass() != this.getClass()) {
            return false;
        }
        OdCellKey odCellKey = (OdCellKey) other;
        if (odCellKey.type != this.type) {
            return false;
        }
        if (odCellKey.currencyInteger != this.currencyInteger) {
            return false;
        }
        if (odCellKey.styleName == null) {
            if (this.styleName == null) {
                return true;
            } else {
                return false;
            }
        } else if (this.styleName == null) {
            return false;
        } else {
            return odCellKey.styleName.equals(this.styleName);
        }
    }

    public static OdCellKey newInstance(short type, int currencyInteger, String styleName) {
        if (type != OdColumnDef.CURRENCY_TYPE) {
            currencyInteger = 0;
        }
        return new OdCellKey(type, currencyInteger, styleName);
    }

    public static OdCellKey newDefaultInstance(String styleName) {
        return new OdCellKey(OdColumnDef.DEFAULT_TYPE, 0, styleName);
    }

    public static OdCellKey newISODateInstance(String styleName) {
        return new OdCellKey(OdColumnDef.ISODATE_TYPE, 0, styleName);
    }

    public static OdCellKey newCurrencyInstance(int currencyInteger, String styleName) {
        return new OdCellKey(OdColumnDef.CURRENCY_TYPE, currencyInteger, styleName);
    }

}
