/* OdLib_Io - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.opendocument.io.odtable;

import java.util.ArrayList;
import java.util.List;


/**
 *
 * @author Vincent Calame
 */
public class OdTableDefBuilder {

    private final List<OdColumnDef> odColumnList = new ArrayList<OdColumnDef>();

    public OdTableDefBuilder() {
    }

    public void addDefaultColumns(String category) {
        odColumnList.add(new InternalOdColumnDef(OdColumnDef.DEFAULT_TYPE, 0, category));
    }

    public void addISODateColumn(String category) {
        odColumnList.add(new InternalOdColumnDef(OdColumnDef.ISODATE_TYPE, 0, category));
    }

    public void addCurrencyColumn(int currencyInteger, String category) {
        odColumnList.add(new InternalOdColumnDef(OdColumnDef.CURRENCY_TYPE, currencyInteger, category));
    }

    public OdTableDef toOdTableDef() {
        OdColumnDef[] odColumnArray = odColumnList.toArray(new OdColumnDef[odColumnList.size()]);
        return new InternalOdTableDef(odColumnArray);
    }


    private static class InternalOdTableDef implements OdTableDef {

        private final OdColumnDef[] odColumnArray;

        private InternalOdTableDef(OdColumnDef[] odColumnArray) {
            this.odColumnArray = odColumnArray;
        }

        @Override
        public int getOdColumnDefCount() {
            return odColumnArray.length;
        }

        @Override
        public OdColumnDef getOdColumnDef(int i) {
            return odColumnArray[i];
        }

    }


    private static class InternalOdColumnDef implements OdColumnDef {

        private final short type;
        private final int currencyInteger;
        private final String category;

        private InternalOdColumnDef(short type, int currencyInteger, String category) {
            this.type = type;
            this.currencyInteger = currencyInteger;
            this.category = category;
        }

        @Override
        public short getType() {
            return type;
        }

        @Override
        public int getCurrencyInteger() {
            return currencyInteger;
        }

        public String getCategory() {
            return category;
        }

    }

}
