/* Scrutari - Copyright (c) 2008-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets;

import fr.exemole.sctserver.api.storage.LogStorage;
import fr.exemole.sctserver.request.xml.XmlRequestUtils;
import fr.exemole.sctserver.tools.log.SctLogUtils;
import net.mapeadores.util.io.DocStream;
import net.mapeadores.util.io.MimeTypeConstants;
import net.mapeadores.util.io.docstream.ClassResourceDocStream;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.servlets.handlers.DocStreamResponseHandler;
import net.mapeadores.util.servlets.handlers.SimpleResponseHandler;


/**
 *
 * @author Vincent Calame
 */
public class ResponseHandlerFactory {

    private ResponseHandlerFactory() {
    }

    public static ResponseHandler getCssHandler(String name) {
        DocStream docStream = getCssDocStream(name.substring(1));
        if (docStream == null) {
            return null;
        }
        return new DocStreamResponseHandler(docStream);
    }

    public static ResponseHandler getDtdHandler(String name) {
        DocStream docStream = getDtdDocStream(name.substring(1));
        if (docStream == null) {
            return null;
        }
        return new DocStreamResponseHandler(docStream);
    }

    public static ResponseHandler getJsHandler(String name) {
        DocStream docStream = getJsDocStream(name.substring(1));
        if (docStream == null) {
            return null;
        }
        return new DocStreamResponseHandler(docStream);
    }

    public static ResponseHandler getLogsHandler(String path, LogStorage logStorage) {
        int idx = path.indexOf('/', 1);
        if (idx == -1) {
            return null;
        }
        short type;
        try {
            type = SctLogUtils.logTypeToShort(path.substring(1, idx));
        } catch (IllegalArgumentException iae) {
            return null;
        }
        String name = path.substring(idx + 1);
        int idx2 = name.lastIndexOf('.');
        if (idx2 == -1) {
            return null;
        }
        name = name.substring(0, idx2);
        String content = logStorage.getLog(type, name);
        if (content == null) {
            return null;
        }
        return new SimpleResponseHandler(content);
    }

    public static ResponseHandler getPngHandler(String name) {
        DocStream docStream = getPngDocStream(name.substring(1));
        if (docStream == null) {
            return null;
        }
        return new DocStreamResponseHandler(docStream);
    }

    private static DocStream getCssDocStream(String fileName) {
        ClassResourceDocStream docStream = getDocStream("resources/theme/css/" + fileName);
        if (docStream == null) {
            return null;
        }
        docStream.setMimeType(MimeTypeConstants.CSS);
        return docStream;
    }

    private static DocStream getDtdDocStream(String fileName) {
        ClassResourceDocStream docStream = ClassResourceDocStream.newInstance(XmlRequestUtils.class, "dtd/" + fileName);
        if (docStream == null) {
            return null;
        }
        docStream.setMimeType(MimeTypeConstants.DTD);
        return docStream;
    }

    private static DocStream getJsDocStream(String fileName) {
        ClassResourceDocStream docStream = getDocStream("resources/js/" + fileName);
        if (docStream == null) {
            return null;
        }
        docStream.setMimeType(MimeTypeConstants.JAVASCRIPT);
        return docStream;
    }

    private static DocStream getPngDocStream(String fileName) {
        ClassResourceDocStream docStream = getDocStream("resources/theme/png/" + fileName);
        if (docStream == null) {
            return null;
        }
        docStream.setMimeType("image/png");
        return docStream;
    }

    private static ClassResourceDocStream getDocStream(String resourcePath) {
        return ClassResourceDocStream.newInstance(ResponseHandlerFactory.class, resourcePath);
    }

}
