/* Scrutari - Copyright (c) 2008-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin.htmlpages;

import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.api.ScrutariSession;
import fr.exemole.sctserver.servlets.admin.AdminConstants;
import fr.exemole.sctserver.servlets.admin.AdminRequest;
import fr.exemole.sctserver.servlets.resources.theme.SctCssCatalog;
import java.io.IOException;
import java.util.Locale;
import java.util.ResourceBundle;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.html.HtmlPrinterUtils;
import net.mapeadores.util.html.UtilHtmlPrinter;
import net.mapeadores.util.localisation.BundleLocalisation;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.localisation.ResourceLocalisation;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.servlets.ResponseHandler;


/**
 *
 * @author Vincent Calame
 */
public abstract class AdminHtmlPage extends UtilHtmlPrinter implements SctCssCatalog, ResponseHandler, AdminConstants {

    private final static HTMLAttributes ERROR_ATTR = HA.classes(ERRORMESSAGE_SPANCLASS);
    private final static HTMLAttributes DONE_ATTR = HA.classes(DONEMESSAGE_SPANCLASS);
    private final static HTMLAttributes TOOLDIV_ATTR = HA.classes(TOOL_DIVCLASS);
    private final static HTMLAttributes CURRENT_ATTR = HA.classes(CURRENT_SPANCLASS);
    protected SctServer sctServer;
    protected ScrutariSession scrutariSession;
    protected int workingLangInteger;
    private CommandMessage commandMessage;
    private short currentPage;

    public AdminHtmlPage() {
        addCssUrl("../css/admin.css");
    }

    protected void init(SctServer sctServer) {
        this.sctServer = sctServer;
        this.scrutariSession = sctServer.getScrutariSession();
    }

    public void setLocalisationInfo(LocalisationInfo localisationInfo) {
        this.workingLangInteger = localisationInfo.getWorkingLangInteger();
        Locale locale = localisationInfo.getFormatLocale();
        ResourceLocalisation textResourceLocalisation = new BundleLocalisation(ResourceBundle.getBundle("fr.exemole.sctserver.servlets.resources.l10n.sctadmin", locale));
        addResourceLocalisation(textResourceLocalisation);
        ResourceLocalisation commandResourceLocalisation = new BundleLocalisation(ResourceBundle.getBundle("fr.exemole.sctserver.servlets.resources.l10n.sctfb", locale));
        setCommandKeyLocalisation(commandResourceLocalisation);

    }

    public void setCurrentPage(short currentPage) {
        this.currentPage = currentPage;
    }

    public void setCommandMessage(CommandMessage commandMessage) {
        this.commandMessage = commandMessage;
    }

    @Override
    public long getLastModified() {
        return ResponseHandler.LASTMODIFIED_NOCACHE;
    }

    protected void initResponse(HttpServletResponse response) throws IOException {
        response.setContentType("text/html;charset=UTF-8");
        initPrinter(response.getWriter());

    }

    protected void printTools() {
        startDiv(TOOLDIV_ATTR);
        printToolLink(INDEX_PAGE);
        printToolSeparator();
        printToolLink(STATS_PAGE);
        printToolSeparator();
        printToolLink(FICHES_PAGE);
        printToolSeparator();
        printToolLink(MOTSCLES_PAGE);
        printToolSeparator();
        printToolLink(LEXIES_PAGE);
        printToolSeparator();
        printToolLink(JOURNAUX_PAGE);
        endDiv();

    }

    protected void printMessage() {
        if (commandMessage != null) {
            HtmlPrinterUtils.printCommandMessage(this, commandMessage, DONE_ATTR, ERROR_ATTR);
        }
    }

    protected void start() {
        start(workingLangInteger);
    }

    private void printToolSeparator() {
        append(" — ");
    }

    private void printToolLink(short page) {
        String key = AdminRequest.pageToString(page);
        if (currentPage == page) {
            startSpan(CURRENT_ATTR);
        } else {
            startA(HA.href(key + ".html").addClass(PAGE_ACLASS));
        }
        localize("page_" + key);
        if (currentPage == page) {
            endSpan();
        } else {
            endA();
        }
    }

    public SctServer getSctServer() {
        return sctServer;
    }

}
