/* Scrutari - Copyright (c) 2008-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin.htmlpages;

import fr.exemole.sctserver.SctServer;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.langinteger.LangInteger;
import net.scrutari.data.BaseData;
import net.scrutari.data.CorpusData;
import net.scrutari.data.FicheData;
import net.scrutari.db.ScrutariDB;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.tools.util.LibelleUtils;


/**
 *
 * @author Vincent Calame
 */
public class FichesPage extends AdminHtmlPage {

    HTMLAttributes baseAttributes = HA.classes("d_SCT_lst_base");
    HTMLAttributes corpusAttributes = HA.classes("l_SCT_lst_corpus");
    HTMLAttributes ficheAttributes = HA.classes("l_SCT_lst_fiche");
    HTMLAttributes ulAttributes = HA.classes("u_SCT_lst_u").style("display: none;");
    private final ScrutariDB scrutariDB;
    private int langUi = LangInteger.build("fr");

    public FichesPage(SctServer sctServer) {
        init(sctServer);
        scrutariDB = sctServer.getScrutariSession().getScrutariDB();
        addCssUrl("../css/listes.css");
        addJsUrl("../js/affichage.js");
    }

    @Override
    public void handleResponse(HttpServletResponse response) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        initResponse(response);
        start();
        printTools();
        printMessage();

        int baseCount = dataAccess.getBaseCount();
        for (int i = 0; i < baseCount; i++) {
            BaseData baseData = dataAccess.getBaseDataByIndex(i);
            printBaseData(baseData, dataAccess);
        }
        end();
        dataAccess.close();
    }

    private void printBaseData(BaseData baseData, DataAccess dataAccess) {
        int codebase = baseData.getCodeBase();
        int corpusCount = baseData.getCorpusCount();
        startDiv(baseAttributes.attr("onclick", "affiche('ul_" + codebase + "');"));
        escape(LibelleUtils.getLibelleString(baseData, BaseData.INTITULE_LONG, langUi, "?"));
        append(" ");
        startCode();
        append("[");
        append(String.valueOf(corpusCount));
        append("]");
        endDiv();
        startUl(ulAttributes.id("ul_" + codebase));
        for (int i = 0; i < corpusCount; i++) {
            CorpusData corpusData = dataAccess.getCorpusDataByCode(baseData.getCodeCorpus(i));
            printCorpusData(corpusData, dataAccess);
        }
        endUl();
    }

    private int printCorpusData(CorpusData corpusData, DataAccess dataAccess) {
        int codecorpus = corpusData.getCodeCorpus();
        int ficheCount = corpusData.getFicheCount();
        String onClick = "affiche('ul_" + codecorpus + "');";
        startLi(corpusAttributes.attr("onclick", onClick));
        escape(LibelleUtils.getLibelleString(corpusData, CorpusData.INTITULECORPUS_FIELDNUMBER, langUi, "?"));
        append(" ");
        startCode();
        append("[");
        append(String.valueOf(ficheCount));
        append("]");
        endCode();
        endLi();
        startUl(ulAttributes.attr("onclick", onClick).id("ul_" + codecorpus));
        for (int i = 0; i < ficheCount; i++) {
            FicheData ficheData = dataAccess.getFicheDataByCode(corpusData.getCodeFiche(i));
            printFicheData(ficheData);
        }
        endUl();
        return ficheCount;
    }

    private void printFicheData(FicheData ficheData) {
        startLi(ficheAttributes);
        escape(ficheData.getTitre());
        endLi();
    }

}
