/* Scrutari - Copyright (c) 2008-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin.htmlpages;

import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.servlets.selection.SelectionManager;
import java.io.IOException;
import java.util.Set;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.text.Libelle;
import net.scrutari.data.MotcleData;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.LexieAccess;
import net.scrutari.db.api.codes.CodeList;
import net.scrutari.lexie.LexieOccurrencesList;
import net.scrutari.lexie.ScrutariLexieUnit;
import net.scrutari.lexie.ScrutariLexieUnitList;


/**
 *
 * @author Vincent Calame
 */
public class LexiesPage extends AdminHtmlPage {

    private int libLangInteger;
    private CodeList thesaurusCodeList;
    private SelectionManager selectionManager;

    public LexiesPage(SctServer sctServer, SelectionManager selectionManager, int libLangInteger) {
        init(sctServer);
        this.libLangInteger = libLangInteger;
        this.selectionManager = selectionManager;
    }

    @Override
    public void handleResponse(HttpServletResponse response) throws IOException {
        DataAccess dataAccess = scrutariSession.getScrutariDB().openDataAccess();
        LexieAccess lexieAccess = scrutariSession.getScrutariDB().openLexieAccess();
        initResponse(response);
        start();
        printTools();
        printMessage();

        if (libLangInteger == 0) {
            printIndexResponse(response);
        } else {
            printLangResponse(response, dataAccess, lexieAccess);
        }

        end();
        lexieAccess.close();
        dataAccess.close();
    }

    private void printIndexResponse(HttpServletResponse response) throws IOException {
        int[] langArray = scrutariSession.getScrutariDB().getLangStats().getAllMotcleLangArray();
        int length = langArray.length;
        Set<String> set = selectionManager.getNameSet();
        for (int i = 0; i < length; i++) {
            String langString = LangInteger.toString(langArray[i]);
            startP();
            startA(HA.href("lexies_" + langString + ".html"));
            append(langString);
            endA();
            append(" (");
            append("ods : ");
            startA(HA.href("lexies_" + langString + ".ods"));
            append("_all");
            endA();
            for (String name : set) {
                printLien(langString, name);
            }
            append(")");
            endP();
        }
    }

    private void printLien(String langString, String name) {
        append(" - ");
        startA(HA.href("lexies_" + langString + "-" + name + ".ods"));
        append(name);
        endA();
    }

    private void printLangResponse(HttpServletResponse response, DataAccess dataAccess, LexieAccess lexieAccess) throws IOException {
        thesaurusCodeList = dataAccess.getThesaurusCodeList();
        ScrutariLexieUnitList list = lexieAccess.getScrutariLexieUnitList(libLangInteger);
        int count = list.getScrutariLexieUnitCount();
        startUl();
        for (int i = 0; i < count; i++) {
            ScrutariLexieUnit scrutariLexieUnit = list.getScrutariLexieUnit(i);
            if (scrutariLexieUnit.hasMotcleAndLexieLink()) {
                printLexieUnit(scrutariLexieUnit, dataAccess, lexieAccess);
            }
        }
        endUl();
    }

    private void printLexieUnit(ScrutariLexieUnit lexieUnit, DataAccess dataAccess, LexieAccess lexieAccess) throws IOException {
        startLi();
        startP();
        escape(lexieUnit.getCanonicalLexie());
        endP();
        startUl();
        long lexieId = lexieUnit.getLexieId();
        int length = thesaurusCodeList.getCodeCount();
        for (int i = 0; i < length; i++) {
            int codethesaurus = thesaurusCodeList.getCode(i);
            LexieOccurrencesList lexieOccurrencesList = lexieAccess.getLexieOccurrencesList(lexieId, codethesaurus);
            int codeCount = lexieOccurrencesList.getLexieOccurrencesCount();
            for (int j = 0; j < codeCount; j++) {
                int codemotcle = lexieOccurrencesList.getLexieOccurrences(j).getCode();
                MotcleData motcleData = dataAccess.getMotcleDataByCode(codemotcle);
                startLi();
                Libelle libelle = motcleData.getLibelleByLang(libLangInteger);
                escape(libelle.getLibelleString());
                endLi();
            }
        }
        endUl();
        endLi();
    }

}
