/* Scrutari - Copyright (c) 2008-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin.htmlpages;

import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.servlets.selection.SelectionManager;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.text.Libelle;
import net.scrutari.data.BaseData;
import net.scrutari.data.MotcleData;
import net.scrutari.data.ThesaurusData;
import net.scrutari.db.ScrutariDB;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.tools.util.LibelleUtils;


/**
 *
 * @author Vincent Calame
 */
public class MotsclesPage extends AdminHtmlPage {

    private HTMLAttributes thesaurusCellAttributes = HA.classes("c_SCT_lst_thesaurus");
    private HTMLAttributes motcleTBodyAttributes = HA.style("display: none;");
    private ScrutariDB scrutariDB;
    private int[] motcleLangArray;
    private SelectionManager selectionManager;

    public MotsclesPage(SctServer sctServer, SelectionManager selectionManager) {
        init(sctServer);
        this.selectionManager = selectionManager;
        scrutariDB = scrutariSession.getScrutariDB();
        addCssUrl("../css/listes.css");
        addJsUrl("../js/affichage.js");
        motcleLangArray = scrutariDB.getLangStats().getAllMotcleLangArray();
    }

    @Override
    public void handleResponse(HttpServletResponse response) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        initResponse(response);
        start();
        printTools();
        printMessage();
        startP();
        append("ods : ");
        startA(HA.href("motscles.ods"));
        append("_all");
        endA();
        for (String name : selectionManager.getNameSet()) {
            printLien(name);
        }
        endP();
        startTable();
        startTHead(null);
        startRow();
        startCell();
        append("&nbsp;");
        endCell();
        startCell();
        append("&nbsp;");
        endCell();

        int length = motcleLangArray.length;
        for (int i = 0; i < length; i++) {
            startCell();
            append(LangInteger.toString(motcleLangArray[i]));
            endCell();
        }
        endRow();
        endTHead();
        thesaurusCellAttributes.colspan(1 + length);
        HTMLAttributes baseCellAttr = HA.colspan(2 + length);
        int baseCount = dataAccess.getBaseCount();
        for (int i = 0; i < baseCount; i++) {
            BaseData baseData = dataAccess.getBaseDataByIndex(i);
            printBaseData(baseData, dataAccess, baseCellAttr);
        }
        endTable();
        end();
        dataAccess.close();
    }

    private void printLien(String name) {
        append(" - ");
        startA(HA.href("motscles-" + name + ".ods"));
        append(name);
        endA();
    }

    private void printBaseData(BaseData baseData, DataAccess dataAccess, HTMLAttributes baseCellAttr) {
        int codebase = baseData.getCodeBase();
        int thesaurusCount = baseData.getThesaurusCount();
        startTBody(null);
        startRow();
        startCell(baseCellAttr);
        escape(LibelleUtils.getLibelleString(baseData, BaseData.INTITULE_LONG, workingLangInteger, "?"));
        append(" ");
        startCode();
        append("[");
        append(String.valueOf(thesaurusCount));
        append("]");
        endCode();
        endCell();
        endRow();
        endTBody();
        for (int i = 0; i < thesaurusCount; i++) {
            ThesaurusData thesaurusData = dataAccess.getThesaurusDataByCode(baseData.getCodeThesaurus(i));
            printThesaurusData(thesaurusData, dataAccess);
        }
    }

    private void printThesaurusData(ThesaurusData thesaurusData, DataAccess dataAccess) {
        int codethesaurus = thesaurusData.getCodeThesaurus();
        int motcleCount = thesaurusData.getMotcleCount();
        startTBody(HA.attr("onclick", "afficheRowGroup('tbody_" + codethesaurus + "');"));
        startRow();
        appendEmptyCell();
        startCell(thesaurusCellAttributes);
        escape(LibelleUtils.getLibelleString(thesaurusData, workingLangInteger, "?"));
        append(" ");
        startCode();
        append("[");
        append(String.valueOf(motcleCount));
        append("]");
        endCode();
        endCell();

        endRow();
        endTBody();

        motcleTBodyAttributes.id("tbody_" + codethesaurus);
        startTBody(motcleTBodyAttributes);
        for (int i = 0; i < motcleCount; i++) {
            printMotcleData(dataAccess.getMotcleDataByCode(thesaurusData.getCodeMotcle(i)));
        }
        endTBody();

    }

    private void printMotcleData(MotcleData motcleData) {
        startRow();
        appendEmptyCell();
        appendEmptyCell();
        int length = motcleLangArray.length;
        for (int i = 0; i < length; i++) {
            startCell();
            Libelle libelle = motcleData.getLibelleByLang(motcleLangArray[i]);
            if (libelle != null) {
                escape(libelle.getLibelleString());
            }
            endCell();
        }
        endRow();
    }

}
