/* Scrutari - Copyright (c) 2008-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin.htmlpages;

import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.servlets.selection.SelectionManager;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.html.HtmlPrinterUtils;
import net.scrutari.data.BaseData;
import net.scrutari.data.CorpusData;
import net.scrutari.data.ThesaurusData;
import net.scrutari.db.ScrutariDB;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.stats.CountStats;
import net.scrutari.db.tools.util.LibelleUtils;


/**
 *
 * @author Vincent Calame
 */
public class StatsPage extends AdminHtmlPage {

    private ScrutariDB scrutariDB;

    public StatsPage(SctServer sctServer, SelectionManager selectionManager) {
        init(sctServer);
        scrutariDB = scrutariSession.getScrutariDB();
        selectionManager.update();
        System.gc();
    }

    @Override
    public void handleResponse(HttpServletResponse response) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        initResponse(response);
        start();
        printTools();
        printMessage();
        CountStats countStats = scrutariDB.getCountStats();
        startH2(null);
        localize("title_stats");
        endH2();
        startTable();
        printRow("lib_stats_corpus", countStats.getCorpusCount());
        printRow("lib_stats_thesaurus", countStats.getThesaurusCount());
        printRow("lib_stats_fiches", countStats.getFicheCount());
        printRow("lib_stats_motscles", countStats.getMotcleCount());
        printRow("lib_stats_lexie", countStats.getLexieCount());
        printRow("lib_stats_indexation", countStats.getIndexationCount());
        endTable();
        startH2(null);
        localize("title_memory");
        endH2();
        Runtime run = Runtime.getRuntime();
        startTable();
        printRow("lib_memory_free", (int) run.freeMemory() / (1024 * 1024));
        printRow("lib_memory_total", (int) run.totalMemory() / (1024 * 1024));
        printRow("lib_memory_max", (int) run.maxMemory() / (1024 * 1024));
        endTable();
        printCorpusURIList(dataAccess);
        printThesaurusURIList(dataAccess);
        end();
        dataAccess.close();
    }

    private void printRow(String key, int count) {
        startRow();
        HtmlPrinterUtils.printLibCells(this, key, false);
        startCell();
        append(String.valueOf(count));
        endCell();
        endRow();
    }

    private void printCorpusURIList(DataAccess dataAccess) {
        startH2(null);
        localize("title_corpus_list");
        endH2();
        int baseCount = dataAccess.getBaseCount();
        startUl();
        for (int i = 0; i < baseCount; i++) {
            BaseData baseData = dataAccess.getBaseDataByIndex(i);
            printCorpusByBase(baseData, dataAccess);
        }
        endUl();
    }

    private void printThesaurusURIList(DataAccess dataAccess) {
        startH2(null);
        localize("title_thesaurus_list");
        endH2();
        int baseCount = dataAccess.getBaseCount();
        startUl();
        for (int i = 0; i < baseCount; i++) {
            BaseData baseData = dataAccess.getBaseDataByIndex(i);
            printThesaurusByBase(baseData, dataAccess);
        }
        endUl();
    }

    private void printCorpusByBase(BaseData baseData, DataAccess dataAccess) {
        int corpusCount = baseData.getCorpusCount();
        startLi();
        startP();
        escape(LibelleUtils.getLibelleString(baseData, BaseData.INTITULE_LONG, workingLangInteger, "?"));
        endP();
        startUl();
        for (int i = 0; i < corpusCount; i++) {
            CorpusData corpusData = dataAccess.getCorpusDataByCode(baseData.getCodeCorpus(i));
            printCorpusData(corpusData);
        }
        endUl();
        endLi();
    }

    private void printThesaurusByBase(BaseData baseData, DataAccess dataAccess) {
        int thesaurusCount = baseData.getThesaurusCount();
        startLi();
        startP();
        escape(LibelleUtils.getLibelleString(baseData, BaseData.INTITULE_LONG, workingLangInteger, "?"));
        endP();
        startUl();
        for (int i = 0; i < thesaurusCount; i++) {
            ThesaurusData thesaurusData = dataAccess.getThesaurusDataByCode(baseData.getCodeThesaurus(i));
            printThesaurusData(thesaurusData);
        }
        endUl();
        endLi();
    }

    private void printCorpusData(CorpusData corpusData) {
        startLi();
        startP();
        escape(LibelleUtils.getLibelleString(corpusData, CorpusData.INTITULECORPUS_FIELDNUMBER, workingLangInteger, "?"));
        append(" ");
        startCode();
        append("[");
        append(corpusData.getCorpusURI().toString());
        append("]");
        endCode();
        endP();
        endLi();
    }

    private void printThesaurusData(ThesaurusData thesaurusData) {
        startLi();
        startP();
        escape(LibelleUtils.getLibelleString(thesaurusData, workingLangInteger, "?"));
        append(" ");
        startCode();
        append("[");
        append(thesaurusData.getThesaurusURI().toString());
        append("]");
        endCode();
        endP();
        endLi();
    }

}
