/* Scrutari - Copyright (c) 2009-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin.htmlpages.utils;

import fr.exemole.sctserver.admin.OptionsManager;
import fr.exemole.sctserver.servlets.admin.htmlpages.AdminHtmlPage;
import fr.exemole.sctserver.servlets.resources.theme.SctCssCatalog;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.text.Libelle;
import net.scrutari.datauri.CorpusURI;
import net.scrutari.db.api.DataAccess;
import net.scrutari.searchengine.api.options.CorpusClasse;
import net.scrutari.searchengine.api.options.GlobalSearchOptions;


/**
 *
 * @author Vincent Calame
 */
public class CorpusClassesPrinter implements SctCssCatalog {

    private CorpusClassesPrinter() {
    }

    public static void printCorpusClasses(AdminHtmlPage page) {
        DataAccess dataAccess = page.getSctServer().getScrutariSession().getScrutariDB().openDataAccess();
        OptionsManager optionsManager = page.getSctServer().getOptionsManager();
        GlobalSearchOptions globalSearchOptions = optionsManager.getGlobalSearchOptions();
        int count = globalSearchOptions.getCorpusClasseCount();
        if (count == 0) {
            page.startP();
            page.startSpan(HA.classes(INFO_SPANCLASS));
            page.localize("info_corpusclasses_none");
            page.endSpan();
            page.endP();
        } else {
            page.startUl();
            for (int i = 0; i < count; i++) {
                CorpusClasse corpusClasse = globalSearchOptions.getCorpusClasse(i);
                printCorpusClasseLi(page, corpusClasse, dataAccess);
            }
            page.endUl();
        }
        dataAccess.close();
    }

    private static void printCorpusClasseLi(AdminHtmlPage page, CorpusClasse corpusClasse, DataAccess dataAccess) {
        HTMLAttributes errorAttributes = HA.classes(ERROR_SPANCLASS);
        String name = corpusClasse.getName();
        if (name == null) {
            name = "_default";
        }
        page.startLi();
        page.startP();
        page.startCode();
        page.append("[");
        page.append(String.valueOf(corpusClasse.getRank()));
        page.append(" / ");
        page.append(name);
        page.append("]");
        page.endCode();
        page.append(" - ");
        int libelleCount = corpusClasse.getLibelleCount();
        for (int i = 0; i < libelleCount; i++) {
            if (i > 0) {
                page.append(" / ");
            }
            Libelle libelle = corpusClasse.getLibelleByIndex(i);
            page.escape(libelle.getLibelleString());
        }
        page.endP();
        int uriCount = corpusClasse.getCorpusURICount();
        page.startUl();
        if (uriCount == 0) {
            page.startLi();
            page.startP();
            page.startEm();
            if (name.equals("_default")) {
                page.localize("info_corpusuri_default");
            } else {
                page.localize("info_corpusuri_none");
            }
            page.endEm();
            page.endP();
            page.endLi();
        } else {
            for (int i = 0; i < uriCount; i++) {
                CorpusURI corpusURI = corpusClasse.getCorpusURI(i);
                int codecorpus = dataAccess.getCode(corpusURI);
                boolean isHere = (codecorpus > 0);
                page.startLi();
                page.startP();
                if (!isHere) {
                    page.startSpan(errorAttributes);
                }
                page.startCode();
                page.append("[");
                page.append(corpusURI.toString());
                page.append("]");
                page.endCode();
                if (!isHere) {
                    page.endSpan();
                }
                page.endP();
                page.endLi();
            }
        }
        page.endUl();
        page.endLi();
    }

}
