/* Scrutari - Copyright (c) 2009-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin.htmlpages.utils;

import fr.exemole.sctserver.admin.ScrutariInfoManager;
import fr.exemole.sctserver.api.collect.ScrutariInfo;
import fr.exemole.sctserver.servlets.admin.htmlpages.AdminHtmlPage;
import fr.exemole.sctserver.servlets.resources.theme.SctCssCatalog;
import fr.exemole.sctserver.tools.collect.CollectUtils;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.primitives.DateInteger;


/**
 *
 * @author Vincent Calame
 */
public class SourcesPrinter implements SctCssCatalog {

    private SourcesPrinter() {
    }

    public static void printSourceInfoList(AdminHtmlPage page) {
        ScrutariInfoManager scrutariInfoManager = page.getSctServer().getScrutariInfoManager();
        int count = scrutariInfoManager.getScrutariInfoCount();
        if (count == 0) {
            page.startP();
            page.startSpan(HA.classes(INFO_SPANCLASS));
            page.localize("info_list_none");
            page.endSpan();
            page.endP();
            return;
        }
        page.startTable();
        for (int i = 0; i < count; i++) {
            ScrutariInfo scrutariInfo = scrutariInfoManager.getScrutariInfo(i);
            page.startRow();
            page.startCell();
            page.append(scrutariInfo.getUrl().toString());
            page.endCell();

            short state = scrutariInfo.getState();
            page.startCell();
            page.startSpan(getSpanAttributes(state));
            page.localize(CollectUtils.stateToString(state));
            page.endSpan();
            page.endCell();
            page.startCell();
            page.append(DateInteger.toISOString(scrutariInfo.getLastUpdate()));
            page.endCell();
            page.endRow();
        }
        page.endTable();
    }

    private static HTMLAttributes getSpanAttributes(short state) {
        switch (state) {
            case ScrutariInfo.OK_STATE:
                return HA.classes(DONEMESSAGE_SPANCLASS);
            case ScrutariInfo.UNKNOWN_STATE:
            case ScrutariInfo.INFO_NOCONNECTION_STATE:
            case ScrutariInfo.INFO_BROKENURL_STATE:
            case ScrutariInfo.INFO_DOWNLOADERROR_STATE:
            case ScrutariInfo.INFO_NOTXML_STATE:
            case ScrutariInfo.INFO_WRONGXML_STATE:
            case ScrutariInfo.DATA_NOCONNECTION_STATE:
            case ScrutariInfo.DATA_BROKENURL_STATE:
            case ScrutariInfo.DATA_DOWNLOADERROR_STATE:
            case ScrutariInfo.DATA_NOTXML_STATE:
                return HA.classes(ERRORMESSAGE_SPANCLASS);
            default:
                throw new SwitchException("state = " + state);
        }
    }

}
