/* Scrutari - Copyright (c) 2008 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.selection;

import net.mapeadores.util.text.StringUtils;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;


/**
 *
 * @author Vincent Calame
 */
public class SelectionManager {

    private File rootDirectory;
    private Map<String, SelectionUnit> selectionUnitMap = new LinkedHashMap<String, SelectionUnit>();

    public SelectionManager(File rootDirectory) {
        this.rootDirectory = rootDirectory;
        update();
    }

    public void clear() {
        selectionUnitMap.clear();
    }

    public void update() {
        clear();
        if ((!rootDirectory.exists()) || (!rootDirectory.isDirectory())) {
            return;
        }
        File[] fichiers = rootDirectory.listFiles();
        for (int i = 0; i < fichiers.length; i++) {
            File f = fichiers[i];
            if (f.isDirectory()) {
                continue;
            }
            String name = f.getName();
            if (!name.endsWith(".txt")) {
                continue;
            }
            String sub = name.substring(0, name.length() - 4);
            if (sub.length() == 0) {
                continue;
            }
            if (!StringUtils.isTechnicalName(sub, true)) {
                continue;
            }
            try {
                InputStreamReader inputStreamReader = new InputStreamReader(new FileInputStream(f), "UTF-8");
                SelectionUnit selectionUnit = SelectionUnit.read(sub, inputStreamReader);
                addSelectionUnit(selectionUnit);
                inputStreamReader.close();
            } catch (IOException ioe) {
            }
        }
    }

    public int getCount() {
        return selectionUnitMap.size();
    }

    public Set<String> getNameSet() {
        return selectionUnitMap.keySet();
    }

    public SelectionUnit getSelectionUnit(String name) {
        return selectionUnitMap.get(name);
    }

    private void addSelectionUnit(SelectionUnit selectionUnit) {
        selectionUnitMap.put(selectionUnit.getName(), selectionUnit);
    }

}
