/* ScrutariDataURI - Copyright (c) 2005-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.datauri;


/**
 *
 * @author  Vincent Calame
 */
public final class BaseURI extends ScrutariDataURI implements Comparable<BaseURI> {

    private String authorityUUID;
    private String baseName;

    public BaseURI(String authorityUUID, String baseName) {
        this.authorityUUID = authorityUUID;
        this.baseName = baseName;
    }

    public String getAuthorityUUID() {
        return authorityUUID;
    }

    public String getBaseName() {
        return baseName;
    }

    @Override
    public int hashCode() {
        return authorityUUID.hashCode() + baseName.hashCode();
    }

    public int compareTo(BaseURI baseURI) {
        int result = this.baseName.compareTo(baseURI.baseName);
        if (result == 0) {
            result = this.authorityUUID.compareTo(baseURI.authorityUUID);
        }
        return result;
    }

    @Override
    public boolean equals(Object o) {
        if (o == this) {
            return true;
        }
        if (!(o instanceof BaseURI)) {
            return false;
        }
        return (compareTo((BaseURI)o) == 0);
    }

    public short getType() {
        return BASEURI_TYPE;
    }

    protected void appendPath(StringBuilder buf) {
        buf.append('/');
        buf.append(authorityUUID);
        buf.append('/');
        buf.append(baseName);
    }

}
