/* ScrutariLib_DB - Copyright (c) 2008-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.db.tools.lexies;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.AttributeHolder;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.localisation.LocalisationLexieFilters;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.collation.CollationUnitPosition;
import net.mapeadores.util.text.lexie.LexieFilter;
import net.mapeadores.util.text.lexie.LexieParser;
import net.mapeadores.util.text.lexie.LexieParserListener;
import net.scrutari.data.FicheData;
import net.scrutari.data.MotcleData;
import net.scrutari.db.api.AttributeKeyCoder;
import net.scrutari.lexie.Alinea;
import net.scrutari.lexie.AlineaInteger;
import net.scrutari.lexie.FicheLexification;
import net.scrutari.lexie.LexieOccurrences;
import net.scrutari.lexie.LexieOccurrencesList;
import net.scrutari.lexie.MotcleLexification;


/**
 *
 * @author Vincent Calame
 */
public class LexieOccurrencesMapBuilder {

    private LexieUnitBuilder lexieUnitBuilder;
    private InternalLexieParserListener listener = new InternalLexieParserListener();
    private int currentSubsetCode;
    private int currentLangInteger;
    private int currentAlineaInteger;
    private int currentCode;
    private int currentWholeLength;
    private List<Long> currentLexieIdList = new ArrayList<Long>();
    private List<CollationUnitPosition> currentCollationUnitPositionList = new ArrayList<CollationUnitPosition>();
    private LexieFilter currentLexieFilter;
    private Map<Long, InternalLexieOccurencesList> lexieOccurencesListMap = new HashMap<Long, InternalLexieOccurencesList>();
    private Map<Long, LexieOccurrencesBuilder> currentLexieOccurrencesBuilderMap = new HashMap<Long, LexieOccurrencesBuilder>();
    private boolean onFiche = false;
    private AttributeKeyCoder attributeKeyCoder;

    public LexieOccurrencesMapBuilder(AttributeKeyCoder attributeKeyCoder, LexieUnitBuilder lexieUnitBuilder, boolean onFiche) {
        this.attributeKeyCoder = attributeKeyCoder;
        this.lexieUnitBuilder = lexieUnitBuilder;
        this.onFiche = onFiche;
    }

    public int getCurrentSubsetCode() {
        return currentSubsetCode;
    }

    public void setCurrentSubsetCode(int currentSubsetCode) {
        this.currentSubsetCode = currentSubsetCode;
    }

    public Map<Long, LexieOccurrencesList> popLexieOccurencesListMap() {
        Map<Long, LexieOccurrencesList> result = new HashMap<Long, LexieOccurrencesList>(lexieOccurencesListMap);
        lexieOccurencesListMap.clear();
        return result;
    }

    public FicheLexification initFicheData(FicheData ficheData) {
        setCurrentCode(ficheData.getCodeFiche());
        setCurrentLangInteger(ficheData.getLangInteger());
        FicheLexification ficheLexification = new FicheLexification();
        initAlinea(AlineaInteger.TITRE, ficheLexification, ficheData.getTitre());
        initAlinea(AlineaInteger.SOUSTITRE, ficheLexification, ficheData.getSoustitre());
        int complementMaxNumber = ficheData.getComplementMaxNumber();
        for (int i = 1; i <= complementMaxNumber; i++) {
            String complement = ficheData.getComplementByNumber(i);
            initAlinea(AlineaInteger.getComplementAlineaInteger(i), ficheLexification, complement);
        }
        AttributeHolder attributeHolder = ficheData.getAttributHolder();
        int attributeCount = attributeHolder.getAttributeCount();
        for (int i = 0; i < attributeCount; i++) {
            Attribute attribute = attributeHolder.getAttributeByIndex(i);
            AttributeKey attributeKey = attribute.getAttributeKey();
            int alineaRootCode = attributeKeyCoder.getAlineaRootCode(attributeKey);
            if (alineaRootCode > 0) {
                int valueCount = attribute.getValueCount();
                for (int j = 0; j < valueCount; j++) {
                    String value = attribute.getValue(j);
                    initAlinea(AlineaInteger.getValueAlineaInteger(alineaRootCode, j), ficheLexification, value);
                }
            }
        }
        flushLexieOccurrencesBuilderMap();
        return ficheLexification;
    }

    public MotcleLexification initMotcleData(MotcleData motcleData) {
        setCurrentCode(motcleData.getCodeMotcle());
        MotcleLexification motcleLexification = new MotcleLexification();
        int libelleCount = motcleData.getLibelleCount();
        for (int i = 0; i < libelleCount; i++) {
            Libelle libelle = motcleData.getLibelleByIndex(i);
            setCurrentLangInteger(libelle.getLangInteger());
            tokenize(libelle.getLibelleString());
            flushLexification(motcleLexification);
        }
        flushLexieOccurrencesBuilderMap();
        return motcleLexification;
    }

    private void initAlinea(int alineaInteger, FicheLexification ficheLexification, String chaine) {
        setCurrentAlineaInteger(alineaInteger);
        tokenize(chaine);
        flushLexification(ficheLexification);
    }

    private void tokenize(String chaine) {
        if (chaine == null) {
            return;
        }
        int length = chaine.length();
        if (length == 0) {
            return;
        }
        setCurrentWholeLength(length);
        LexieParser.parse(chaine, listener);
    }

    private void addLexieOccurrence(long lexieId, CollationUnitPosition collationUnitPosition) {
        LexieOccurrencesBuilder lexieOccurrencesBuilder = currentLexieOccurrencesBuilderMap.get(lexieId);
        if (lexieOccurrencesBuilder == null) {
            lexieOccurrencesBuilder = new LexieOccurrencesBuilder(getCurrentCode(), lexieId);
            currentLexieOccurrencesBuilderMap.put(lexieId, lexieOccurrencesBuilder);
        }
        int lexieIndexInField = currentLexieIdList.size();
        currentLexieIdList.add(lexieId);
        currentCollationUnitPositionList.add(collationUnitPosition);
        lexieOccurrencesBuilder.addOccurrence(lexieIndexInField, collationUnitPosition, getCurrentAlineaInteger(), getCurrentWholeLength());
    }

    private void flushLexieOccurrencesBuilderMap() {
        for (LexieOccurrencesBuilder lexieOccurrencesBuilder : currentLexieOccurrencesBuilderMap.values()) {
            long lexieId = lexieOccurrencesBuilder.lexieId;
            InternalLexieOccurencesList lexieOccurencesArrayBuilder = lexieOccurencesListMap.get(lexieId);
            if (lexieOccurencesArrayBuilder == null) {
                lexieOccurencesArrayBuilder = new InternalLexieOccurencesList();
                lexieOccurencesListMap.put(lexieId, lexieOccurencesArrayBuilder);
            }
            lexieOccurencesArrayBuilder.addLexieOccurrences(lexieOccurrencesBuilder.toLexieOccurences());
        }
        currentLexieOccurrencesBuilderMap.clear();
    }

    private Alinea flushAlinea() {
        int size = currentLexieIdList.size();
        if (size == 0) {
            return null;
        }
        long[] lexieIdArray = new long[size];
        CollationUnitPosition[] collationUnitPositionArray = new CollationUnitPosition[size];
        for (int i = 0; i < size; i++) {
            lexieIdArray[i] = currentLexieIdList.get(i);
            collationUnitPositionArray[i] = currentCollationUnitPositionList.get(i);
        }
        Alinea alinea = new Alinea();
        alinea.setWholeLength(getCurrentWholeLength());
        alinea.setLexieIdArray(lexieIdArray);
        alinea.setCollationUnitPositionArray(collationUnitPositionArray);
        currentLexieIdList.clear();
        currentCollationUnitPositionList.clear();
        return alinea;
    }

    private int getCurrentLangInteger() {
        return currentLangInteger;
    }

    private void setCurrentLangInteger(int currentLangInteger) {
        this.currentLangInteger = currentLangInteger;
        currentLexieFilter = LocalisationLexieFilters.getLexieFilter(currentLangInteger);
    }

    private int getCurrentAlineaInteger() {
        return currentAlineaInteger;
    }

    private void setCurrentAlineaInteger(int currentAlineaInteger) {
        this.currentAlineaInteger = currentAlineaInteger;
    }

    private int getCurrentCode() {
        return currentCode;
    }

    private void setCurrentCode(int currentCode) {
        this.currentCode = currentCode;
    }

    private int getCurrentWholeLength() {
        return currentWholeLength;
    }

    private void setCurrentWholeLength(int currentWholeLength) {
        this.currentWholeLength = currentWholeLength;
    }

    private void flushLexification(FicheLexification ficheLexification) {
        Alinea alinea = flushAlinea();
        if (alinea != null) {
            ficheLexification.putAlinea(getCurrentAlineaInteger(), alinea);
        }
    }

    private void flushLexification(MotcleLexification motcleLexification) {
        Alinea lexification = flushAlinea();
        if (lexification != null) {
            motcleLexification.putAlinea(getCurrentLangInteger(), lexification);
        }
    }


    private static class InternalLexieOccurencesList implements LexieOccurrencesList {

        private List<LexieOccurrences> lexieOccurrencesList = new ArrayList<LexieOccurrences>();

        private InternalLexieOccurencesList() {
        }

        public int getLexieOccurrencesCount() {
            return lexieOccurrencesList.size();
        }

        public LexieOccurrences getLexieOccurrences(int i) {
            return lexieOccurrencesList.get(i);
        }

        private void addLexieOccurrences(LexieOccurrences lexieOccurrences) {
            lexieOccurrencesList.add(lexieOccurrences);
        }

    }


    private static class LexieOccurrencesBuilder {

        private int code;
        private long lexieId;
        private List<OccurrenceByAlinea> occurenceByAlineaList = new ArrayList<OccurrenceByAlinea>();
        private OccurrenceByAlinea lastOccurrenceByAlinea;

        private LexieOccurrencesBuilder(int code, long lexieId) {
            this.code = code;
            this.lexieId = lexieId;
        }

        protected void addOccurrence(int lexieIndexInAlinea, CollationUnitPosition collationUnitPosition, int currentAlineaInteger, int alineaWholeLength) {
            if ((lastOccurrenceByAlinea == null) || (lastOccurrenceByAlinea.alineaInteger != currentAlineaInteger)) {
                lastOccurrenceByAlinea = new OccurrenceByAlinea(currentAlineaInteger, collationUnitPosition, alineaWholeLength);
                occurenceByAlineaList.add(lastOccurrenceByAlinea);
            }
            lastOccurrenceByAlinea.addLexieIndexInLexification(lexieIndexInAlinea);
        }

        public LexieOccurrences toLexieOccurences() {
            LexieOccurrences lexieOccurrences = new LexieOccurrences();
            lexieOccurrences.setCode(code);
            int length = occurenceByAlineaList.size();
            int[] alineaIntegerArray = new int[length];
            CollationUnitPosition[] bestPositionInFieldArray = new CollationUnitPosition[length];
            int[] alineaWholeLengthArray = new int[length];
            int[][] lexieIndexInLexificationArray = new int[length][];
            for (int i = 0; i < length; i++) {
                OccurrenceByAlinea occurrenceByAlinea = occurenceByAlineaList.get(i);
                alineaIntegerArray[i] = occurrenceByAlinea.alineaInteger;
                bestPositionInFieldArray[i] = occurrenceByAlinea.bestPositionInfield;
                alineaWholeLengthArray[i] = occurrenceByAlinea.alineaWholeLength;
                lexieIndexInLexificationArray[i] = occurrenceByAlinea.getLexieIndexInLexificationArray();
            }
            lexieOccurrences.setArrays(alineaIntegerArray, bestPositionInFieldArray, alineaWholeLengthArray, lexieIndexInLexificationArray);
            return lexieOccurrences;
        }

    }


    private static class OccurrenceByAlinea {

        private int alineaInteger;
        private CollationUnitPosition bestPositionInfield;
        private int alineaWholeLength;
        private List<Integer> lexieIndexInLexificationList = new ArrayList<Integer>();

        private OccurrenceByAlinea(int alineaInteger, CollationUnitPosition bestPositionInfield, int alineaWholeLength) {
            this.alineaInteger = alineaInteger;
            this.bestPositionInfield = bestPositionInfield;
            this.alineaWholeLength = alineaWholeLength;
        }

        private void addLexieIndexInLexification(int lexieIndexInLexification) {
            lexieIndexInLexificationList.add(lexieIndexInLexification);
        }

        private int[] getLexieIndexInLexificationArray() {
            int size = lexieIndexInLexificationList.size();
            int[] result = new int[size];
            for (int i = 0; i < size; i++) {
                result[i] = lexieIndexInLexificationList.get(i);
            }
            return result;
        }

    }


    private class InternalLexieParserListener implements LexieParserListener {

        InternalLexieParserListener() {
        }

        public void flushLexie(String lexie, int lexieStartIndex) {
            if (currentLexieFilter.acceptLexie(lexie)) {
                LexieOccurrence lexieOccurence = lexieUnitBuilder.getLexieOccurence(getCurrentLangInteger(), lexie, onFiche);
                CollationUnitPosition collationUnitPosition = new CollationUnitPosition(lexieOccurence.getCollationUnit(), lexieStartIndex);
                addLexieOccurrence(lexieOccurence.getLexieId(), collationUnitPosition);
            }
        }

        public void checkSpecialChar(char carac) {
        }

    }

}
