/* ScrutariLib_DB - Copyright (c) 2007-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.db.tools.util;

import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.LibelleHolder;
import net.scrutari.data.BaseData;
import net.scrutari.data.CorpusData;
import net.scrutari.data.ThesaurusData;


/**
 *
 * @author Vincent Calame
 */
public class LibelleUtils {

    private LibelleUtils() {
    }

    public static String getLibelleString(CorpusData corpusData, int fieldNumber, int langInteger, String defaultString) {
        CorpusData.CorpusLangData corpusLangData = corpusData.getLangDataByLang(langInteger);
        String res = getLibelleString(corpusLangData, fieldNumber);
        if (res != null) {
            return res;
        }
        int count = corpusData.getLangDataCount();
        for (int i = 0; i < count; i++) {
            corpusLangData = corpusData.getLangDataByIndex(i);
            res = getLibelleString(corpusLangData, fieldNumber);
            if (res != null) {
                return res;
            }
        }
        return defaultString;
    }

    private static String getLibelleString(CorpusData.CorpusLangData corpusLangData, int fieldNumber) {
        if (corpusLangData == null) {
            return null;
        }
        Libelle lib = corpusLangData.getLibelle(fieldNumber);
        if (lib == null) {
            return null;
        }
        return lib.getLibelleString();
    }

    public static String getLibelleString(BaseData baseData, short type, int langInteger, String defaultString) {
        BaseData.BaseLangData baseLangData = baseData.getLangDataByLang(langInteger);
        String res = getLibelleString(baseLangData, type);
        if (res != null) {
            return res;
        }
        int count = baseData.getLangDataCount();
        for (int i = 0; i < count; i++) {
            baseLangData = baseData.getLangDataByIndex(i);
            res = getLibelleString(baseLangData, type);
            if (res != null) {
                return res;
            }
        }
        return defaultString;
    }

    private static String getLibelleString(BaseData.BaseLangData baseLangData, short type) {
        if (baseLangData == null) {
            return null;
        }
        Libelle lib = baseLangData.getLibelle(type);
        if (lib == null) {
            return null;
        }
        return lib.getLibelleString();
    }

    public static String getLibelleString(ThesaurusData thesaurusData, int langInteger, String defaultString) {
        ThesaurusData.ThesaurusLangData thesaurusLangData = thesaurusData.getLangDataByLang(langInteger);
        String res = getLibelleString(thesaurusLangData);
        if (res != null) {
            return res;
        }
        int count = thesaurusData.getLangDataCount();
        for (int i = 0; i < count; i++) {
            thesaurusLangData = thesaurusData.getLangDataByIndex(i);
            res = getLibelleString(thesaurusLangData);
            if (res != null) {
                return res;
            }
        }
        return defaultString;
    }

    public static String getLibelleString(LibelleHolder libelleHolder, int langInteger, String defaultString) {
        Libelle libelle = libelleHolder.getLibelleByLang(langInteger);
        if (libelle != null) {
            return libelle.getLibelleString();
        }
        int count = libelleHolder.getLibelleCount();
        if (count > 0) {
            return libelleHolder.getLibelleByIndex(0).getLibelleString();
        }
        return defaultString;
    }

    private static String getLibelleString(ThesaurusData.ThesaurusLangData thesaurusLangData) {
        if (thesaurusLangData == null) {
            return null;
        }
        Libelle lib = thesaurusLangData.getLibelle();
        if (lib == null) {
            return null;
        }
        return lib.getLibelleString();
    }

}
