/* ScrutariLib_Data - Copyright (c) 2005-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.data;

import java.io.IOException;
import java.io.Serializable;
import net.mapeadores.util.primitives.io.PrimitivesReader;
import net.mapeadores.util.primitives.io.PrimitivesWriter;


/**
 *
 * @author Vincent Calame
 */
public final class IndexationData implements Serializable {

    private final int codecorpus;
    private final int codethesaurus;
    private final int codemotcle;
    private final int codefiche;
    private final float ponderation;

    private IndexationData(int codefiche, int codecorpus, int codemotcle, int codethesaurus, float ponderation) {
        this.codecorpus = codecorpus;
        this.codefiche = codefiche;
        this.codethesaurus = codethesaurus;
        this.codemotcle = codemotcle;
        this.ponderation = ponderation;
    }

    public int getCodeMotcle() {
        return codemotcle;
    }

    public int getCodeThesaurus() {
        return codethesaurus;
    }

    public int getCodeFiche() {
        return codefiche;
    }

    public int getCodeCorpus() {
        return codecorpus;
    }

    public float getPonderation() {
        return ponderation;
    }

    public static IndexationData fromPrimitives(PrimitivesReader primitivesReader) throws IOException {
        int codefiche = primitivesReader.readInt();
        int codemotcle = primitivesReader.readInt();
        int codecorpus = primitivesReader.readInt();
        int codethesaurus = primitivesReader.readInt();
        float ponderation = primitivesReader.readFloat();
        return new IndexationData(codefiche, codecorpus, codemotcle, codethesaurus, ponderation);
    }

    public static void toPrimitives(IndexationData indexationData, PrimitivesWriter primitivesWriter) throws IOException {
        primitivesWriter.writeInt(indexationData.codefiche);
        primitivesWriter.writeInt(indexationData.codemotcle);
        primitivesWriter.writeInt(indexationData.codecorpus);
        primitivesWriter.writeInt(indexationData.codethesaurus);
        primitivesWriter.writeFloat(indexationData.ponderation);
    }

    public static IndexationData newInstance(int codefiche, int codecorpus, int codemotcle, int codethesaurus, int poids, int poidsMax) {
        float ponderation = 0;
        if (poids > 1) {
            ponderation = ((float) poids) / ((float) (poidsMax + 1));
        }
        return new IndexationData(codefiche, codecorpus, codemotcle, codethesaurus, ponderation);
    }

}
