/* ScrutariLib_Lexie - Copyright (c) 2008-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.lexie;

import net.mapeadores.util.text.collation.CollationUnitPosition;
import net.mapeadores.util.text.SubstringPosition;
import java.io.IOException;
import net.mapeadores.util.primitives.io.PrimitivesReader;
import net.mapeadores.util.primitives.io.PrimitivesWriter;


/**
 *
 * @author Vincent Calame
 */
public class LexieUtils {

    public final static ScrutariLexieUnitList EMPTY_SCRUTARILEXIEUNITLIST = new EmptyScrutariLexieUnitList();
    public final static LexieOccurrencesList EMPTY_LEXIEOCCURRENCESLIST = new EmptyLexieOccurrencesList();

    private LexieUtils() {
    }

    public static void writeCollationUnitPosition(CollationUnitPosition collationUnitPosition, PrimitivesWriter primitivesWriter) throws IOException {
        SubstringPosition sp = collationUnitPosition.getPositionInWholeSource();
        primitivesWriter.writeInt(sp.getBeginIndex());
        primitivesWriter.writeInt(sp.getLength());
        int[] array = collationUnitPosition.getCorrespondances();
        int length = array.length;
        primitivesWriter.writeInt(length);
        for (int i = 0; i < length; i++) {
            primitivesWriter.writeInt(array[i]);
        }
    }

    public static CollationUnitPosition readCollationUnitPosition(PrimitivesReader primitivesReader) throws IOException {
        int beginIndex = primitivesReader.readInt();
        int length = primitivesReader.readInt();
        SubstringPosition sp = new SubstringPosition(beginIndex, length);
        int al = primitivesReader.readInt();
        int[] correspondances = new int[al];
        for (int i = 0; i < al; i++) {
            correspondances[i] = primitivesReader.readInt();
        }
        return new CollationUnitPosition(sp, correspondances);
    }

    public static ScrutariLexieUnitList toScrutariLexieUnitList(ScrutariLexieUnit[] scrutariLexieUnitArray) {
        if (scrutariLexieUnitArray == null) {
            return EMPTY_SCRUTARILEXIEUNITLIST;
        }
        if (scrutariLexieUnitArray.length == 0) {
            return EMPTY_SCRUTARILEXIEUNITLIST;
        }
        return new ArrayScrutariLexieUnitList(scrutariLexieUnitArray);
    }

    public static ScrutariLexieUnitList toScrutariLexieUnitList(ScrutariLexieUnit scrutariLexieUnit) {
        if (scrutariLexieUnit == null) {
            return EMPTY_SCRUTARILEXIEUNITLIST;
        }
        return new SingletonScrutariLexieUnitList(scrutariLexieUnit);
    }

    public static LexieOccurrencesList toLexieOccurrencesList(LexieOccurrences[] lexieOccurrencesArray) {
        if (lexieOccurrencesArray == null) {
            return EMPTY_LEXIEOCCURRENCESLIST;
        }
        if (lexieOccurrencesArray.length == 0) {
            return EMPTY_LEXIEOCCURRENCESLIST;
        }
        return new ArrayLexieOccurrencesList(lexieOccurrencesArray);
    }

    public static LexieOccurrencesList toLexieOccurrencesList(LexieOccurrences lexieOccurrences) {
        if (lexieOccurrences == null) {
            return EMPTY_LEXIEOCCURRENCESLIST;
        }
        return new SingletonLexieOccurrencesList(lexieOccurrences);
    }


    private final static class EmptyScrutariLexieUnitList implements ScrutariLexieUnitList {

        public int getScrutariLexieUnitCount() {
            return 0;
        }

        public ScrutariLexieUnit getScrutariLexieUnit(int i) {
            throw new IndexOutOfBoundsException();
        }

    }


    private final static class SingletonScrutariLexieUnitList implements ScrutariLexieUnitList {

        private ScrutariLexieUnit scrutariLexieUnit;

        private SingletonScrutariLexieUnitList(ScrutariLexieUnit scrutariLexieUnit) {
            this.scrutariLexieUnit = scrutariLexieUnit;
        }

        public int getScrutariLexieUnitCount() {
            return 1;
        }

        public ScrutariLexieUnit getScrutariLexieUnit(int i) {
            if (i != 0) {
                throw new IndexOutOfBoundsException();
            }
            return scrutariLexieUnit;
        }

    }


    private final static class ArrayScrutariLexieUnitList implements ScrutariLexieUnitList {

        private ScrutariLexieUnit[] scrutariLexieUnitArray;

        private ArrayScrutariLexieUnitList(ScrutariLexieUnit[] scrutariLexieUnitArray) {
            this.scrutariLexieUnitArray = scrutariLexieUnitArray;
        }

        public int getScrutariLexieUnitCount() {
            return scrutariLexieUnitArray.length;
        }

        public ScrutariLexieUnit getScrutariLexieUnit(int i) {
            return scrutariLexieUnitArray[i];
        }

    }


    private final static class EmptyLexieOccurrencesList implements LexieOccurrencesList {

        public int getLexieOccurrencesCount() {
            return 0;
        }

        public LexieOccurrences getLexieOccurrences(int i) {
            throw new IndexOutOfBoundsException();
        }

    }


    private final static class SingletonLexieOccurrencesList implements LexieOccurrencesList {

        private LexieOccurrences lexieOccurrences;

        private SingletonLexieOccurrencesList(LexieOccurrences lexieOccurrences) {
            this.lexieOccurrences = lexieOccurrences;
        }

        public int getLexieOccurrencesCount() {
            return 1;
        }

        public LexieOccurrences getLexieOccurrences(int i) {
            if (i != 0) {
                throw new IndexOutOfBoundsException();
            }
            return lexieOccurrences;
        }

    }


    private final static class ArrayLexieOccurrencesList implements LexieOccurrencesList {

        private LexieOccurrences[] lexieOccurrencesArray;

        private ArrayLexieOccurrencesList(LexieOccurrences[] lexieOccurrencesArray) {
            this.lexieOccurrencesArray = lexieOccurrencesArray;
        }

        public int getLexieOccurrencesCount() {
            return lexieOccurrencesArray.length;
        }

        public LexieOccurrences getLexieOccurrences(int i) {
            return lexieOccurrencesArray[i];
        }

    }

}
