/* ScrutariLib_Lexie - Copyright (c) 2005-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.lexie;

import net.mapeadores.util.text.lexie.LexieUnit;
import net.mapeadores.util.primitives.PrimUtils;
import java.io.IOException;
import net.mapeadores.util.primitives.io.PrimitivesReader;
import net.mapeadores.util.primitives.io.PrimitivesWriter;


/**
 * Point particulier des unités lexicales : elles correspondent à une langue donnée. 
 * 
 * @author Vincent Calame
 */
public class ScrutariLexieUnit implements LexieUnit {

    private long lexieId;
    private String collatedLexie;
    private String canonicalLexie;
    private boolean hasFicheLink = false;
    private boolean hasMotcleLink = false;

    public ScrutariLexieUnit(long lexieId, String collatedLexie) {
        this.lexieId = lexieId;
        this.collatedLexie = collatedLexie;
    }

    public int getLangInteger() {
        return PrimUtils.getInt1(lexieId);
    }

    public String getCollatedLexie() {
        return collatedLexie;

    }

    public long getLexieId() {
        return lexieId;
    }

    public String getCanonicalLexie() {
        return canonicalLexie;
    }

    public void setCanonicalLexie(String canonicalLexie) {
        this.canonicalLexie = canonicalLexie;
    }

    public boolean hasMotcleAndLexieLink() {
        return hasMotcleLink;
    }

    public boolean hasFicheAndLexieLink() {
        return hasFicheLink;
    }

    public void setHasFicheLink(boolean hasFicheLink) {
        this.hasFicheLink = hasFicheLink;
    }

    public void setHasMotcleLink(boolean hasMotcleLink) {
        this.hasMotcleLink = hasMotcleLink;
    }

    public static ScrutariLexieUnit fromPrimitives(PrimitivesReader primitivesReader) throws IOException {
        long lexieId = primitivesReader.readLong();
        String collatedLexie = primitivesReader.readString();
        ScrutariLexieUnit scrutariLexieUnit = new ScrutariLexieUnit(lexieId, collatedLexie);
        scrutariLexieUnit.setCanonicalLexie(primitivesReader.readString());
        scrutariLexieUnit.setHasFicheLink(primitivesReader.readBoolean());
        scrutariLexieUnit.setHasMotcleLink(primitivesReader.readBoolean());
        return scrutariLexieUnit;
    }

    public static void toPrimitives(ScrutariLexieUnit scrutariLexieUnit, PrimitivesWriter primitivesWriter) throws IOException {
        primitivesWriter.writeLong(scrutariLexieUnit.lexieId);
        primitivesWriter.writeString(scrutariLexieUnit.getCollatedLexie());
        primitivesWriter.writeString(scrutariLexieUnit.getCanonicalLexie());
        primitivesWriter.writeBoolean(scrutariLexieUnit.hasFicheLink);
        primitivesWriter.writeBoolean(scrutariLexieUnit.hasMotcleLink);
    }

}
