/* ScrutariLib_SearchEngine - Copyright (c) 2010-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine;

import java.util.ArrayList;
import java.util.List;
import net.scrutari.db.ScrutariDB;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.LexieAccess;
import net.scrutari.searchengine.api.operands.search.SearchOperation;
import net.scrutari.searchengine.api.options.SearchOptions;
import net.scrutari.searchengine.api.pertinence.MotclePertinence;
import net.scrutari.searchengine.api.result.InLangOccurrenceList;
import net.scrutari.searchengine.api.result.MotcleSearchResult;
import net.scrutari.searchengine.api.result.MotcleSearchResultInfo;
import net.scrutari.searchengine.api.seeker.SeekOptions;
import net.scrutari.searchengine.tools.operands.search.SearchOperationUtils;
import net.scrutari.searchengine.tools.pertinence.PertinenceUtils;
import net.scrutari.searchengine.tools.result.MotcleSearchResultBuilder;
import net.scrutari.searchengine.tools.seeker.SeekOptionsBuilder;
import net.scrutari.searchengine.tools.seeker.lexie.LexieSeeker;


/**
 *
 * @author Vincent Calame
 */
public final class MotcleSearchEngine {

    private final SeekOptions seekOptions;
    private final SearchEngine searchEngine;
    private final int[] acceptOperandArray;

    private MotcleSearchEngine(ScrutariDB scrutariDB, SearchOperation searchOperation, SearchOptions searchOptions) {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        this.acceptOperandArray = SearchOperationUtils.toAcceptOperandArray(searchOperation);
        this.seekOptions = SeekOptionsBuilder.build(scrutariDB, dataAccess, searchOptions, false, true);
        this.searchEngine = new SearchEngine(searchOperation.getSearchTokenOperandMaxNumber(), acceptOperandArray);
        LexieAccess lexieAccess = scrutariDB.openLexieAccess();
        LexieSeeker lexieSeeker = new LexieSeeker(seekOptions, searchEngine, lexieAccess);
        lexieSeeker.seek(searchOperation);
        lexieAccess.close();
        dataAccess.close();
    }

    private MotcleSearchResult getResult() {
        List<MotclePertinence> motclePertinenceList = searchEngine.getMotclePertinenceList();
        List<MotcleSearchResultInfo> resultList = new ArrayList<MotcleSearchResultInfo>();
        for (MotclePertinence motclePertinence : motclePertinenceList) {
            InLangOccurrenceList[] inLangOccurrenceListArray = PertinenceUtils.getInLangOccurrenceListArray(motclePertinence, acceptOperandArray);
            resultList.add(MotcleSearchResultBuilder.toMotcleSearchResultInfo(motclePertinence.getCodemotcle(), inLangOccurrenceListArray));
        }
        return new InternalMotcleSearchResult(resultList.toArray(new MotcleSearchResultInfo[resultList.size()]));
    }

    public static MotcleSearchResult search(ScrutariDB scrutariDB, SearchOperation searchOperation, SearchOptions searchOptions) {
        MotcleSearchEngine searchEngine = new MotcleSearchEngine(scrutariDB, searchOperation, searchOptions);
        return searchEngine.getResult();
    }


    private class InternalMotcleSearchResult implements MotcleSearchResult {

        private final MotcleSearchResultInfo[] motcleArray;

        private InternalMotcleSearchResult(MotcleSearchResultInfo[] motcleArray) {
            this.motcleArray = motcleArray;
        }

        @Override
        public int getMotcleCount() {
            return motcleArray.length;
        }

        @Override
        public MotcleSearchResultInfo getMotcleResult(int i) {
            return motcleArray[i];
        }

    }

}
