/* ScrutariLib_SearchEngine - Copyright (c) 2010-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.api.options;

import net.mapeadores.util.logicaloperation.Operand;
import net.scrutari.db.api.codes.CodeList;
import net.scrutari.db.api.codes.ElementCodeList;


/**
 *
 * @author Vincent Calame
 */
public interface SearchOptions {

    public int getLangUi();

    public int[] getLangIntegerArray();

    /**
     * Différentes listes servant de filtres (seuls les types corpus, thesaurus et fiche) sont traités.
     * Cette liste décrit les corpus, fiches et thésaurus indiqués explicitement dans les paramètres de recherche.
     * getScrutariDataURIList(ScrutariDataURI.CORPUSURI_TYPE) peut ainsi contenir plus de corpus que getCorpusCodeList()
     * qui est le résultat final de la prise en compte des différentes paramètres de recherche.
     */
    public ScrutariDataURIList getScrutariDataURIList(short type);

    /**
     * Cas particulier où la recherche doit porter sur un ensemble de fiches précis
     * getCorpusCodeList() est censé être cohérent avec getFicheCodeList (c'est à dire
     * ne contenir que les corpus qui ont des fiches dans getFicheCodeList)
     * Null si non pertinent
     */
    public ElementCodeList getFicheCodeList();

    /**
     * Cette liste est une construction à partir des différentes options de la recherche.
     * Elle prend notamment en compte le contenu de getScrutariDataURIList(ScrutariDataURI.CORPUSURI_TYPE)
     * mais pas seulement : elle peut être réduite du fait du contenu getFicheEligibilityOperand() ou d'autres élements.
     * Peut être nul.
     */
    public CodeList getCorpusCodeList();

    /**
     * Cette liste est une construction à partir des différentes options de la recherche.
     * Elle prend notamment en compte le contenu de getScrutariDataURIList(ScrutariDataURI.THESAURUSURI_TYPE)
     * mais pas seulement. Peut être nul.
     */
    public CodeList getThesaurusCodeList();

    public Operand getFicheEligibilityOperand();

    public Operand getMotcleEligibilityOperand();

    public int getDefaultSearchType();

    /**
     * Peut être nul.
     */
    public PertinencePonderation getPertinencePonderation();

}
