/* ScrutariLib_SearchEngine - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools;

import java.util.Collections;
import java.util.Set;
import net.mapeadores.util.logicaloperation.Operand;
import net.scrutari.searchengine.api.operands.filters.CodeOperand;


/**
 *
 * @author Vincent Calame
 */
public class OperandCheck {

    public final static Set<Integer> EMPTY_SET = Collections.emptySet();
    public final static OperandCheck UNDEFINED_OPERANDECHECK = new OperandCheck(null, null);
    public final static OperandCheck NONE_OPERANDECHECK = new OperandCheck(EMPTY_SET, new BooleanCodeOperand(false));
    public final static OperandCheck ALL_OPERANDECHECK = new OperandCheck(null, new BooleanCodeOperand(true));
    private Set<Integer> codeSet;
    private Operand operand;

    public OperandCheck(Set<Integer> codeSet, Operand operand) {
        this.codeSet = codeSet;
        this.operand = operand;
    }

    public Set<Integer> getCodeSet() {
        return codeSet;
    }

    public Operand getOperand() {
        return operand;
    }


    private static class BooleanCodeOperand implements CodeOperand {

        private boolean acceptMode;

        private BooleanCodeOperand(boolean acceptMode) {
            this.acceptMode = acceptMode;
        }

        @Override
        public boolean acceptCode(int code) {
            return acceptMode;
        }

        @Override
        public String toString() {
            if (acceptMode) {
                return "TRUE";
            } else {
                return "FALSE";
            }
        }

    }

}
