/* ScrutariLib_SearchEngine - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.operands.search;

import net.scrutari.lexie.AlineaInteger;
import net.scrutari.searchengine.api.operands.search.AlineaFilter;

/**
 *
 * @author Vincent Calame
 */
public class AlineaFilters {
    
    public final static AlineaFilter ALL_FILTER = new AllAlineaFilter();
    public final static AlineaFilter TITRE_FILTER = new SpecialAlineaFilter(AlineaInteger.TITRE, "titre");
    public final static AlineaFilter SOUSTITRE_FILTER = new SpecialAlineaFilter(AlineaInteger.SOUSTITRE, "soustitre");
    public final static AlineaFilter MOTCLE_FILTER = new MotcleAlineaFilter("motcle");
    public final static AlineaFilter SUPPLEMENT_FILTER = new SupplementAlineaFilter("supplement");

    
    private AlineaFilters() {
        
    }
    
    public final static AlineaFilter getSpecialFilter(String scope) {
        if (scope.equals("titre")){
            return TITRE_FILTER;
        }
        if (scope.equals("soustitre"))  {
            return SOUSTITRE_FILTER;
        }
        if (scope.equals("motcle")) {
            return MOTCLE_FILTER;
        }
        if (scope.equals("supplement")) {
            return SUPPLEMENT_FILTER;
        }
        return null;
    }
    
    public final static AlineaFilter getAlineaRootCodeFilter(String prefix, int alineaRootCode) {
        return new RootCodeAlineaFilter(prefix, alineaRootCode);
    }
    
    private static class AllAlineaFilter implements AlineaFilter {
        
        private AllAlineaFilter() {
            
        }
        
        public boolean acceptAlinea(int codecorpus, int alineaInteger) {
            return true;
        }

        public boolean acceptMotcle() {
            return true;
        }

        public String getFilterPrefix() {
            return "";
        }
    }
    
    private static class MotcleAlineaFilter implements AlineaFilter {
        
        private String prefix;
        
        private MotcleAlineaFilter(String prefix) {
            this.prefix = prefix;
        }
        
        public boolean acceptAlinea(int codecorpus, int alineaInteger) {
            return false;
        }

        public boolean acceptMotcle() {
            return true;
        }

        public String getFilterPrefix() {
            return prefix;
        }
    }
    
    private static class SpecialAlineaFilter implements AlineaFilter {
        
        private int specialAlineaInteger;
        private String prefix;
        
        private SpecialAlineaFilter(int specialAlineaInteger, String prefix) {
            this.specialAlineaInteger = specialAlineaInteger;
            this.prefix = prefix;
        }
        
        public boolean acceptAlinea(int codecorpus, int alineaInteger) {
            return (alineaInteger == specialAlineaInteger);
        }

        public boolean acceptMotcle() {
            return false;
        }

        public String getFilterPrefix() {
            return prefix;
        }
    }
    
    private static class SupplementAlineaFilter implements AlineaFilter {
        
        private String prefix;
        
        private SupplementAlineaFilter(String prefix) {
            this.prefix = prefix;
        }
        
        public boolean acceptAlinea(int codecorpus, int alineaInteger) {
            short category = AlineaInteger.getCategory(alineaInteger);
            return (category != AlineaInteger.SPECIAL_CATEGORY);
        }

        public boolean acceptMotcle() {
            return false;
        }

        public String getFilterPrefix() {
            return prefix;
        }
    }
    
    private static class RootCodeAlineaFilter implements AlineaFilter {
        
        private String prefix;
        private int filterAlineaRootCode;
        
        private RootCodeAlineaFilter(String prefix, int filterAlineaRootCode) {
            this.prefix = prefix;
            this.filterAlineaRootCode = filterAlineaRootCode;
        }
        
        public boolean acceptAlinea(int codecorpus, int alineaInteger) {
            int alineaRootCode = AlineaInteger.toAlineaRootCode(alineaInteger);
            return (alineaRootCode == filterAlineaRootCode);
        }

        public boolean acceptMotcle() {
            return false;
        }

        public String getFilterPrefix() {
            return prefix;
        }
    }
}
