/* ScrutariLib_SearchEngine - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.operands.search;

import net.mapeadores.util.logging.ProcessMessageHandler;
import net.mapeadores.util.logicaloperation.LogicalOperationParser;
import net.mapeadores.util.logicaloperation.LogicalOperationUtils;
import net.mapeadores.util.logicaloperation.Operand;
import net.mapeadores.util.logicaloperation.OperandException;
import net.mapeadores.util.text.SearchSequence;
import net.scrutari.db.api.AttributeKeyCoder;
import net.scrutari.searchengine.api.operands.search.SearchOperation;
import net.scrutari.searchengine.api.operands.search.QInfo;
import net.scrutari.searchengine.api.operands.search.SearchTokenOperand;


/**
 *
 * @author Vincent Calame
 */
public class SearchOperationBuilder {

    private SearchOperationBuilder() {
    }

    public static SearchOperation fromSimpleOperandOperation(Operand operand, String sourceString, ProcessMessageHandler processMessageHandler, AttributeKeyCoder attributeKeyCoder) {
        SearchOperandConverterEngine.SearchOperandCheck operandCheck = SearchOperandConverterEngine.convert(operand, processMessageHandler, attributeKeyCoder);
        if (operandCheck == null) {
            return null;
        }
        int operandMaxNumber = operandCheck.getOperandMaxNumber();
        Operand newOperand = operandCheck.getOperand();
        QInfo qInfo = SearchOperationUtils.toQInfo(LogicalOperationUtils.toInformatiqueString(newOperand), sourceString, operandMaxNumber);
        return new InternalSearchOperation(newOperand, operandMaxNumber, qInfo);
    }

    public static SearchOperation fromSearchSequence(SearchSequence searchSequence, short operator) {
        int tokenCount = searchSequence.getTokenCount();
        if (tokenCount == 0) {
            return null;
        }
        Operand rootOperand;
        if (tokenCount == 1) {
            rootOperand = SearchOperationUtils.toSearchTokenOperand(1, true, searchSequence.getToken(0), AlineaFilters.ALL_FILTER);
        } else {
            SearchTokenOperand[] array = new SearchTokenOperand[tokenCount];
            for (int i = 0; i < tokenCount; i++) {
                array[i] = SearchOperationUtils.toSearchTokenOperand(i + 1, true, searchSequence.getToken(i), AlineaFilters.ALL_FILTER);
            }
            rootOperand = LogicalOperationUtils.toSubOperand(array, operator);
        }
        QInfo qInfo = SearchOperationUtils.toQInfo(LogicalOperationUtils.toInformatiqueString(rootOperand), searchSequence.getSource(), tokenCount);
        return new InternalSearchOperation(rootOperand, tokenCount, qInfo);
    }

    public static SearchOperation fromQInfo(QInfo qInfo, ProcessMessageHandler processMessageHandler, AttributeKeyCoder attributeKeyCoder) {
        Operand operand;
        try {
            operand = LogicalOperationParser.parseInformatique(qInfo.getQ());
            if (operand == null) {
                return null;
            }
        } catch (OperandException pe) {
            return null;
        }
        return fromSimpleOperandOperation(operand, qInfo.getQSource(), processMessageHandler, attributeKeyCoder);
    }


    private static class InternalSearchOperation implements SearchOperation {

        private final Operand rootOperand;
        private final int searchTokenOperandMaxNumber;
        private final QInfo qInfo;

        private InternalSearchOperation(Operand rootOperand, int searchTokenOperandMaxNumber, QInfo qInfo) {
            this.rootOperand = rootOperand;
            this.searchTokenOperandMaxNumber = searchTokenOperandMaxNumber;
            this.qInfo = qInfo;
        }

        @Override
        public Operand getRootOperand() {
            return rootOperand;
        }

        @Override
        public int getSearchTokenOperandMaxNumber() {
            return searchTokenOperandMaxNumber;
        }

        @Override
        public QInfo getQInfo() {
            return qInfo;
        }

    }

}
