/* ScrutariLib_SearchEngine - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.options;

import net.scrutari.searchengine.api.options.PertinencePonderation;


/**
 *
 * @author Vincent Calame
 */
public final class PertinencePonderationBuilder {

    public final static PertinencePonderation DEFAULT_PONDERATION = new InternalPertinencePonderation(0.6f, 0.25f, 0.15f);

    private PertinencePonderationBuilder() {
    }

    public static PertinencePonderation toPertinencePonderation(int occurrence, int date, int origin) {
        if (date < 0) {
            date = 0;
        }
        if (origin < 0) {
            origin = 0;
        }
        if (occurrence < 0) {
            occurrence = 0;
        }
        float somme = date + origin + occurrence;
        if (somme == 0) {
            return DEFAULT_PONDERATION;
        }
        float occurrenceF = ((float) occurrence) / somme;
        float dateF = ((float) date) / somme;
        float originF = ((float) origin) / somme;
        return new InternalPertinencePonderation(occurrenceF, dateF, originF);
    }

    public static PertinencePonderation toPertinencePonderation(float occurrence, float date, float origin) {
        if (date < 0) {
            date = 0;
        }
        if (origin < 0) {
            origin = 0;
        }
        if (occurrence < 0) {
            occurrence = 0;
        }
        float somme = date + origin + occurrence;
        if (somme == 0) {
            return DEFAULT_PONDERATION;
        }
        if (somme != 1) {
            date = date / somme;
            origin = origin / somme;
            occurrence = occurrence / somme;
        }
        return new InternalPertinencePonderation(occurrence, date, origin);
    }


    private static class InternalPertinencePonderation implements PertinencePonderation {

        private final float origin;
        private final float date;
        private final float occurrence;

        private InternalPertinencePonderation(float occurrence, float date, float origin) {
            this.occurrence = occurrence;
            this.date = date;
            this.origin = origin;
        }

        @Override
        public float getOriginPonderation() {
            return origin;
        }

        @Override
        public float getDatePonderation() {
            return date;
        }

        @Override
        public float getOccurrencePonderation() {
            return occurrence;
        }

    }

}
