/* ScrutariLib_SearchEngine - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.result;

import net.scrutari.searchengine.QId;
import net.scrutari.searchengine.api.operands.search.QInfo;
import net.scrutari.searchengine.api.options.SearchOptions;
import net.scrutari.searchengine.api.result.FicheSearchSource;


/**
 *
 * @author Vincent Calame
 */
public class FicheSearchSourceBuilder {

    private QId qId;
    private QInfo qInfo;
    private SearchOptions searchOptions;

    public FicheSearchSourceBuilder() {
    }

    public void setQId(QId qId) {
        this.qId = qId;
    }

    public void setQInfo(QInfo qInfo) {
        this.qInfo = qInfo;
    }

    public void setSearchOptions(SearchOptions searchOptions) {
        this.searchOptions = searchOptions;
    }

    public boolean isValid() {
        return (qInfo != null);
    }

    public FicheSearchSource toFicheSearchSource() {
        if (qInfo == null) {
            throw new IllegalStateException("qInfo is not defined");
        }
        return new InternalFicheSearchSource(qId, qInfo, searchOptions);
    }

    public static FicheSearchSource build(QId qId, QInfo qInfo, SearchOptions searchOptions) {
        if (qInfo == null) {
            throw new IllegalArgumentException("qInfo is null");
        }
        return new InternalFicheSearchSource(qId, qInfo, searchOptions);
    }


    private static class InternalFicheSearchSource implements FicheSearchSource {

        private final QId qId;
        private final QInfo qInfo;
        private final SearchOptions searchOptions;

        private InternalFicheSearchSource(QId qId, QInfo qInfo, SearchOptions searchOptions) {
            this.qId = qId;
            this.qInfo = qInfo;
            this.searchOptions = searchOptions;
        }

        @Override
        public QId getQId() {
            return qId;
        }

        @Override
        public QInfo getQInfo() {
            return qInfo;
        }

        @Override
        public SearchOptions getSearchOptions() {
            return searchOptions;
        }

    }

}
