/* ScrutariLib_SearchEngine - Copyright (c) 2010-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.result;

import net.scrutari.searchengine.api.result.InAlineaOccurrenceList;
import net.scrutari.searchengine.api.result.InLangOccurrenceList;
import net.scrutari.searchengine.api.result.SearchTokenOccurrence;
import net.scrutari.searchengine.api.result.SearchTokenOccurrenceList;
import net.scrutari.searchengine.api.result.SupermotcleSearchResult;
import net.scrutari.searchengine.api.result.SupermotcleSearchResultInfo;


/**
 *
 * @author Vincent Calame
 */
public class ResultUtils {

    public final static SupermotcleSearchResult EMPTY_SUPERMOTCLE_SEARCHRESULT = new EmptySupermotcleSearchResult();

    private ResultUtils() {
    }

    public static SearchTokenOccurrenceList toSearchTokenPositionList(SearchTokenOccurrence[] array) {
        return new ArraySearchTokenOccurrenceList(array);
    }

    public static InAlineaOccurrenceList toInAlineaOccurrenceList(int alineaInteger, SearchTokenOccurrence[] array) {
        return new ArrayInAlineaOccurrenceList(alineaInteger, array);
    }

    public static InLangOccurrenceList toInLangOccurrenceList(int langInteger, SearchTokenOccurrence[] array) {
        return new ArrayInLangOccurrenceList(langInteger, array);
    }


    private static class EmptySupermotcleSearchResult implements SupermotcleSearchResult {

        public int getSupermotcleSearchResultInfoCount() {
            return 0;
        }

        public SupermotcleSearchResultInfo getSupermotcleSearchResultInfo(int i) {
            throw new IndexOutOfBoundsException();
        }

    }


    private static class ArraySearchTokenOccurrenceList implements SearchTokenOccurrenceList {

        private SearchTokenOccurrence[] array;

        private ArraySearchTokenOccurrenceList(SearchTokenOccurrence[] array) {
            this.array = array;
        }

        public int getSearchTokenOccurrenceCount() {
            return array.length;
        }

        public SearchTokenOccurrence getSearchTokenOccurrence(int i) {
            return array[i];
        }

    }


    private static class ArrayInAlineaOccurrenceList implements InAlineaOccurrenceList {

        private int alineaInteger;
        private SearchTokenOccurrence[] array;

        private ArrayInAlineaOccurrenceList(int alineaInteger, SearchTokenOccurrence[] array) {
            this.alineaInteger = alineaInteger;
            this.array = array;
        }

        public int getAlineaInteger() {
            return alineaInteger;
        }

        public int getSearchTokenOccurrenceCount() {
            return array.length;
        }

        public SearchTokenOccurrence getSearchTokenOccurrence(int i) {
            return array[i];
        }

    }


    private static class ArrayInLangOccurrenceList implements InLangOccurrenceList {

        private int langInteger;
        private SearchTokenOccurrence[] array;

        private ArrayInLangOccurrenceList(int langInteger, SearchTokenOccurrence[] array) {
            this.langInteger = langInteger;
            this.array = array;
        }

        public int getLangInteger() {
            return langInteger;
        }

        public int getSearchTokenOccurrenceCount() {
            return array.length;
        }

        public SearchTokenOccurrence getSearchTokenOccurrence(int i) {
            return array[i];
        }

    }

}
