/* ScrutariLib_SearchEngine - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.result;

import net.mapeadores.util.text.SubstringPosition;
import net.scrutari.searchengine.api.result.InAlineaOccurrenceList;
import net.scrutari.searchengine.api.result.InLangOccurrenceList;
import net.scrutari.searchengine.api.result.SearchTokenOccurrence;
import net.scrutari.searchengine.api.result.SearchTokenOccurrenceList;


/**
 *
 * @author Vincent Calame
 */
public class SearchTokenOccurrenceListBuilder {

    private SubstringPosition[] positionArray;
    private int[] operandNumberArray;
    private int size = 0;

    public SearchTokenOccurrenceListBuilder(int operandMaxNumber) {
        positionArray = new SubstringPosition[operandMaxNumber];
        operandNumberArray = new int[operandMaxNumber];
    }

    public void add(int operandNumber, SubstringPosition substringPosition) {
        boolean insert = false;
        for (int i = 0; i < size; i++) {
            if (substringPosition.compareTo(positionArray[i]) < 0) {
                System.arraycopy(positionArray, i, positionArray, i + 1, size - i);
                System.arraycopy(operandNumberArray, i, operandNumberArray, i + 1, size - i);
                operandNumberArray[i] = operandNumber;
                positionArray[i] = substringPosition;
                insert = true;
                break;
            }
        }
        if (!insert) {
            operandNumberArray[size] = operandNumber;
            positionArray[size] = substringPosition;
        }
        size++;
    }

    public SearchTokenOccurrenceList toSearchTokenOccurrenceList() {
        return ResultUtils.toSearchTokenPositionList(toSearchTokenOccurrenceArray());
    }

    public InAlineaOccurrenceList toInAlineaOccurrenceList(int alineaInteger) {
        return ResultUtils.toInAlineaOccurrenceList(alineaInteger, toSearchTokenOccurrenceArray());
    }

    public InLangOccurrenceList toInLangOccurrenceList(int langInteger) {
        return ResultUtils.toInLangOccurrenceList(langInteger, toSearchTokenOccurrenceArray());
    }

    private SearchTokenOccurrence[] toSearchTokenOccurrenceArray() {
        SearchTokenOccurrence[] searchTokenPositionArray = new SearchTokenOccurrence[size];
        for (int i = 0; i < size; i++) {
            searchTokenPositionArray[i] = SearchTokenOccurrenceBuilder.toSearchTokenOccurrence(operandNumberArray[i], positionArray[i].getBeginIndex(), positionArray[i].getLength());
        }
        return searchTokenPositionArray;
    }

}
