/* ScrutariLib_SearchEngine - Copyright (c) 2010-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.seeker;

import java.util.HashSet;
import java.util.Set;
import net.mapeadores.util.text.TextConstants;
import net.scrutari.db.ScrutariDB;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.codes.CodeList;
import net.scrutari.searchengine.api.options.SearchOptions;
import net.scrutari.searchengine.api.seeker.SeekOptions;
import net.scrutari.searchengine.tools.FilterBuffer;


/**
 *
 * @author Vincent Calame
 */
public class SeekOptionsBuilder {
    
    private SeekOptionsBuilder() {
    }
    
    public static SeekOptions build(ScrutariDB scrutariDB, DataAccess dataAccess, SearchOptions searchOptions, boolean withFiches, boolean withMotscles) {
        int defaultSearchType = TextConstants.STARTSWITH;
        boolean touteslangues = false;
        CodeList corpusCodeList = null;
        CodeList thesaurusCodeList = null;
        FilterBuffer filterBuffer = null;
        Set<Integer> codeficheSet = null;
        int[] langIntegerArray = null;
        if (searchOptions != null) {
            defaultSearchType = searchOptions.getDefaultSearchType();
            int[] langIntArray = searchOptions.getLangIntegerArray();
            if (langIntArray != null) {
                touteslangues = false;
                langIntegerArray = langIntArray;
            }
            corpusCodeList = searchOptions.getCorpusCodeList();
            thesaurusCodeList = searchOptions.getThesaurusCodeList();
            filterBuffer = new FilterBuffer(dataAccess, searchOptions);
        }
        if (langIntegerArray == null) {
            touteslangues = true;
            langIntegerArray = scrutariDB.getLangStats().getContentLangArray();
        }
        if (corpusCodeList == null) {
            corpusCodeList = dataAccess.getCorpusCodeList();
        }
        if (thesaurusCodeList == null) {
            thesaurusCodeList = dataAccess.getThesaurusCodeList();
        }
        int corpusCount = corpusCodeList.getCodeCount();
        int[] codecorpusArray = new int[corpusCount];
        for (int i = 0; i < corpusCount; i++) {
            codecorpusArray[i] = corpusCodeList.getCode(i);
        }
        int thesaurusCount = thesaurusCodeList.getCodeCount();
        int[] codethesaurusArray = new int[thesaurusCount];
        for (int i = 0; i < thesaurusCount; i++) {
            codethesaurusArray[i] = thesaurusCodeList.getCode(i);
        }
        CodeList ficheCodeList = searchOptions.getFicheCodeList();
        if (ficheCodeList != null) {
            codeficheSet = new HashSet<Integer>();
            int ficheCount = ficheCodeList.getCodeCount();
            for (int i = 0; i < ficheCount; i++) {
                codeficheSet.add(ficheCodeList.getCode(i));
            }
        }
        return new InternalSeekOptions(touteslangues, langIntegerArray, withFiches, codecorpusArray, withMotscles, codethesaurusArray, defaultSearchType, codeficheSet, filterBuffer);
    }
    
    
    private static class InternalSeekOptions implements SeekOptions {
        
        private final boolean allLang;
        private final int[] langIntegerArray;
        private final boolean withFiches;
        private final int[] codecorpusArray;
        private final boolean withMotscles;
        private final int[] codethesaurusArray;
        private final int defaultSearchType;
        private final Set<Integer> codeficheSet;
        private final FilterBuffer filterBuffer;
        
        private InternalSeekOptions(boolean allLang, int[] langIntegerArray, boolean withFiches, int[] codecorpusArray, boolean withMotscles, int[] codethesaurusArray, int defaultSearchType, Set<Integer> codeficheSet, FilterBuffer filterBuffer) {
            this.allLang = allLang;
            this.langIntegerArray = langIntegerArray;
            this.withFiches = withFiches;
            this.codecorpusArray = codecorpusArray;
            this.withMotscles = withMotscles;
            this.codethesaurusArray = codethesaurusArray;
            this.defaultSearchType = defaultSearchType;
            this.codeficheSet = codeficheSet;
            this.filterBuffer = filterBuffer;
        }
        
        @Override
        public int[] getLangIntegerArray() {
            return langIntegerArray;
        }
        
        @Override
        public boolean isWithFiches() {
            return withFiches;
        }
        
        @Override
        public boolean isWithMotscles() {
            return withMotscles;
        }
        
        @Override
        public int getDefaultSearchType() {
            return defaultSearchType;
        }
        
        @Override
        public boolean isAllLang() {
            return allLang;
        }
        
        @Override
        public int[] getCodecorpusArray() {
            return codecorpusArray;
        }
        
        @Override
        public int[] getCodethesaurusArray() {
            return codethesaurusArray;
        }
        
        @Override
        public boolean acceptFiche(int codefiche) {
            if (codeficheSet != null) {
                if (!codeficheSet.contains(codefiche)) {
                    return false;
                }
            }
            if (filterBuffer == null) {
                return true;
            }
            return filterBuffer.acceptFiche(codefiche);
        }
        
        @Override
        public boolean acceptMotcle(int codemotcle) {
            if (filterBuffer == null) {
                return true;
            }
            return filterBuffer.acceptMotcle(codemotcle);
        }
        
    }
    
}
