/* ScrutariLib_SearchEngine - Copyright (c) 2008-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.seeker.lexie;

import net.mapeadores.util.text.SubstringPosition;
import net.mapeadores.util.text.collation.map.SearchResultUnit;
import net.mapeadores.util.text.collation.map.SearchResultUnitList;
import net.mapeadores.util.text.search.MultiSearchToken;
import net.mapeadores.util.text.search.SimpleSearchToken;
import net.scrutari.db.api.LexieAccess;
import net.scrutari.lexie.Alinea;
import net.scrutari.lexie.FicheLexification;
import net.scrutari.lexie.LexieOccurrences;
import net.scrutari.lexie.LexieOccurrencesList;
import net.scrutari.lexie.MotcleLexification;
import net.scrutari.searchengine.api.operands.search.AlineaFilter;
import net.scrutari.searchengine.api.seeker.SeekHandler;
import net.scrutari.searchengine.api.seeker.SeekOptions;


/**
 *
 * @author Vincent Calame
 */
class MultiTokenSeeker {

    private MultiTokenSeeker() {
    }

    static void seek(MultiSearchToken multiToken, int operandNumber, int langInteger, LexieSeeker lexieSeeker, AlineaFilter alineaFilter) {
        PotentialLexieSequence potentialLexieSequence = new PotentialLexieSequence(multiToken, langInteger, lexieSeeker);
        if (!potentialLexieSequence.isInit()) {
            return;
        }
        int length = potentialLexieSequence.getPotentialStartCount();
        for (int i = 0; i < length; i++) {
            long lexieId = potentialLexieSequence.setCurrentPotentialStart(i);
            scanLexieOccurrences(potentialLexieSequence, lexieId, operandNumber, langInteger, lexieSeeker, alineaFilter);
        }
    }

    private static void scanLexieOccurrences(PotentialLexieSequence potentialLexieSequence, long lexieId, int operandNumber, int langInteger, LexieSeeker lexieSeeker, AlineaFilter alineaFilter) {
        SeekOptions seekOptions = lexieSeeker.getSeekOptions();
        LexieAccess lexieAccess = lexieSeeker.getLexieAccess();
        if (seekOptions.isWithFiches()) {
            int[] codecorpusArray = seekOptions.getCodecorpusArray();
            int corpusCount = codecorpusArray.length;
            for (int i = 0; i < corpusCount; i++) {
                int codecorpus = codecorpusArray[i];
                LexieOccurrencesList lexieOccurrencesList = lexieAccess.getLexieOccurrencesList(lexieId, codecorpus);
                int lexieCount = lexieOccurrencesList.getLexieOccurrencesCount();
                for (int j = 0; j < lexieCount; j++) {
                    LexieOccurrences lexieOccurrences = lexieOccurrencesList.getLexieOccurrences(j);
                    int codefiche = lexieOccurrences.getCode();
                    if (seekOptions.acceptFiche(codefiche)) {
                        scanFicheLexification(potentialLexieSequence, codecorpus, lexieOccurrences, operandNumber, lexieSeeker, alineaFilter);
                    }
                }
            }
        }
        if (seekOptions.isWithMotscles() && alineaFilter.acceptMotcle()) {
            int[] codethesaurusArray = seekOptions.getCodethesaurusArray();
            int thesaurusCount = codethesaurusArray.length;
            for (int j = 0; j < thesaurusCount; j++) {
                int codethesaurus = codethesaurusArray[j];
                LexieOccurrencesList lexieOccurrencesList = lexieAccess.getLexieOccurrencesList(lexieId, codethesaurus);
                int lexieCount = lexieOccurrencesList.getLexieOccurrencesCount();
                for (int k = 0; k < lexieCount; k++) {
                    LexieOccurrences lexieOccurrences = lexieOccurrencesList.getLexieOccurrences(k);
                    int codemotcle = lexieOccurrences.getCode();
                    if (seekOptions.acceptMotcle(codemotcle)) {
                        scanMotcleLexification(potentialLexieSequence, codethesaurus, lexieOccurrences, operandNumber, langInteger, lexieSeeker);
                    }
                }
            }
        }
    }

    private static void scanFicheLexification(PotentialLexieSequence potentialLexieSequence, int codecorpus, LexieOccurrences lexieOccurrences, int operandNumber, LexieSeeker lexieSeeker, AlineaFilter alineaFilter) {
        SeekHandler seekHandler = lexieSeeker.getSeekHandler();
        LexieAccess lexieAccess = lexieSeeker.getLexieAccess();
        int codefiche = lexieOccurrences.getCode();
        int count = lexieOccurrences.getAlineaCount();
        FicheLexification ficheLexification = lexieAccess.getFicheLexification(codefiche);
        for (int i = 0; i < count; i++) {
            int alineaInteger = lexieOccurrences.getAlineaInteger(i);
            if (alineaFilter.acceptAlinea(codecorpus, alineaInteger)) {
                Alinea alinea = ficheLexification.getAlinea(alineaInteger);
                int[] array = lexieOccurrences.getLexieIndexInFieldLexification(i);
                int ln2 = array.length;
                for (int j = 0; j < ln2; j++) {
                    int indexInLexification = array[j];
                    SubstringPosition substringPosition = potentialLexieSequence.testAlinea(alinea, indexInLexification);
                    if (substringPosition != null) {
                        seekHandler.addAlineaOccurrence(codefiche, codecorpus, operandNumber, alineaInteger, substringPosition);
                        lexieSeeker.addFiche(codefiche, codecorpus);
                    }
                }
            }
        }
    }

    private static void scanMotcleLexification(PotentialLexieSequence potentialLexieSequence, int codethesaurus, LexieOccurrences lexieOccurrences, int operandNumber, int langInteger, LexieSeeker lexieSeeker) {
        SeekHandler seekHandler = lexieSeeker.getSeekHandler();
        LexieAccess lexieAccess = lexieSeeker.getLexieAccess();
        int codemotcle = lexieOccurrences.getCode();
        MotcleLexification motcleLexification = lexieAccess.getMotcleLexification(codemotcle);
        Alinea alinea = motcleLexification.getAlinea(langInteger);
        int[] array = lexieOccurrences.getLexieIndexInFieldLexification(0);
        int ln2 = array.length;
        for (int j = 0; j < ln2; j++) {
            int indexInLexification = array[j];
            SubstringPosition substringPosition = potentialLexieSequence.testAlinea(alinea, indexInLexification);
            if (substringPosition != null) {
                float coverRate = ((float) substringPosition.getLength()) / (alinea.getWholeLength());
                seekHandler.addLangOccurrence(codemotcle, codethesaurus, operandNumber, langInteger, substringPosition, coverRate);
                lexieSeeker.addMotcle(codemotcle, codethesaurus);
            }
        }
    }


    private static class PotentialLexieSequence {

        private long[][] lexieIdArrays;
        private SubstringPosition[] firstSearchTextePositionArray;
        private SubstringPosition[] lastSearchTextePositionArray;
        private int currentPotentialStart = 0;

        private PotentialLexieSequence(MultiSearchToken subtokenArray, int langInteger, LexieSeeker lexieSeeker) {
            int length = subtokenArray.getSubtokenCount();
            lexieIdArrays = new long[length][];
            for (int i = 0; i < length; i++) {
                SimpleSearchToken token = subtokenArray.getSubtoken(i);
                SearchResultUnitList list = lexieSeeker.search(token.getTokenString(), langInteger, token.getSearchType());
                if ((list == null) || (list.getSearchResultUnitCount() == 0)) {
                    lexieIdArrays = null;
                    return;
                }
                int size = list.getSearchResultUnitCount();
                long[] array = new long[size];
                SubstringPosition[] searchTextePositionArray = null;
                if (i == 0) {
                    firstSearchTextePositionArray = new SubstringPosition[size];
                    searchTextePositionArray = firstSearchTextePositionArray;
                } else if (i == (length - 1)) {
                    lastSearchTextePositionArray = new SubstringPosition[size];
                    searchTextePositionArray = lastSearchTextePositionArray;
                }
                for (int j = 0; j < size; j++) {
                    SearchResultUnit searchResultItem = list.getSearchResultUnit(j);
                    Long lexieId = (Long) searchResultItem.getValue();
                    array[j] = lexieId.longValue();
                    if (searchTextePositionArray != null) {
                        searchTextePositionArray[j] = searchResultItem.getSearchTextPositionInCollatedKey();
                    }
                }
                lexieIdArrays[i] = array;
            }

        }

        private boolean isInit() {
            return (lexieIdArrays != null);
        }

        /*
         * Retourne le nombre de départs possibles pour la séquence. Exemple, si
         * la séquence est "gouv* eau", les départs peuvent être gouvernement ou
         * gouvernance.
         */
        private int getPotentialStartCount() {
            return lexieIdArrays[0].length;
        }

        /*
         * Indique que les tests (voir testLexification) vont se faire sur le
         * i-ème départ possible et retourne l'identifiant de ce i-ème départ
         * (voir getPotentialStartCount ci-dessus)
         */
        private long setCurrentPotentialStart(int i) {
            currentPotentialStart = i;
            return lexieIdArrays[0][i];
        }

        private SubstringPosition testAlinea(Alinea alinea, int index) {
            int length = lexieIdArrays.length;
            if ((index + length) > alinea.getLexieCount()) {
                return null;
            }
            int lastTokenLexieIndex = -1;
            for (int i = 1; i < length; i++) {
                long lexieId = alinea.getLexieId(index + i);
                int test = test(lexieId, i);
                if (test == -1) {
                    return null;
                } else if (i == (length - 1)) {
                    lastTokenLexieIndex = test;
                }
            }
            SubstringPosition firstSubstringPosition = alinea.getCollationUnitPosition(index).convertToPositionInWholeSource(firstSearchTextePositionArray[currentPotentialStart]);
            SubstringPosition lastSubstringPosition = alinea.getCollationUnitPosition(index + length - 1).convertToPositionInWholeSource(lastSearchTextePositionArray[lastTokenLexieIndex]);
            SubstringPosition finalSubstringPosition = SubstringPosition.merge(firstSubstringPosition, lastSubstringPosition);
            return finalSubstringPosition;
        }

        private int test(long lexieId, int tokenIndex) {
            long[] potentialArray = lexieIdArrays[tokenIndex];
            int length = potentialArray.length;
            for (int i = 0; i < length; i++) {
                if (potentialArray[i] == lexieId) {
                    return i;
                }
            }
            return -1;
        }

    }

}
