/* ScrutariLib_SearchEngine - Copyright (c) 2008-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.seeker.lexie;

import net.mapeadores.util.text.SubstringPosition;
import net.mapeadores.util.text.collation.CollationUnitPosition;
import net.mapeadores.util.text.collation.map.SearchResultUnit;
import net.mapeadores.util.text.collation.map.SearchResultUnitList;
import net.mapeadores.util.text.search.SimpleSearchToken;
import net.scrutari.db.api.LexieAccess;
import net.scrutari.lexie.LexieOccurrences;
import net.scrutari.lexie.LexieOccurrencesList;
import net.scrutari.searchengine.api.operands.search.AlineaFilter;
import net.scrutari.searchengine.api.seeker.SeekHandler;
import net.scrutari.searchengine.api.seeker.SeekOptions;


/**
 *
 * @author Vincent Calame
 */
class SimpleTokenSeeker {

    private SimpleTokenSeeker() {
    }

    static void seek(SimpleSearchToken searchToken, int operandNumber, int langInteger, LexieSeeker lexieSeeker, AlineaFilter alineaFilter) {
        SearchResultUnitList list = lexieSeeker.search(searchToken.getTokenString(), langInteger, searchToken.getSearchType());
        if (list != null) {
            int count = list.getSearchResultUnitCount();
            for (int j = 0; j < count; j++) {
                SearchResultUnit searchResultItem = list.getSearchResultUnit(j);
                Long lexieId = (Long) searchResultItem.getValue();
                add(lexieId.longValue(), searchResultItem.getSearchTextPositionInCollatedKey(), operandNumber, langInteger, lexieSeeker, alineaFilter);
            }
        }
    }

    private static void add(long lexieId, SubstringPosition searchTextePositionInCollatedKey, int operandNumber, int langInteger, LexieSeeker lexieSeeker, AlineaFilter alineaFilter) {
        LexieAccess lexieAccess = lexieSeeker.getLexieAccess();
        SeekOptions seekOptions = lexieSeeker.getSeekOptions();
        SeekHandler seekHandler = lexieSeeker.getSeekHandler();
        if (seekOptions.isWithFiches()) {
            int[] codecorpusArray = seekOptions.getCodecorpusArray();
            int corpusCount = codecorpusArray.length;
            for (int i = 0; i < corpusCount; i++) {
                int codecorpus = codecorpusArray[i];
                LexieOccurrencesList lexieOccurrencesList = lexieAccess.getLexieOccurrencesList(lexieId, codecorpus);
                int lexieCount = lexieOccurrencesList.getLexieOccurrencesCount();
                for (int j = 0; j < lexieCount; j++) {
                    LexieOccurrences lexieOccurrences = lexieOccurrencesList.getLexieOccurrences(j);
                    int codefiche = lexieOccurrences.getCode();
                    if (seekOptions.acceptFiche(codefiche)) {
                        int alineaCount = lexieOccurrences.getAlineaCount();
                        for (int k = 0; k < alineaCount; k++) {
                            int alineaInteger = lexieOccurrences.getAlineaInteger(k);
                            if (alineaFilter.acceptAlinea(codecorpus, alineaInteger)) {
                                CollationUnitPosition bestCollationUnitPosition = lexieOccurrences.getBestPositionInAlinea(k);
                                int bestAlineaInteger = alineaInteger;
                                SubstringPosition sourcePosition = bestCollationUnitPosition.convertToPositionInWholeSource(searchTextePositionInCollatedKey);
                                seekHandler.addAlineaOccurrence(codefiche, codecorpus, operandNumber, bestAlineaInteger, sourcePosition);
                                lexieSeeker.addFiche(codefiche, codecorpus);
                            }
                        }
                    }
                }
            }
        }
        if (seekOptions.isWithMotscles() && alineaFilter.acceptMotcle()) {
            int[] codethesaurusArray = seekOptions.getCodethesaurusArray();
            int thesaurusCount = codethesaurusArray.length;
            for (int j = 0; j < thesaurusCount; j++) {
                int codethesaurus = codethesaurusArray[j];
                LexieOccurrencesList lexieOccurrencesList = lexieAccess.getLexieOccurrencesList(lexieId, codethesaurus);
                int lexieCount = lexieOccurrencesList.getLexieOccurrencesCount();
                for (int k = 0; k < lexieCount; k++) {
                    LexieOccurrences p = lexieOccurrencesList.getLexieOccurrences(k);
                    int codemotcle = p.getCode();
                    if (seekOptions.acceptMotcle(codemotcle)) {
                        CollationUnitPosition bestCollationUnitPosition = p.getBestPositionInAlinea(0);
                        SubstringPosition sourcePosition = bestCollationUnitPosition.convertToPositionInWholeSource(searchTextePositionInCollatedKey);
                        float coverRate = ((float) sourcePosition.getLength()) / (p.getAlineaWholeLength(0));
                        seekHandler.addLangOccurrence(codemotcle, codethesaurus, operandNumber, langInteger, sourcePosition, coverRate);
                        lexieSeeker.addMotcle(codemotcle, codethesaurus);
                    }
                }
            }
        }
    }

}
