/* ScrutariLib_SearchEngine - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.xml;

import java.io.IOException;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.logicaloperation.Operand;
import net.mapeadores.util.text.TextConstants;
import net.mapeadores.util.xml.XMLPart;
import net.mapeadores.util.xml.XMLWriter;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.db.api.DataAccess;
import net.scrutari.searchengine.api.operands.search.QInfo;
import net.scrutari.searchengine.api.options.PertinencePonderation;
import net.scrutari.searchengine.api.options.ScrutariDataURIList;
import net.scrutari.searchengine.api.options.SearchOptions;
import net.scrutari.searchengine.api.result.FicheSearchSource;


/**
 *
 * @author Vincent Calame
 */
public class FicheSearchSourceXMLPart extends XMLPart {

    public FicheSearchSourceXMLPart(XMLWriter xmlWriter) {
        super(xmlWriter);
    }

    public void appendFicheSearchSource(DataAccess dataAccess, FicheSearchSource ficheSearchSource) throws IOException {
        openTag("fiche-search");
        appendQInfo(ficheSearchSource.getQInfo());
        appendSearchOptions(dataAccess, ficheSearchSource.getSearchOptions());
        closeTag("fiche-search");
    }

    private void appendQInfo(QInfo qInfo) throws IOException {
        startOpenTag("q-info");
        addAttribute("q", qInfo.getQ());
        addAttribute("q-source", qInfo.getQSource());
        addAttribute("q-length", qInfo.getQLength());
        closeEmptyTag();
    }

    private void appendSearchOptions(DataAccess dataAccess, SearchOptions searchOptions) throws IOException {
        if (searchOptions == null) {
            return;
        }
        startOpenTag("options");
        int langUi = searchOptions.getLangUi();
        if (langUi != LangInteger.UNDETERMINED) {
            addAttribute("lang-ui", LangInteger.toString(langUi));
        }
        addAttribute("default-search-type", TextConstants.toString(searchOptions.getDefaultSearchType()));
        endOpenTag();
        int[] langIntegerArray = searchOptions.getLangIntegerArray();
        if (langIntegerArray != null) {
            openTag("langs");
            int length = langIntegerArray.length;
            for (int i = 0; i < length; i++) {
                addSimpleElement("lang", LangInteger.toString(langIntegerArray[i]));
            }
            closeTag("langs");
        }
        addList("base-list", ScrutariDataURI.BASEURI_TYPE, searchOptions);
        addList("corpus-list", ScrutariDataURI.CORPUSURI_TYPE, searchOptions);
        addList("thesaurus-list", ScrutariDataURI.THESAURUSURI_TYPE, searchOptions);
        addList("fiche-list", ScrutariDataURI.FICHEURI_TYPE, searchOptions);
        addList("motcle-list", ScrutariDataURI.MOTCLEURI_TYPE, searchOptions);
        Operand ficheEligibilityOperand = searchOptions.getFicheEligibilityOperand();
        if (ficheEligibilityOperand != null) {
            addSimpleElement("fiche-eligibility", ficheEligibilityOperand.toString());
        }
        Operand motcleEligibilityOperand = searchOptions.getMotcleEligibilityOperand();
        if (motcleEligibilityOperand != null) {
            addSimpleElement("motcle-eligibility", motcleEligibilityOperand.toString());
        }
        PertinencePonderation pertinencePonderation = searchOptions.getPertinencePonderation();
        if (pertinencePonderation != null) {
            startOpenTag("ponderation");
            addAttribute("occurrence", toPertinenceInt(pertinencePonderation.getOccurrencePonderation()));
            addAttribute("date", toPertinenceInt(pertinencePonderation.getDatePonderation()));
            addAttribute("origin", toPertinenceInt(pertinencePonderation.getOriginPonderation()));
            closeEmptyTag();
        }
        closeTag("options");
    }

    private void addList(String tagName, short type, SearchOptions searchOptions) throws IOException {
        ScrutariDataURIList scrutariDataURIList = searchOptions.getScrutariDataURIList(type);
        if (scrutariDataURIList != null) {
            openTag(tagName);
            int thesaurusCount = scrutariDataURIList.getScrutariDataURICount();
            for (int i = 0; i < thesaurusCount; i++) {
                addSimpleElement("uri", scrutariDataURIList.getScrutariDataURI(i).toString());
            }
            closeTag(tagName);
        }
    }

    private int toPertinenceInt(float f) {
        return (int) (f * 100);
    }

}
