/* ScrutariLib_Supermotcle - Copyright (c) 2010-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.supermotcle.tools;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;
import net.mapeadores.util.text.collation.CollationUnit;
import net.scrutari.data.BaseData;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.codes.CodeList;
import net.scrutari.db.api.codes.ElementCodeList;
import net.scrutari.db.api.codes.IndexationCodeList;
import net.scrutari.db.tools.codes.CodesUtils;
import net.scrutari.supermotcle.api.Supermotcle;
import net.scrutari.supermotcle.api.SupermotcleIndexation;


/**
 *
 * @author Vincent Calame
 */
public class SupermotcleBuilder {

    private final static SupermotcleIndexationComparator INDEXATION_COMPARATOR = new SupermotcleIndexationComparator();
    private final int langInteger;
    private final int code;
    private final List<Integer> codemotcleList = new ArrayList<Integer>();
    private final List<Integer> codethesaurusList = new ArrayList<Integer>();
    private final DataAccess dataAccess;
    private final List<SupermotcleIndexationBuilder> builderList = new ArrayList<SupermotcleIndexationBuilder>();
    private CollationUnit collationUnit;

    public SupermotcleBuilder(DataAccess dataAccess, int langInteger, int code, int codethesaurus, int codemotcle, CollationUnit collationUnit) {
        this.dataAccess = dataAccess;
        this.langInteger = langInteger;
        this.code = code;
        this.collationUnit = collationUnit;
        codemotcleList.add(codemotcle);
        codethesaurusList.add(codethesaurus);
        int baseCount = dataAccess.getBaseCount();
        for (int i = 0; i < baseCount; i++) {
            BaseData baseData = dataAccess.getBaseDataByIndex(i);
            int codebase = baseData.getCodeBase();
            SupermotcleIndexationBuilder builder = new SupermotcleIndexationBuilder(codebase);
            builderList.add(builder);
            int corpusCount = baseData.getCorpusCount();
            for (int j = 0; j < corpusCount; j++) {
                int codecorpus = baseData.getCodeCorpus(j);
                builder.initCorpus(codecorpus);
            }
        }
        checkIndexation(codemotcle);
    }

    public void addMotcle(int codethesaurus, int codemotcle, CollationUnit newCollationUnit) { //On compare pour voir quelle est la meilleur « collation »
        String oldSourceString = collationUnit.getSourceString();
        String newSourceString = newCollationUnit.getSourceString();
        boolean replace = false;
        int newLength = newSourceString.length();
        int oldLength = oldSourceString.length();
        if (newLength < oldLength) {
            replace = true;
        } else if (newLength == oldLength) {
            if (newSourceString.compareTo(oldSourceString) > 0) {
                replace = true;
            }
        }
        if (replace) {
            collationUnit = newCollationUnit;
        }
        codemotcleList.add(codemotcle);
        codethesaurusList.add(codethesaurus);
        checkIndexation(codemotcle);
    }

    public Supermotcle toSupermotcle() {
        InternalSupermotcle is = new InternalSupermotcle(this);
        return is;
    }

    private void checkIndexation(int codemotcle) {
        int baseCount = builderList.size();
        for (int i = 0; i < baseCount; i++) {
            SupermotcleIndexationBuilder builder = builderList.get(i);
            builder.checkMotcle(codemotcle);
        }
    }


    private class SupermotcleIndexationBuilder {

        private final int codebase;
        private final List<CorpusListBuilder> corpusListBuilderList = new ArrayList<CorpusListBuilder>();

        private SupermotcleIndexationBuilder(int codebase) {
            this.codebase = codebase;
        }

        private void initCorpus(int codecorpus) {
            corpusListBuilderList.add(new CorpusListBuilder(codecorpus));
        }

        private void checkMotcle(int codemotcle) {
            int size = corpusListBuilderList.size();
            for (int i = 0; i < size; i++) {
                CorpusListBuilder corpusListBuilder = corpusListBuilderList.get(i);
                IndexationCodeList indexationCodeList = dataAccess.getIndexationCodeList(codemotcle, corpusListBuilder.codecorpus);
                int ficheCount = indexationCodeList.getCodeCount();
                for (int j = 0; j < ficheCount; j++) {
                    corpusListBuilder.addCodeFiche(indexationCodeList.getCode(j));
                }
            }
        }

    }


    private class CorpusListBuilder {

        private final int codecorpus;
        private final List<Integer> codeficheBufferList = new ArrayList<Integer>();
        private CodeList codeList;

        private CorpusListBuilder(int codecorpus) {
            this.codecorpus = codecorpus;
        }

        private void addCodeFiche(int codefiche) {
            codeficheBufferList.add(codefiche);
        }

        private boolean hasFiche() {
            return (!codeficheBufferList.isEmpty());
        }

        private void endInit() {
            int size = codeficheBufferList.size();
            Set<Integer> existingSet = new HashSet<Integer>();
            int[] array = new int[size];
            int p = 0;
            for (int i = 0; i < size; i++) {
                Integer itg = codeficheBufferList.get(i);
                if (!existingSet.contains(itg)) {
                    array[p] = itg.intValue();
                    p++;
                    existingSet.add(itg);
                }
            }
            if (p < size) {
                int[] temp = new int[p];
                System.arraycopy(array, 0, temp, 0, p);
                array = temp;
            }
            codeList = CodesUtils.toCodeList(array);
        }

    }


    private static class InternalSupermotcle implements Supermotcle {

        private final int langInteger;
        private final int code;
        private final CollationUnit collationUnit;
        private final ElementCodeList codemotcleList;
        private final int indexationCount;
        private final SupermotcleIndexation[] supermotcleIndexationArray;

        private InternalSupermotcle(SupermotcleBuilder builder) {
            this.langInteger = builder.langInteger;
            this.code = builder.code;
            this.collationUnit = builder.collationUnit;
            this.codemotcleList = CodesUtils.toElementCodeList(builder.codemotcleList, builder.codethesaurusList);
            List<SupermotcleIndexationBuilder> builderList = builder.builderList;
            int builderSize = builderList.size();
            Set<SupermotcleIndexation> resultSet = new TreeSet<SupermotcleIndexation>(INDEXATION_COMPARATOR);
            int indexCount = 0;
            for (int i = 0; i < builderSize; i++) {
                SupermotcleIndexationBuilder supermotcleIndexationBuiler = builderList.get(i);
                SupermotcleIndexation supermotcleIndexation = new InternalSupermotcleIndexation(supermotcleIndexationBuiler);
                int ic = supermotcleIndexation.getIndexationCount();
                if (ic > 0) {
                    indexCount = indexCount + ic;
                    resultSet.add(supermotcleIndexation);
                }
            }
            this.indexationCount = indexCount;
            this.supermotcleIndexationArray = resultSet.toArray(new SupermotcleIndexation[resultSet.size()]);
        }

        @Override
        public int getLangInteger() {
            return langInteger;
        }

        @Override
        public int getCode() {
            return code;
        }

        @Override
        public CollationUnit getCollationUnit() {
            return collationUnit;
        }

        @Override
        public ElementCodeList getCodemotcleList() {
            return codemotcleList;
        }

        @Override
        public int getSupermotcleIndexationCount() {
            return supermotcleIndexationArray.length;
        }

        @Override
        public SupermotcleIndexation getSupermotcleIndexation(int i) {
            return supermotcleIndexationArray[i];
        }

        @Override
        public int getIndexationCount() {
            return indexationCount;
        }

    }


    private static class SupermotcleIndexationComparator implements Comparator<SupermotcleIndexation> {

        private SupermotcleIndexationComparator() {
        }

        @Override
        public int compare(SupermotcleIndexation s1, SupermotcleIndexation s2) {
            int ic1 = s1.getIndexationCount();
            int ic2 = s2.getIndexationCount();
            if (ic1 > ic2) {
                return -1;
            }
            if (ic1 < ic2) {
                return 1;
            }
            int c1 = s1.getCodeBase();
            int c2 = s2.getCodeBase();
            if (c1 < c2) {
                return -1;
            }
            if (c2 > c1) {
                return 1;
            }
            return 0;
        }

    }


    private static class InternalSupermotcleIndexation implements SupermotcleIndexation {

        private final int codebase;
        private final int indexationCount;
        private final int[] codecorpusArray;
        private final CodeList[] ficheCodeListByCorpusArray;

        InternalSupermotcleIndexation(SupermotcleIndexationBuilder builder) {
            this.codebase = builder.codebase;
            List<CorpusListBuilder> corpusListBuilderList = builder.corpusListBuilderList;
            int size = corpusListBuilderList.size();
            List<CorpusListBuilder> validList = new ArrayList<CorpusListBuilder>();
            for (int i = 0; i < size; i++) {
                CorpusListBuilder corpusListBuilder = corpusListBuilderList.get(i);
                if (corpusListBuilder.hasFiche()) {
                    corpusListBuilder.endInit();
                    validList.add(corpusListBuilder);
                }
            }
            int size2 = validList.size();
            this.codecorpusArray = new int[size2];
            this.ficheCodeListByCorpusArray = new CodeList[size2];
            int indexCount = 0;
            for (int i = 0; i < size2; i++) {
                CorpusListBuilder corpusListBuilder = validList.get(i);
                codecorpusArray[i] = corpusListBuilder.codecorpus;
                ficheCodeListByCorpusArray[i] = corpusListBuilder.codeList;
                indexCount = indexCount + corpusListBuilder.codeList.getCodeCount();
            }
            this.indexationCount = indexCount;
        }

        @Override
        public int getCodeBase() {
            return codebase;
        }

        @Override
        public int getIndexationCount() {
            return indexationCount;
        }

        @Override
        public int getCorpusCount() {
            return codecorpusArray.length;
        }

        @Override
        public int getCodeCorpus(int i) {
            return codecorpusArray[i];
        }

        @Override
        public CodeList getFicheCodeListByCorpus(int i) {
            return ficheCodeListByCorpusArray[i];
        }

    }

}
