/* ScrutariLib_Supermotcle - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.supermotcle.tools;

import java.util.HashMap;
import java.util.Map;
import net.mapeadores.util.text.Libelle;
import net.scrutari.data.MotcleData;
import net.scrutari.data.ThesaurusData;
import net.scrutari.db.ScrutariDB;
import net.scrutari.db.api.DataAccess;
import net.scrutari.supermotcle.api.SupermotcleMap;
import net.scrutari.supermotcle.api.SupermotcleProvider;


/**
 *
 * @author Vincent Calame
 */
public class SupermotcleProviderBuilder {

    private final Map<Integer, SupermotcleMapBuilder> builderMap = new HashMap<Integer, SupermotcleMapBuilder>();
    private final Map<Integer, SupermotcleMap> mapOfSupermotcleMap = new HashMap<Integer, SupermotcleMap>();

    private SupermotcleProviderBuilder() {
    }

    private SupermotcleProvider toSupermotcleProvider() {
        return new InternalSupermotcleProvider(mapOfSupermotcleMap);
    }

    private void build(DataAccess dataAccess) {
        int thesaurusCount = dataAccess.getThesaurusCount();
        for (int i = 0; i < thesaurusCount; i++) {
            ThesaurusData thesaurusData = dataAccess.getThesaurusDataByIndex(i);
            int motcleCount = thesaurusData.getMotcleCount();
            for (int j = 0; j < motcleCount; j++) {
                int codemotcle = thesaurusData.getCodeMotcle(j);
                MotcleData motcleData = dataAccess.getMotcleDataByCode(codemotcle);
                initMotcle(dataAccess, thesaurusData, motcleData);
            }
        }
        for (Map.Entry<Integer, SupermotcleMapBuilder> entry : builderMap.entrySet()) {
            Integer itg = entry.getKey();
            SupermotcleMapBuilder builder = entry.getValue();
            SupermotcleMap supermotcleMap = builder.toSupermotcleMap();
            mapOfSupermotcleMap.put(itg, supermotcleMap);
        }
    }

    private void initMotcle(DataAccess dataAccess, ThesaurusData thesaurusData, MotcleData motcleData) {
        int codemotcle = motcleData.getCodeMotcle();
        int libelleCount = motcleData.getLibelleCount();
        for (int i = 0; i < libelleCount; i++) {
            Libelle libelle = motcleData.getLibelleByIndex(i);
            SupermotcleMapBuilder supermotcleMapBuilder = getOrCreateSupermotcleMapBuilder(dataAccess, libelle.getLangInteger());
            supermotcleMapBuilder.addMotcle(thesaurusData.getCodeBase(), thesaurusData.getCodeThesaurus(), codemotcle, libelle.getLibelleString());
        }
    }

    private SupermotcleMapBuilder getOrCreateSupermotcleMapBuilder(DataAccess dataAccess, int langInteger) {
        SupermotcleMapBuilder supermotcleMapBuilder = builderMap.get(langInteger);
        if (supermotcleMapBuilder == null) {
            supermotcleMapBuilder = new SupermotcleMapBuilder(dataAccess, langInteger);
            builderMap.put(langInteger, supermotcleMapBuilder);
        }
        return supermotcleMapBuilder;
    }

    public static SupermotcleProvider build(ScrutariDB scrutariDB) {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        SupermotcleProviderBuilder builder = new SupermotcleProviderBuilder();
        builder.build(dataAccess);
        dataAccess.close();
        return builder.toSupermotcleProvider();
    }


    private static class InternalSupermotcleProvider implements SupermotcleProvider {

        private final Map<Integer, SupermotcleMap> mapOfSupermotcleMap;

        private InternalSupermotcleProvider(Map<Integer, SupermotcleMap> mapOfSupermotcleMap) {
            this.mapOfSupermotcleMap = mapOfSupermotcleMap;
        }

        @Override
        public SupermotcleMap getSupermotcleMap(int langInteger) {
            return mapOfSupermotcleMap.get(langInteger);
        }

    }

}
