/* ScrutariLib_Xml - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.xml.xmlpull;

import net.scrutari.datauri.BaseURI;
import net.scrutari.datauri.CorpusURI;
import net.scrutari.datauri.ThesaurusURI;
import net.scrutari.datauri.MotcleURI;
import net.scrutari.datauri.FicheURI;
import net.scrutari.datauri.IndexationKey;
import net.scrutari.xml.Parameters;
import net.scrutari.xml.ErrorKeys;
import net.mapeadores.util.xml.xmlpull.DefaultElementHandler;
import net.mapeadores.util.xml.xmlpull.XmlPullElementHandler;
import java.util.Map;
import net.scrutari.data.IndexationBuilder;


/**
 *
 * @author Vincent Calame
 */
public class IndexationGroupHandler extends DefaultElementHandler implements ErrorKeys {

    CorpusURI corpusURI;
    ThesaurusURI thesaurusURI;
    private Parameters parameters;
    private IndexationBuilder indexationBuilder;

    public IndexationGroupHandler(Map<String, String> attributeMap, Parameters parameters) {
        String corpuspath = getAttributeString(attributeMap, "corpus-path");
        String thesauruspath = getAttributeString(attributeMap, "thesaurus-path");
        boolean error = false;
        if (corpuspath == null) {
            parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_INDEXATION_MISSINGCORPUSPATH);
            error = true;
        }
        if (thesauruspath == null) {
            parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_INDEXATION_MISSINGTHESAURUSPATH);
            error = true;
        }
        if (error) {
            throw new IllegalArgumentException();
        }
        this.parameters = parameters;
        BaseURI baseURI = parameters.getBaseURI();
        corpusURI = (CorpusURI) parameters.intern(new CorpusURI(baseURI, corpuspath));
        thesaurusURI = (ThesaurusURI) parameters.intern(new ThesaurusURI(baseURI, thesauruspath));
        indexationBuilder = parameters.getIndexationBuilder();
        indexationBuilder.setCurrentGroup(corpusURI, thesaurusURI);
    }

    @Override
    public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
        if (tagname.equals("indexation")) {
            boolean error = false;
            String ficheid = getAttributeString(attributeMap, "fiche-id");
            if (ficheid == null) {
                parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_INDEXATION_MISSINGFICHEID);
                error = true;
            }
            String motcleid = getAttributeString(attributeMap, "motcle-id");
            if (motcleid == null) {
                parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_INDEXATION_MISSINGMOTCLEID);
                error = true;
            }
            if (!error) {
                FicheURI ficheURI = (FicheURI) parameters.intern(new FicheURI(corpusURI, ficheid));
                MotcleURI motcleURI = (MotcleURI) parameters.intern(new MotcleURI(thesaurusURI, motcleid));
                IndexationKey indexationKey = new IndexationKey(ficheURI, motcleURI);
                if (!parameters.testDoublon(indexationKey)) {
                    parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_INDEXATION_DOUBLEINDEXATION);
                    error = true;
                    return null;
                }
                int poids = 1;
                String poidsString = getAttributeString(attributeMap, "poids");
                if (poidsString == null) {
                    poidsString = getAttributeString(attributeMap, "weight");
                }
                if (poidsString != null) {
                    try {
                        poids = Integer.parseInt(poidsString);
                        if (poids < 1) {
                            poids = 1;
                        }
                    } catch (NumberFormatException nfe) {
                    }
                }
                indexationBuilder.addIndexation(indexationKey, poids);
            }
        }
        return null;
    }

}
