/* SctServer - Copyright (c) 2008-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.admin;

import fr.exemole.sctserver.SctServer;
import net.mapeadores.util.collections.IntHashMap;
import net.scrutari.datauri.CorpusURI;
import net.scrutari.db.api.DataAccess;
import net.scrutari.searchengine.api.options.CorpusClasse;
import net.scrutari.searchengine.api.options.GlobalSearchOptions;
import net.scrutari.searchengine.api.options.PertinencePonderation;
import net.scrutari.searchengine.tools.options.PertinencePonderationBuilder;


/**
 *
 * @author Vincent Calame
 */
public class OptionsManager {

    private final SctServer sctServer;
    private InternalGlobalSearchOptions internalGlobalSearchOptions = new InternalGlobalSearchOptions();
    private boolean withClasses = false;
    private CorpusClasse[] corpusClasseArray;

    public OptionsManager(SctServer sctServer) {
        this.sctServer = sctServer;
    }

    public void reloadCorpusClasseArray(CorpusClasse[] corpusClasseArray) {
        withClasses = false;
        this.corpusClasseArray = null;
        DataAccess dataAccess = sctServer.getScrutariSession().getScrutariDB().openDataAccess();
        int defaultClasseValue = 0;
        IntHashMap codeMap = null;
        if (corpusClasseArray != null) {
            this.corpusClasseArray = corpusClasseArray;
            withClasses = true;
            codeMap = new IntHashMap();
            int length = corpusClasseArray.length;
            for (int i = 0; i < length; i++) {
                CorpusClasse corpusClasse = corpusClasseArray[i];
                String name = corpusClasse.getName();
                if (name == null) {
                    defaultClasseValue = corpusClasse.getRank();
                } else {
                    int count = corpusClasse.getCorpusURICount();
                    for (int j = 0; j < count; j++) {
                        CorpusURI corpusURI = corpusClasse.getCorpusURI(j);
                        int codecorpus = dataAccess.getCode(corpusURI);
                        if (codecorpus > 0) {
                            codeMap.put(codecorpus, corpusClasse);
                        }
                    }
                }
            }
        }
        dataAccess.close();
        internalGlobalSearchOptions.reload(codeMap, defaultClasseValue);
    }

    public void setGlobalPertinencePonderation(PertinencePonderation pertinencePonderation) {
        internalGlobalSearchOptions.pertinencePonderation = pertinencePonderation;
    }

    public GlobalSearchOptions getGlobalSearchOptions() {
        return internalGlobalSearchOptions;
    }


    private class InternalGlobalSearchOptions implements GlobalSearchOptions {

        private IntHashMap codeMap;
        private int defautClassValue;
        private PertinencePonderation pertinencePonderation = PertinencePonderationBuilder.DEFAULT_PONDERATION;

        private InternalGlobalSearchOptions() {
        }

        public boolean isWithCorpusClasse() {
            return withClasses;
        }

        @Override
        public int getCorpusClasseRank(int codecorpus) {
            if (!withClasses) {
                return 0;
            }
            CorpusClasse corpusClasse = (CorpusClasse) codeMap.get(codecorpus);
            if (corpusClasse == null) {
                return defautClassValue;
            } else {
                return corpusClasse.getRank();
            }
        }

        @Override
        public int getCorpusClasseCount() {
            if (!withClasses) {
                return 0;
            }
            return corpusClasseArray.length;
        }

        @Override
        public CorpusClasse getCorpusClasse(int i) {
            return corpusClasseArray[i];
        }

        @Override
        public CorpusClasse getCorpusClasseByRank(int rank) {
            if (!withClasses) {
                return null;
            }
            int length = corpusClasseArray.length;
            for (int i = 0; i < length; i++) {
                CorpusClasse corpusClasse = corpusClasseArray[i];
                if (corpusClasse.getRank() == rank) {
                    return corpusClasse;
                }
            }
            return null;
        }

        @Override
        public PertinencePonderation getPertinencePonderation() {
            return pertinencePonderation;
        }

        private void reload(IntHashMap codeMap, int defautClassValue) {
            this.defautClassValue = defautClassValue;
            this.codeMap = codeMap;
        }

    }

}
