/* SctServer - Copyright (c) 2008-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.admin;

import fr.exemole.sctserver.api.collect.CollectErrorHandler;
import fr.exemole.sctserver.api.collect.CollectResult;
import fr.exemole.sctserver.api.collect.ScrutariInfo;
import fr.exemole.sctserver.api.collect.ScrutariInfoList;
import fr.exemole.sctserver.api.storage.SctServerStorage;
import fr.exemole.sctserver.tools.collect.CollectEngine;
import fr.exemole.sctserver.tools.collect.CollectUtils;
import fr.exemole.sctserver.tools.collect.ScrutariInfoStorage;
import java.net.URL;
import java.util.List;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.text.UrlKey;
import net.scrutari.datauri.BaseURI;


/**
 *
 * @author Vincent Calame
 */
public class ScrutariInfoManager implements ScrutariInfoList {

    private InternalScrutariInfo[] scrutariInfoArray;
    private SctServerStorage sctServerStorage;

    public ScrutariInfoManager(SctServerStorage sctServerStorage) {
        this.sctServerStorage = sctServerStorage;
        List<ScrutariInfo> scrutariInfoList = ScrutariInfoStorage.fromString(sctServerStorage.getContent(SctServerStorage.INFOS_NAME));
        int size = scrutariInfoList.size();
        if (size > 0) {
            scrutariInfoArray = new InternalScrutariInfo[size];
            for (int i = 0; i < size; i++) {
                scrutariInfoArray[i] = new InternalScrutariInfo(scrutariInfoList.get(i));
            }
        }
    }

    public int getScrutariInfoCount() {
        if (scrutariInfoArray == null) {
            return 0;
        }
        return scrutariInfoArray.length;
    }

    public ScrutariInfo getScrutariInfo(int i) {
        return scrutariInfoArray[i];
    }

    public void reloadInfoUrlArray(UrlKey[] urlKeyArray) {
        if (urlKeyArray == null) {
            scrutariInfoArray = null;
            return;
        }
        int length = urlKeyArray.length;
        InternalScrutariInfo[] newArray = new InternalScrutariInfo[length];
        for (int i = 0; i < length; i++) {
            UrlKey urlKey = urlKeyArray[i];
            InternalScrutariInfo sourceInfo = geScrutariInfo(urlKey.getKeyString());
            if (sourceInfo != null) {
                newArray[i] = sourceInfo;
            } else {
                newArray[i] = new InternalScrutariInfo(urlKey);
            }
        }
        this.scrutariInfoArray = newArray;
    }

    public boolean collect(CollectErrorHandler collectErrorHandler, boolean forceDataDownload) {
        if (scrutariInfoArray == null) {
            return false;
        }
        int length = scrutariInfoArray.length;
        boolean dataChange = false;
        for (int i = 0; i < length; i++) {
            InternalScrutariInfo scrutariInfo = scrutariInfoArray[i];
            CollectResult collectResult = CollectEngine.collectData(scrutariInfo, sctServerStorage, collectErrorHandler, forceDataDownload);
            if (CollectUtils.hasChange(scrutariInfo, collectResult)) {
                dataChange = true;
                scrutariInfo.update(collectResult);
            }
        }
        if (forceDataDownload) {
            return true;
        } else {
            return dataChange;
        }
    }

    public boolean setState(ScrutariInfo scrutariInfo, short state) {
        return ((InternalScrutariInfo) scrutariInfo).setState(state);
    }

    public void setBaseURI(ScrutariInfo scrutariInfo, BaseURI baseURI) {
        ((InternalScrutariInfo) scrutariInfo).setBaseURI(baseURI);
    }

    public void store() {
        sctServerStorage.saveContent(SctServerStorage.INFOS_NAME, ScrutariInfoStorage.toString(this));
    }

    private InternalScrutariInfo geScrutariInfo(String keyString) {
        if (scrutariInfoArray == null) {
            return null;
        }
        int length = scrutariInfoArray.length;
        for (int i = 0; i < length; i++) {
            InternalScrutariInfo scrutariInfo = scrutariInfoArray[i];
            if (scrutariInfo.getKey().equals(keyString)) {
                return scrutariInfo;
            }
        }
        return null;
    }


    private class InternalScrutariInfo implements ScrutariInfo {

        private String key;
        private URL url;
        private short state = ScrutariInfo.UNKNOWN_STATE;
        private int lastUpdate = DateInteger.NULL;
        private BaseURI baseURI;

        private InternalScrutariInfo(UrlKey urlKey) {
            this.url = urlKey.getUrl();
            this.key = urlKey.getKeyString();
        }

        private InternalScrutariInfo(ScrutariInfo scrutariInfo) {
            this.key = scrutariInfo.getKey();
            this.url = scrutariInfo.getUrl();
            this.lastUpdate = scrutariInfo.getLastUpdate();
            this.state = scrutariInfo.getState();
            this.baseURI = scrutariInfo.getBaseURI();
        }

        public String getKey() {
            return key;
        }

        public URL getUrl() {
            return url;
        }

        public short getState() {
            return state;
        }

        public int getLastUpdate() {
            return lastUpdate;
        }

        private boolean setState(short state) {
            if (this.state == state) {
                return false;
            }
            this.state = state;
            return true;
        }

        public BaseURI getBaseURI() {
            return baseURI;
        }

        private void setBaseURI(BaseURI baseURI) {
            this.baseURI = baseURI;
        }

        private void update(CollectResult collectResult) {
            this.state = collectResult.getState();
            this.lastUpdate = collectResult.getLastUpdateDateInteger();
        }

    }

}
