/* SctServer - Copyright (c) 2011-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request;

import java.util.ArrayList;
import java.util.List;
import net.mapeadores.util.logging.ProcessMessageHandler;
import net.scrutari.db.api.CodeMessageKeys;
import net.scrutari.searchengine.api.operands.OperandMessageKeys;


/**
 *
 * @author Vincent Calame
 */
public class WarningHandler {

    private String currentParameter;
    private List<Warning> warningList = new ArrayList<Warning>();
    private CodeProcessMessageHandler codeProcessMessageHandler = new CodeProcessMessageHandler();

    public WarningHandler() {
    }

    public boolean isEmpty() {
        return (warningList.isEmpty());
    }

    public void setCurrentParameter(String parameter) {
        this.currentParameter = parameter;
    }

    public void addWarning(String warningKey, String value) {
        warningList.add(new Warning(currentParameter, warningKey, value));
    }

    public int getWarningCount() {
        return warningList.size();
    }

    public Warning getWarning(int i) {
        return warningList.get(i);
    }

    public ProcessMessageHandler getCodeProcessMessageHandler() {
        return codeProcessMessageHandler;
    }


    public static class Warning {

        private final String warningParameter;
        private final String warningKey;
        private final String warningValue;

        private Warning(String warningParameter, String warningKey, String warningValue) {
            this.warningParameter = warningParameter;
            this.warningKey = warningKey;
            this.warningValue = warningValue;
        }

        public String getWarningParameter() {
            return warningParameter;
        }

        public String getWarningKey() {
            return warningKey;
        }

        public String getWarningValue() {
            return warningValue;
        }

    }


    private class CodeProcessMessageHandler implements ProcessMessageHandler {

        private CodeProcessMessageHandler() {
        }

        @Override
        public void addMessage(String category, String key, Object value) {
            String warningValue = value.toString();
            if (key.equals(CodeMessageKeys.NOT_INTEGER)) {
                addWarning(WarningKeys.MALFORMED_PARAMETER_VALUE, warningValue);
            } else if (key.equals(CodeMessageKeys.MALFORMED_URI)) {
                addWarning(WarningKeys.MALFORMED_PARAMETER_VALUE, warningValue);
            } else if (key.equals(CodeMessageKeys.UNKNOWN_URI)) {
                addWarning(WarningKeys.UNKNOWN_PARAMETER_VALUE, warningValue);
            } else if (key.equals(CodeMessageKeys.WRONG_URI_TYPE)) {
                addWarning(WarningKeys.WRONG_SCRUTARIDATATYPE_PARAMETER_VALUE, warningValue);
            } else if (key.equals(CodeMessageKeys.UNKNOWN_CODE)) {
                addWarning(WarningKeys.UNKNOWN_PARAMETER_VALUE, warningValue);
            } else if (key.equals(OperandMessageKeys.UNKNOWN_OPERAND_TYPE)) {
                addWarning(WarningKeys.MALFORMED_PARAMETER_VALUE, warningValue);
            } else if (key.equals(OperandMessageKeys.MALFORMED_DATE)) {
                addWarning(WarningKeys.MALFORMED_PARAMETER_VALUE, warningValue);
            } else if (key.equals(OperandMessageKeys.UNKNOWN_FICHESEARCHRESULT)) {
                addWarning(WarningKeys.UNKNOWN_PARAMETER_VALUE, warningValue);
            } else if (key.equals(OperandMessageKeys.MALFORMED_ADD_OPERAND)) {
                addWarning(WarningKeys.MALFORMED_PARAMETER_VALUE, warningValue);
            }
        }

    }

}
