/* SctServer - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import fr.exemole.sctserver.request.WarningHandler;
import net.mapeadores.util.json.JSONException;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.json.JsonProducer;


/**
 *
 * @author Vincent Calame
 */
public abstract class AbstractJsonProducer implements JsonProducer {

    private WarningHandler warningHandler;

    public WarningHandler getWarningHandler() {
        return warningHandler;
    }

    public void setWarningHandler(WarningHandler warningHandler) {
        this.warningHandler = warningHandler;
    }

    public void writeWarnings(JSONWriter jsonWriter) throws JSONException {
        if (warningHandler == null) {
            return;
        }
        int warningCount = warningHandler.getWarningCount();
        if (warningCount == 0) {
            return;
        }
        jsonWriter.key("warnings");
        jsonWriter.array();
        for (int i = 0; i < warningCount; i++) {
            WarningHandler.Warning warning = warningHandler.getWarning(i);
            jsonWriter.object();
            jsonWriter.key("key");
            jsonWriter.value(warning.getWarningKey());
            jsonWriter.key("parameter");
            jsonWriter.value(warning.getWarningParameter());
            String value = warning.getWarningValue();
            if (value != null) {
                jsonWriter.key("value");
                jsonWriter.value(value);
            }
            jsonWriter.endObject();
        }
        jsonWriter.endArray();

    }

}
