/* SctServer - Copyright (c) 2010-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import java.io.IOException;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.json.JSONException;
import net.mapeadores.util.json.JSONWriter;
import net.scrutari.data.BaseData;
import net.scrutari.datauri.BaseURI;
import net.scrutari.db.ScrutariDB;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.tools.util.LibelleUtils;


/**
 *
 * @author Vincent Calame
 */
public class BaseDataJsonProducer extends AbstractJsonProducer {

    private ScrutariDB scrutariDB;
    private int langUi;

    public BaseDataJsonProducer(ScrutariDB scrutariDB, int langUi) {
        this.scrutariDB = scrutariDB;
        this.langUi = langUi;
    }

    public void writeJson(Appendable appendable) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        JSONWriter jsonWriter = new JSONWriter(appendable);
        try {
            jsonWriter.object();
            jsonWriter.key("baseData");
            jsonWriter.object();
            int count = dataAccess.getBaseCount();
            jsonWriter.key("baseCount");
            jsonWriter.value(count);
            if (count > 0) {
                jsonWriter.key("baseArray");
                jsonWriter.array();
                for (int i = 0; i < count; i++) {
                    BaseData baseData = dataAccess.getBaseDataByIndex(i);
                    int code = baseData.getCodeBase();
                    jsonWriter.object();
                    jsonWriter.key("codebase");
                    jsonWriter.value(code);
                    BaseURI baseURI = baseData.getBaseURI();
                    jsonWriter.key("authorityuuid");
                    jsonWriter.value(baseURI.getAuthorityUUID());
                    String baseName = baseURI.getBaseName();
                    jsonWriter.key("basename");
                    jsonWriter.value(baseName);
                    int checkedLangInteger = baseData.checkLangUi(langUi);
                    String baseIcon = baseData.getBaseIcon(checkedLangInteger);
                    if ((baseIcon != null) && (baseIcon.length() > 0)) {
                        jsonWriter.key("baseicon");
                        jsonWriter.value(baseIcon);
                    }
                    int corpusCount = baseData.getCorpusCount();
                    jsonWriter.key("codecorpusArray");
                    jsonWriter.array();
                    for (int j = 0; j < corpusCount; j++) {
                        jsonWriter.value(baseData.getCodeCorpus(j));
                    }
                    jsonWriter.endArray();
                    int thesaurusCount = baseData.getThesaurusCount();
                    jsonWriter.key("codethesaurusArray");
                    jsonWriter.array();
                    for (int j = 0; j < thesaurusCount; j++) {
                        jsonWriter.value(baseData.getCodeThesaurus(j));
                    }
                    jsonWriter.endArray();
                    jsonWriter.key("intitules");
                    jsonWriter.object();
                    jsonWriter.key("short");
                    jsonWriter.value(LibelleUtils.getLibelleString(baseData, BaseData.INTITULE_SHORT, langUi, baseName));
                    jsonWriter.key("long");
                    jsonWriter.value(LibelleUtils.getLibelleString(baseData, BaseData.INTITULE_LONG, langUi, baseName));
                    jsonWriter.endObject();
                    jsonWriter.endObject();
                }
                jsonWriter.endArray();
            }
            jsonWriter.endObject();
            writeWarnings(jsonWriter);
            jsonWriter.endObject();
        } catch (JSONException je) {
            Throwable throwable = je.getCause();
            if (throwable == null) {
                throw new ShouldNotOccurException(je.getMessage(), je);
            } else {
                if (throwable instanceof IOException) {
                    throw ((IOException) throwable);
                } else {
                    throw new ShouldNotOccurException(throwable.getMessage(), (Exception) throwable);
                }
            }
        }
        dataAccess.close();
    }

}
