/* SctServer - Copyright (c) 2011-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import java.io.IOException;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.json.JSONException;
import net.mapeadores.util.json.JSONWriter;
import net.scrutari.searchengine.api.options.CorpusClasse;
import net.scrutari.searchengine.api.options.GlobalSearchOptions;


/**
 *
 * @author Vincent Calame
 */
public class ClasseDefJsonProducer extends AbstractJsonProducer {

    private final int langUi;
    private final GlobalSearchOptions globalSearchOptions;

    public ClasseDefJsonProducer(GlobalSearchOptions globalSearchOptions, int langUi) {
        this.globalSearchOptions = globalSearchOptions;
        this.langUi = langUi;
    }

    @Override
    public void writeJson(Appendable appendable) throws IOException {
        int classeCount = globalSearchOptions.getCorpusClasseCount();
        JSONWriter jsonWriter = new JSONWriter(appendable);
        try {
            jsonWriter.object();
            jsonWriter.key("classeDef");
            jsonWriter.object();
            jsonWriter.key("classeCount");
            jsonWriter.value(classeCount);
            if (classeCount > 0) {
                jsonWriter.key("classeArray");
                jsonWriter.array();
                for (int i = 0; i < classeCount; i++) {
                    CorpusClasse corpusClasse = globalSearchOptions.getCorpusClasse(i);
                    int rank = corpusClasse.getRank();
                    jsonWriter.object();
                    jsonWriter.key("rank");
                    jsonWriter.value(rank);
                    jsonWriter.key("name");
                    jsonWriter.value(corpusClasse.getName());
                    jsonWriter.key("intitule");
                    jsonWriter.value(corpusClasse.getLibelleString(langUi, true));
                    jsonWriter.endObject();
                }
                jsonWriter.endArray();
            }
            jsonWriter.endObject();
            writeWarnings(jsonWriter);
            jsonWriter.endObject();
        } catch (JSONException je) {
            Throwable throwable = je.getCause();
            if (throwable == null) {
                throw new ShouldNotOccurException(je.getMessage(), je);
            } else {
                if (throwable instanceof IOException) {
                    throw ((IOException) throwable);
                } else {
                    throw new ShouldNotOccurException(throwable.getMessage(), (Exception) throwable);
                }
            }
        }
    }

}
