/* SctServer  - Copyright (c) 2011-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import java.io.IOException;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.json.JSONException;
import net.mapeadores.util.json.JSONWriter;
import net.scrutari.data.CorpusData;
import net.scrutari.datauri.CorpusURI;
import net.scrutari.db.ScrutariDB;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.tools.util.LibelleUtils;


/**
 *
 * @author Vincent Calame
 */
public class CorpusDataJsonProducer extends AbstractJsonProducer {

    private ScrutariDB scrutariDB;
    private int langUi;

    public CorpusDataJsonProducer(ScrutariDB scrutariDB, int langUi) {
        this.scrutariDB = scrutariDB;
        this.langUi = langUi;
    }

    public void writeJson(Appendable appendable) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        JSONWriter jsonWriter = new JSONWriter(appendable);
        try {
            jsonWriter.object();
            jsonWriter.key("corpusData");
            jsonWriter.object();
            int count = dataAccess.getCorpusCount();
            jsonWriter.key("corpusCount");
            jsonWriter.value(count);
            if (count > 0) {
                jsonWriter.key("corpusArray");
                jsonWriter.array();
                for (int i = 0; i < count; i++) {
                    CorpusData corpusData = dataAccess.getCorpusDataByIndex(i);
                    int code = corpusData.getCodeCorpus();
                    jsonWriter.object();
                    jsonWriter.key("codecorpus");
                    jsonWriter.value(code);
                    jsonWriter.key("codebase");
                    jsonWriter.value(corpusData.getCodeBase());
                    CorpusURI corpusURI = corpusData.getCorpusURI();
                    String corpusName = corpusURI.getCorpusName();
                    jsonWriter.key("corpusname");
                    jsonWriter.value(corpusName);
                    int complementCount = corpusData.getComplementMaxNumber();
                    jsonWriter.key("complementCount");
                    jsonWriter.value(complementCount);
                    jsonWriter.key("intitules");
                    jsonWriter.object();
                    jsonWriter.key("corpus");
                    jsonWriter.value(LibelleUtils.getLibelleString(corpusData, CorpusData.INTITULECORPUS_FIELDNUMBER, langUi, corpusName));
                    jsonWriter.key("fiche");
                    jsonWriter.value(LibelleUtils.getLibelleString(corpusData, CorpusData.INTITULEFICHE_FIELDNUMBER, langUi, corpusName));
                    for (int j = 1; j <= complementCount; j++) {
                        jsonWriter.key("complement_" + j);
                        jsonWriter.value(LibelleUtils.getLibelleString(corpusData, j, langUi, "complement_" + j));
                    }
                    jsonWriter.endObject();
                    jsonWriter.endObject();
                }
                jsonWriter.endArray();
            }
            jsonWriter.endObject();
            writeWarnings(jsonWriter);
            jsonWriter.endObject();
        } catch (JSONException je) {
            Throwable throwable = je.getCause();
            if (throwable == null) {
                throw new ShouldNotOccurException(je.getMessage(), je);
            } else {
                if (throwable instanceof IOException) {
                    throw ((IOException) throwable);
                } else {
                    throw new ShouldNotOccurException(throwable.getMessage(), (Exception) throwable);
                }
            }
        }
        dataAccess.close();
    }

}
