/* SctServer - Copyright (c) 2010-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import fr.exemole.sctserver.request.FicheFieldsFilter;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.json.JSONException;
import net.mapeadores.util.json.JSONWriter;
import net.scrutari.data.BaseData;
import net.scrutari.data.FicheData;
import net.scrutari.db.ScrutariDB;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.codes.CodeList;
import net.scrutari.db.tools.util.BaseCheck;


/**
 *
 * @author Vincent Calame
 */
public class FicheDataJsonProducer extends AbstractJsonProducer {

    private final ScrutariDB scrutariDB;
    private final CodeList ficheCodeList;
    private final FicheFieldsFilter ficheFieldsFilter;
    private final int langUi;

    public FicheDataJsonProducer(ScrutariDB scrutariDB, CodeList ficheCodeList, FicheFieldsFilter ficheFieldsFilter, int langUi) {
        this.scrutariDB = scrutariDB;
        this.ficheCodeList = ficheCodeList;
        this.ficheFieldsFilter = ficheFieldsFilter;
        this.langUi = langUi;
    }

    @Override
    public void writeJson(Appendable appendable) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        Map<Integer, BaseCheck> bufferMap = new HashMap<Integer, BaseCheck>();
        JSONWriter jsonWriter = new JSONWriter(appendable);
        try {
            jsonWriter.object();
            jsonWriter.key("ficheData");
            jsonWriter.object();
            int ficheCount = ficheCodeList.getCodeCount();
            jsonWriter.key("ficheCount");
            jsonWriter.value(ficheCount);
            if (ficheCount > 0) {
                jsonWriter.key("ficheArray");
                jsonWriter.array();
                for (int i = 0; i < ficheCount; i++) {
                    int codefiche = ficheCodeList.getCode(i);
                    FicheData ficheData = dataAccess.getFicheDataByCode(codefiche);
                    BaseCheck baseCheck = getBaseCheck(dataAccess, ficheData.getCodeBase(), bufferMap, langUi);
                    jsonWriter.object();
                    JsonUtils.addFicheData(jsonWriter, ficheData, ficheFieldsFilter, baseCheck, null, null);
                    jsonWriter.endObject();
                }
                jsonWriter.endArray();
            }
            jsonWriter.endObject();
            writeWarnings(jsonWriter);
            jsonWriter.endObject();
        } catch (JSONException je) {
            Throwable throwable = je.getCause();
            if (throwable == null) {
                throw new ShouldNotOccurException(je.getMessage(), je);
            } else {
                if (throwable instanceof IOException) {
                    throw ((IOException) throwable);
                } else {
                    throw new ShouldNotOccurException(throwable.getMessage(), (Exception) throwable);
                }
            }
        }
        dataAccess.close();
    }

    private BaseCheck getBaseCheck(DataAccess dataAccess, int codebase, Map<Integer, BaseCheck> bufferMap, int langUi) {
        BaseCheck baseCheck = bufferMap.get(codebase);
        if (baseCheck != null) {
            return baseCheck;
        }
        BaseData baseData = dataAccess.getBaseDataByCode(codebase);
        baseCheck = BaseCheck.check(baseData, langUi);
        bufferMap.put(codebase, baseCheck);
        return baseCheck;
    }

}
