/* SctServer - Copyright (c) 2010-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import fr.exemole.sctserver.request.FicheFieldsFilter;
import fr.exemole.sctserver.request.IntitulesFilter;
import fr.exemole.sctserver.request.MotcleFieldsFilter;
import java.io.IOException;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;
import net.mapeadores.util.attr.AttributeDef;
import net.mapeadores.util.attr.AttributeDefList;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.json.JSONException;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.text.LibelleHolder;
import net.scrutari.data.BaseData;
import net.scrutari.data.CorpusData;
import net.scrutari.data.FicheData;
import net.scrutari.data.MotcleData;
import net.scrutari.data.ScrutariAttributes;
import net.scrutari.db.ScrutariDB;
import net.scrutari.db.api.AttributeKeyCoder;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.tools.util.BaseCheck;
import net.scrutari.searchengine.QId;
import net.scrutari.searchengine.api.operands.search.QInfo;
import net.scrutari.searchengine.api.options.CorpusClasse;
import net.scrutari.searchengine.api.options.GlobalSearchOptions;
import net.scrutari.searchengine.api.options.SearchOptions;
import net.scrutari.searchengine.api.result.FicheSearchResult;
import net.scrutari.searchengine.api.result.FicheSearchResultGroup;
import net.scrutari.searchengine.api.result.FicheSearchResultInfo;
import net.scrutari.searchengine.api.result.FicheSearchSource;
import net.scrutari.searchengine.api.result.MotcleSearchResultInfo;


/**
 *
 * @author vic
 */
public class FicheSearchResultJsonProducer extends AbstractJsonProducer {

    private final ScrutariDB scrutariDB;
    private final FicheSearchResult searchResult;
    private final FicheFieldsFilter ficheFieldsFilter;
    private final MotcleFieldsFilter motcleFieldsFilter;
    private final IntitulesFilter intitulesFilter;
    private final int limit;
    private final GlobalSearchOptions globalSearchOptions;
    private int globalStartIndex = -1;

    public FicheSearchResultJsonProducer(FicheSearchResult searchResult, ScrutariDB scrutariDB, GlobalSearchOptions globalSearchOptions, FicheFieldsFilter ficheFieldsFilter, MotcleFieldsFilter motcleFieldsFilter, IntitulesFilter intitulesFilter, int limit, int start) {
        this.searchResult = searchResult;
        this.ficheFieldsFilter = ficheFieldsFilter;
        this.motcleFieldsFilter = motcleFieldsFilter;
        this.intitulesFilter = intitulesFilter;
        this.limit = limit;
        this.scrutariDB = scrutariDB;
        this.globalSearchOptions = globalSearchOptions;
        if (start > 0) {
            this.globalStartIndex = start - 1;
        }
    }

    @Override
    public void writeJson(Appendable appendable) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        AttributeKeyCoder attributeKeyCoder = scrutariDB.getAttributeKeyCoder();
        Map<Integer, BaseCheck> bufferMap = new HashMap<Integer, BaseCheck>();
        int classeCount = globalSearchOptions.getCorpusClasseCount();
        FicheSearchSource ficheSearchSource = searchResult.getFicheSearchSource();
        QInfo qInfo = ficheSearchSource.getQInfo();
        SearchOptions searchOptions = ficheSearchSource.getSearchOptions();
        boolean withGlobalStart = (globalStartIndex >= 0);
        boolean withIncrement = ((limit > 0) || (withGlobalStart));
        boolean limitStop = false;
        int ficheIncrement = 0;
        int langUi = searchOptions.getLangUi();
        int motcleCount = searchResult.getMotcleCount();
        Set<Integer> codecorpusSet = new LinkedHashSet<Integer>();
        JSONWriter jsonWriter = new JSONWriter(appendable);
        try {
            jsonWriter.object();
            jsonWriter.key("ficheSearchResult");
            jsonWriter.object();
            jsonWriter.key("q");
            jsonWriter.value(qInfo.getQ());
            QId qId = ficheSearchSource.getQId();
            if (qId != null) {
                jsonWriter.key("qId");
                jsonWriter.value(qId.toString());
            }
            jsonWriter.key("qLength");
            jsonWriter.value(qInfo.getQLength());
            jsonWriter.key("qSource");
            jsonWriter.value(qInfo.getQSource());
            if (langUi != LangInteger.UNDETERMINED) {
                jsonWriter.key("langUi");
                jsonWriter.value(LangInteger.toString(langUi));
            }
            jsonWriter.key("ficheCount");
            jsonWriter.value(searchResult.getFicheCount());
            jsonWriter.key("ficheMaximum");
            jsonWriter.value(searchResult.getFicheMaximum());
            jsonWriter.key("motcleCount");
            jsonWriter.value(motcleCount);
            jsonWriter.key("ficheGroupType");
            if (classeCount > 0) {
                jsonWriter.value("corpusclasse");
            } else {
                jsonWriter.value("none");
            }
            int groupCount = searchResult.getFicheSearchResultGroupCount();
            if (groupCount > 0) {
                jsonWriter.key("ficheGroupArray");
                jsonWriter.array();
                for (int i = 0; i < groupCount; i++) {
                    FicheSearchResultGroup ficheSearchResultGroup = searchResult.getFicheSearchResultGroup(i);
                    int ficheByGroupCount = ficheSearchResultGroup.getFicheSearchResultInfoCount();
                    int startIndex = 0;
                    if (withGlobalStart) {
                        if (ficheIncrement == 0) {
                            if (ficheByGroupCount < globalStartIndex) {
                                globalStartIndex = globalStartIndex - ficheByGroupCount;
                                continue;
                            } else {
                                startIndex = globalStartIndex;
                            }
                        }
                    }
                    jsonWriter.object();
                    if (classeCount > 0) {
                        jsonWriter.key("corpusClasse");
                        jsonWriter.object();
                        int corpusClasseRank = ficheSearchResultGroup.getCorpusClasseRank();
                        jsonWriter.key("rank");
                        jsonWriter.value(corpusClasseRank);
                        CorpusClasse corpusClasse = globalSearchOptions.getCorpusClasseByRank(corpusClasseRank);
                        if (corpusClasse != null) {
                            jsonWriter.key("name");
                            jsonWriter.value(corpusClasse.getName());
                            jsonWriter.key("intitule");
                            jsonWriter.value(corpusClasse.getLibelleString(langUi, true));
                        } else {
                            jsonWriter.key("name");
                            jsonWriter.value("unknown_rank_" + corpusClasseRank);
                            jsonWriter.key("intitule");
                            jsonWriter.key("## Unknown Rank : " + corpusClasseRank + " ##");
                        }
                        jsonWriter.endObject();
                    }
                    jsonWriter.key("ficheArray");
                    jsonWriter.array();
                    for (int j = startIndex; j < ficheByGroupCount; j++) {
                        FicheSearchResultInfo ficheSearchResultInfo = ficheSearchResultGroup.getFicheSearchResultInfo(j);
                        int codefiche = ficheSearchResultInfo.getCode();
                        FicheData ficheData = dataAccess.getFicheDataByCode(codefiche);
                        codecorpusSet.add(ficheData.getCodeCorpus());
                        BaseCheck baseCheck = getBaseCheck(dataAccess, ficheData.getCodeBase(), bufferMap, langUi);
                        jsonWriter.object();
                        JsonUtils.addFicheData(jsonWriter, ficheData, ficheFieldsFilter, baseCheck, ficheSearchResultInfo, attributeKeyCoder);
                        int ficheMotcleCount = ficheSearchResultInfo.getMotcleCount();
                        if (ficheMotcleCount > 0) {
                            jsonWriter.key("codemotcleArray");
                            jsonWriter.array();
                            for (int k = 0; k < ficheMotcleCount; k++) {
                                jsonWriter.value(ficheSearchResultInfo.getMotcleCode(k));
                            }
                            jsonWriter.endArray();
                        }
                        jsonWriter.endObject();
                        if (withIncrement) {
                            ficheIncrement++;
                            if (ficheIncrement == limit) {
                                limitStop = true;
                                break;
                            }
                        }
                    }
                    jsonWriter.endArray();
                    jsonWriter.endObject();
                    if (limitStop) {
                        break;
                    }
                }
                jsonWriter.endArray();
            }
            if (motcleCount > 0) {
                jsonWriter.key("motcleArray");
                jsonWriter.array();
                for (int i = 0; i < motcleCount; i++) {
                    MotcleSearchResultInfo motcleSearchResultInfo = searchResult.getMotcleResult(i);
                    MotcleData motcleData = dataAccess.getMotcleDataByCode(motcleSearchResultInfo.getCode());
                    jsonWriter.object();
                    JsonUtils.addMotcleData(jsonWriter, motcleData, motcleFieldsFilter, motcleSearchResultInfo);
                    jsonWriter.endObject();
                }
                jsonWriter.endArray();
            }
            if (intitulesFilter != null) {
                boolean withComplement = (intitulesFilter.isWithComplement() && ((ficheFieldsFilter.isWithComplements()) || (ficheFieldsFilter.isWithMComplements())));
                boolean withCorpus = intitulesFilter.isWithCorpus();
                if (withCorpus || withComplement) {
                    jsonWriter.key("corpusIntituleArray");
                    jsonWriter.array();
                    for (int codecorpus : codecorpusSet) {
                        CorpusData corpusData = dataAccess.getCorpusDataByCode(codecorpus);
                        if (corpusData.getLangDataCount() == 0) {
                            continue;
                        }
                        CorpusData.CorpusLangData corpusLangData = corpusData.getLangDataByLang(langUi);
                        if (corpusLangData == null) {
                            corpusLangData = corpusData.getLangDataByIndex(0);
                        }
                        jsonWriter.object();
                        jsonWriter.key("codecorpus");
                        jsonWriter.value(codecorpus);
                        if (withCorpus) {
                            jsonWriter.key("corpus");
                            jsonWriter.value(corpusLangData.getLibelleString(CorpusData.INTITULECORPUS_FIELDNUMBER));
                            jsonWriter.key("fiche");
                            jsonWriter.value(corpusLangData.getLibelleString(CorpusData.INTITULEFICHE_FIELDNUMBER));
                        }
                        if (withComplement) {
                            int compMaxNumber = corpusData.getComplementMaxNumber();
                            for (int num = 1; num <= compMaxNumber; num++) {
                                String lib = corpusLangData.getLibelleString(num);
                                if (lib != null) {
                                    jsonWriter.key("complement_" + num);
                                    jsonWriter.value(lib);
                                }
                            }
                        }
                        jsonWriter.endObject();
                    }
                    jsonWriter.endArray();
                }
            }
            if (ficheFieldsFilter.getMAttributesFilter() != FicheFieldsFilter.NONE_FILTER) {
                jsonWriter.key("attributes");
                jsonWriter.object();
                addAttributeDefList(jsonWriter, attributeKeyCoder, AttributeKeyCoder.PRIMARY_GROUP, langUi);
                addAttributeDefList(jsonWriter, attributeKeyCoder, AttributeKeyCoder.SECONDARY_GROUP, langUi);
                jsonWriter.endObject();
            }
            jsonWriter.endObject();
            writeWarnings(jsonWriter);
            jsonWriter.endObject();
        } catch (JSONException je) {
            Throwable throwable = je.getCause();
            if (throwable == null) {
                throw new ShouldNotOccurException(je.getMessage(), je);
            } else {
                if (throwable instanceof IOException) {
                    throw ((IOException) throwable);
                } else {
                    throw new ShouldNotOccurException(throwable.getMessage(), (Exception) throwable);
                }
            }
        }
        dataAccess.close();
    }

    private void addAttributeDefList(JSONWriter jsonWriter, AttributeKeyCoder attributeKeyCoder, String name, int langUi) throws JSONException {
        AttributeDefList attributeDefList = attributeKeyCoder.getAttributeDefList(name);
        int count = attributeDefList.getAttributeDefCount();
        if (count == 0) {
            return;
        }
        jsonWriter.key(name);
        jsonWriter.array();
        for (int i = 0; i < count; i++) {
            AttributeDef attributeDef = attributeDefList.getAttributeDef(i);
            jsonWriter.object();
            jsonWriter.key("key");
            jsonWriter.value(attributeDef.getAttributeKey().toString());
            String type = attributeDef.getFormatType();
            if (type.isEmpty()) {
                type = ScrutariAttributes.INLINE_TYPE;
            }
            jsonWriter.key("type");
            jsonWriter.value(type);
            LibelleHolder libelleHolder = attributeDef.getLibelleHolder();
            jsonWriter.key("lib");
            jsonWriter.value(libelleHolder.getLibelleString(langUi, true));
            jsonWriter.endObject();
        }
        jsonWriter.endArray();
    }

    private BaseCheck getBaseCheck(DataAccess dataAccess, int codebase, Map<Integer, BaseCheck> bufferMap, int langUi) {
        BaseCheck baseCheck = bufferMap.get(codebase);
        if (baseCheck != null) {
            return baseCheck;
        }
        BaseData baseData = dataAccess.getBaseDataByCode(codebase);
        baseCheck = BaseCheck.check(baseData, langUi);
        bufferMap.put(codebase, baseCheck);
        return baseCheck;
    }

}
