/* SctServer - Copyright (c) 2010-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import fr.exemole.sctserver.request.FicheFieldsFilter;
import fr.exemole.sctserver.request.MotcleFieldsFilter;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.AttributeDef;
import net.mapeadores.util.attr.AttributeHolder;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.json.JSONException;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.text.Libelle;
import net.scrutari.data.FicheData;
import net.scrutari.data.MotcleData;
import net.scrutari.data.ScrutariAttributes;
import net.scrutari.datauri.FicheURI;
import net.scrutari.datauri.MotcleURI;
import net.scrutari.db.api.AttributeKeyCoder;
import net.scrutari.db.tools.util.BaseCheck;
import net.scrutari.lexie.AlineaInteger;
import net.scrutari.searchengine.api.result.FicheSearchResultInfo;
import net.scrutari.searchengine.api.result.InAlineaOccurrenceList;
import net.scrutari.searchengine.api.result.InLangOccurrenceList;
import net.scrutari.searchengine.api.result.MotcleSearchResultInfo;
import net.scrutari.searchengine.api.result.SearchTokenOccurrence;
import net.scrutari.searchengine.api.result.SearchTokenOccurrenceList;


/**
 *
 * @author Vincent Calame
 */
public class JsonUtils {

    public static void writeMLib(JSONWriter jsonWriter, String lib, SearchTokenOccurrenceList positionList) throws JSONException {
        if (positionList == null) {
            jsonWriter.array();
            jsonWriter.value(lib);
            jsonWriter.endArray();
            return;
        }
        PositionListBuilder builder = new PositionListBuilder(jsonWriter, lib, positionList);
        builder.buildArray();
    }

    public static void addMotcleData(JSONWriter jsonWriter, MotcleData motcleData, MotcleFieldsFilter motcleFieldsFilter, MotcleSearchResultInfo motcleSearchResultInfo) throws JSONException {
        jsonWriter.key("codemotcle");
        jsonWriter.value(motcleData.getCodeMotcle());
        if (motcleFieldsFilter.isWithCodeThesaurus()) {
            jsonWriter.key("codethesaurus");
            jsonWriter.value(motcleData.getCodeThesaurus());
        }
        if (motcleFieldsFilter.isWithCodeBase()) {
            jsonWriter.key("codebase");
            jsonWriter.value(motcleData.getCodeBase());
        }
        if (motcleFieldsFilter.isWithBaseName()) {
            MotcleURI motcleURI = motcleData.getMotcleURI();
            jsonWriter.key("basename");
            jsonWriter.value(motcleURI.getThesaurusURI().getBaseURI().getBaseName());
        }
        if (motcleFieldsFilter.isWithThesaurusName()) {
            MotcleURI motcleURI = motcleData.getMotcleURI();
            jsonWriter.key("thesaurusname");
            jsonWriter.value(motcleURI.getThesaurusURI().getThesaurusName());
        }
        if (motcleFieldsFilter.isWithMotcleId()) {
            MotcleURI motcleURI = motcleData.getMotcleURI();
            jsonWriter.key("motcleid");
            jsonWriter.value(motcleURI.getMotcleId());
        }
        if (motcleFieldsFilter.isWithLibelles()) {
            int libcount = motcleData.getLibelleCount();
            if (libcount > 0) {
                jsonWriter.key("libelleArray");
                jsonWriter.array();
                for (int i = 0; i < libcount; i++) {
                    jsonWriter.object();
                    Libelle lib = motcleData.getLibelleByIndex(i);
                    jsonWriter.key("lang");
                    jsonWriter.value(LangInteger.toString(lib.getLangInteger()));
                    jsonWriter.key("lib");
                    jsonWriter.value(lib.getLibelleString());
                    jsonWriter.endObject();
                }
                jsonWriter.endArray();
            }
        }
        if (motcleFieldsFilter.isWithMLibelles()) {
            int libcount = motcleData.getLibelleCount();
            if (libcount > 0) {
                boolean withDefault = motcleFieldsFilter.isWithLibellesDefault();
                boolean inclus = false;
                for (int i = 0; i < libcount; i++) {
                    Libelle lib = motcleData.getLibelleByIndex(i);
                    int langInteger = lib.getLangInteger();
                    InLangOccurrenceList occurrenceList = motcleSearchResultInfo.getInLangOccurrenceListByLang(langInteger);
                    if ((!withDefault) && (occurrenceList == null)) {
                        continue;
                    }
                    if (!inclus) {
                        jsonWriter.key("mlibelleArray");
                        jsonWriter.array();
                        inclus = true;
                    }
                    jsonWriter.object();
                    jsonWriter.key("lang");
                    jsonWriter.value(LangInteger.toString(langInteger));
                    jsonWriter.key("mlib");
                    writeMLib(jsonWriter, lib.getLibelleString(), occurrenceList);
                    jsonWriter.endObject();
                }
                if (inclus) {
                    jsonWriter.endArray();
                }
            }
        }
    }

    public static void addFicheData(JSONWriter jsonWriter, FicheData ficheData, FicheFieldsFilter ficheFieldsFilter, BaseCheck baseCheck, FicheSearchResultInfo info, AttributeKeyCoder attributeKeyCoder) throws JSONException {
        jsonWriter.key("codefiche");
        jsonWriter.value(ficheData.getCodeFiche());
        if (ficheFieldsFilter.isWithCodeCorpus()) {
            jsonWriter.key("codecorpus");
            jsonWriter.value(ficheData.getCodeCorpus());
        }
        if (ficheFieldsFilter.isWithCodeBase()) {
            jsonWriter.key("codebase");
            jsonWriter.value(ficheData.getCodeBase());
        }
        if (ficheFieldsFilter.isWithBaseName()) {
            FicheURI ficheURI = ficheData.getFicheURI();
            jsonWriter.key("basename");
            jsonWriter.value(ficheURI.getCorpusURI().getBaseURI().getBaseName());
        }
        if (ficheFieldsFilter.isWithCorpusName()) {
            FicheURI ficheURI = ficheData.getFicheURI();
            jsonWriter.key("corpusname");
            jsonWriter.value(ficheURI.getCorpusURI().getCorpusName());
        }
        if (ficheFieldsFilter.isWithFicheId()) {
            FicheURI ficheURI = ficheData.getFicheURI();
            jsonWriter.key("ficheid");
            jsonWriter.value(ficheURI.getFicheId());
        }
        if (ficheFieldsFilter.isWithTitre()) {
            String titre = ficheData.getTitre();
            if (titre != null) {
                jsonWriter.key("titre");
                jsonWriter.value(ficheData.getTitre());
            }
        }
        if (ficheFieldsFilter.isWithSoustitre()) {
            String soustitre = ficheData.getSoustitre();
            if (soustitre != null) {
                jsonWriter.key("soustitre");
                jsonWriter.value(soustitre);
            }
        }
        if (ficheFieldsFilter.isWithHref()) {
            String href = ficheData.getHref(baseCheck.getCheckedLangUi());
            if (href != null) {
                jsonWriter.key("href");
                jsonWriter.value(href);
            }
        }
        if (ficheFieldsFilter.isWithLang()) {
            int lang = ficheData.getLangInteger();
            if (lang != LangInteger.UNDETERMINED) {
                jsonWriter.key("lang");
                jsonWriter.value(LangInteger.toString(lang));
            }
        }
        if (ficheFieldsFilter.isWithAnnee()) {
            int dateInteger = ficheData.getDateInteger();
            if (dateInteger != DateInteger.NULL) {
                jsonWriter.key("annee");
                jsonWriter.value(DateInteger.getAnnee(dateInteger));
            }
        }
        if (ficheFieldsFilter.isWithFicheIcon()) {
            String ficheIcon = ficheData.getFicheIcon();
            if (ficheIcon != null) {
                jsonWriter.key("ficheicon");
                jsonWriter.value(ficheIcon);
            }
        }
        if (ficheFieldsFilter.isWithIcon()) {
            String icon = ficheData.getFicheIcon();
            if (icon == null) {
                icon = baseCheck.getBaseIcon();
            }
            if (icon != null) {
                jsonWriter.key("icon");
                jsonWriter.value(icon);
            }
        }
        if (ficheFieldsFilter.isWithGeo()) {
            String latitude = ficheData.getLatitude();
            if (latitude != null) {
                jsonWriter.key("lat");
                jsonWriter.value(latitude);
                jsonWriter.key("lon");
                jsonWriter.value(ficheData.getLongitude());
            }
        }
        if (ficheFieldsFilter.isWithComplements()) {
            int maxNumber = ficheData.getComplementMaxNumber();
            if (maxNumber > 0) {
                boolean inclus = false;
                for (int num = 1; num <= maxNumber; num++) {
                    String comp = ficheData.getComplementByNumber(num);
                    if (comp == null) {
                        continue;
                    }
                    if (!inclus) {
                        jsonWriter.key("complementArray");
                        jsonWriter.array();
                        inclus = true;
                    }
                    jsonWriter.object();
                    jsonWriter.key("num");
                    jsonWriter.value(num);
                    jsonWriter.key("comp");
                    jsonWriter.value(comp);
                    jsonWriter.endObject();
                }
                if (inclus) {
                    jsonWriter.endArray();
                }
            }
        }
        if (info == null) {
            return;
        }
        Map<Integer, InAlineaOccurrenceList> map = new HashMap<Integer, InAlineaOccurrenceList>();
        Set<Integer> alineaRootCodeSet = null;
        int occurrenceListCount = info.getInAlineaOccurrenceListCount();
        for (int i = 0; i < occurrenceListCount; i++) {
            InAlineaOccurrenceList occurrenceList = info.getInAlineaOccurrenceList(i);
            int alineaInteger = occurrenceList.getAlineaInteger();
            map.put(alineaInteger, occurrenceList);
            if (AlineaInteger.isAttributeAlineaInteger(alineaInteger)) {
                int alineaRootCode = AlineaInteger.toAlineaRootCode(alineaInteger);
                if (alineaRootCodeSet == null) {
                    alineaRootCodeSet = new HashSet<Integer>();
                }
                alineaRootCodeSet.add(alineaRootCode);
            }
        }
        if (ficheFieldsFilter.isWithMTitre()) {
            String titre = ficheData.getTitre();
            if (titre != null) {
                jsonWriter.key("mtitre");
                writeMLib(jsonWriter, titre, map.get(AlineaInteger.TITRE));
            }
        }
        if (ficheFieldsFilter.isWithMSoustitre()) {
            String soustitre = ficheData.getSoustitre();
            if (soustitre != null) {
                jsonWriter.key("msoustitre");
                writeMLib(jsonWriter, soustitre, map.get(AlineaInteger.SOUSTITRE));
            }
        }
        if (ficheFieldsFilter.isWithMComplements()) {
            boolean all = ficheFieldsFilter.isAllMComplements();
            int maxNumber = ficheData.getComplementMaxNumber();
            if (maxNumber > 0) {
                boolean inclus = false;
                for (int num = 1; num <= maxNumber; num++) {
                    String comp = ficheData.getComplementByNumber(num);
                    if (comp == null) {
                        continue;
                    }
                    SearchTokenOccurrenceList positionList = map.get(AlineaInteger.getComplementAlineaInteger(num));
                    if ((positionList == null) && (!all)) {
                        continue;
                    }
                    if (!inclus) {
                        jsonWriter.key("mcomplementArray");
                        jsonWriter.array();
                        inclus = true;
                    }
                    jsonWriter.object();
                    jsonWriter.key("num");
                    jsonWriter.value(num);
                    jsonWriter.key("mcomp");
                    writeMLib(jsonWriter, comp, positionList);
                    jsonWriter.endObject();
                }
                if (inclus) {
                    jsonWriter.endArray();
                }
            }
        }
        short mAttributesFilter = ficheFieldsFilter.getMAttributesFilter();
        if (mAttributesFilter != FicheFieldsFilter.NONE_FILTER) {
            AttributeHolder attributeHolder = ficheData.getAttributHolder();
            int attributeCount = attributeHolder.getAttributeCount();
            boolean inclus = false;
            for (int i = 0; i < attributeCount; i++) {
                Attribute attribute = attributeHolder.getAttributeByIndex(i);
                AttributeKey attributeKey = attribute.getAttributeKey();
                int alineaRootCode = attributeKeyCoder.getAlineaRootCode(attributeKey);
                if (alineaRootCode == -1) {
                    continue;
                }
                AttributeDef attributeDef = attributeKeyCoder.getAttributeDef(attributeKey);
                boolean isNotConcerned = ((alineaRootCodeSet == null) || (!alineaRootCodeSet.contains(alineaRootCode)));
                boolean isBlock = (attributeDef.getFormatType().equals(ScrutariAttributes.BLOCK_TYPE));
                if ((isBlock) && (isNotConcerned)) {
                    continue;
                }
                boolean stop = false;
                switch (mAttributesFilter) {
                    case FicheFieldsFilter.CONCERNED_FILTER:
                        if (isNotConcerned) {
                            stop = true;
                        }
                        break;
                    case FicheFieldsFilter.PRIMARY_FILTER:
                        if (AlineaInteger.getAttributeCategory(alineaRootCode) == AlineaInteger.SECONDARYATTRIBUTE_CATEGORY) {
                            if (isNotConcerned) {
                                stop = true;
                            }
                        }
                        break;
                }
                if (stop) {
                    continue;
                }
                if (!inclus) {
                    jsonWriter.key("mattrMap");
                    jsonWriter.object();
                    inclus = true;
                }
                jsonWriter.key(attributeKey.toString());
                int valCount = attribute.getValueCount();
                jsonWriter.array();
                for (int j = 0; j < valCount; j++) {
                    String value = attribute.getValue(j);
                    int alineaInteger = AlineaInteger.getValueAlineaInteger(alineaRootCode, j);
                    SearchTokenOccurrenceList positionList = map.get(alineaInteger);
                    if ((positionList != null) || (!isBlock)) {
                        writeMLib(jsonWriter, value, positionList);
                    }
                }
                jsonWriter.endArray();
            }
            if (inclus) {
                jsonWriter.endObject();
            }
        }
        if (ficheFieldsFilter.isWithScore()) {
            jsonWriter.key("score");
            int[] array = info.getScoreArray();
            int scoreLength = array.length;
            jsonWriter.array();
            for (int i = 0; i < scoreLength; i++) {
                jsonWriter.value(array[i]);
            }
            jsonWriter.endArray();
        }
    }


    private static class PositionListBuilder {

        private JSONWriter jsonWriter;
        SearchTokenOccurrenceList positionList;
        StringBuilder buf;
        String s;
        int openIndex;
        int closeIndex;
        int currentPosition;

        /*
         * Constructeur unique.
         */
        PositionListBuilder(JSONWriter jsonWriter, String s, SearchTokenOccurrenceList positionList) {
            this.jsonWriter = jsonWriter;
            this.positionList = positionList;
            this.s = s;
            currentPosition = 0;
            SearchTokenOccurrence firstTokenOccurrence = positionList.getSearchTokenOccurrence(0);
            openIndex = firstTokenOccurrence.getBeginIndex();
            closeIndex = openIndex + firstTokenOccurrence.getLength() - 1;
            buf = new StringBuilder();
        }

        /*
         * Lancement de l'écriture.
         */
        void buildArray() throws JSONException {
            jsonWriter.array();
            int length = s.length();
            boolean onObject = false;
            for (int i = 0; i < length; i++) {
                if (i == openIndex) {
                    flushBuffer(false);
                    jsonWriter.object();
                    jsonWriter.key("i");
                    jsonWriter.value(positionList.getSearchTokenOccurrence(currentPosition).getOperandNumber());
                    jsonWriter.key("s");
                    onObject = true;
                }
                char carac = s.charAt(i);
                buf.append(carac);
                if (i == closeIndex) {
                    flushBuffer(true);
                    jsonWriter.endObject();
                    increment();
                    onObject = false;
                }
            }
            flushBuffer(false);
            if (onObject) {
                jsonWriter.endObject();
            }
            jsonWriter.endArray();
        }

        void flushBuffer(boolean withEmpty) throws JSONException {
            String bufString = buf.toString();
            if ((!withEmpty) && (bufString.length() == 0)) {
                return;
            }
            jsonWriter.value(bufString);
            buf = new StringBuilder();
        }

        /*
         * gère les éventuels recouvrements de position. Cela peut aller jusqu'à
         * générer un élément s vide (<s token-index=""/>) si une position est
         * complètement incluse dans une autre.
         */
        void increment() throws JSONException {
            if (currentPosition == (positionList.getSearchTokenOccurrenceCount() - 1)) {
                openIndex = -1;
                closeIndex = -1;
                return;
            }
            currentPosition++;
            SearchTokenOccurrence tokenOccurrence = positionList.getSearchTokenOccurrence(currentPosition);
            int beginIndex = tokenOccurrence.getBeginIndex();
            int endIndex = beginIndex + tokenOccurrence.getLength() - 1;
            if (beginIndex > closeIndex) {
                openIndex = beginIndex;
                closeIndex = endIndex;
                return;
            }
            openIndex = -1;
            jsonWriter.object();
            jsonWriter.key("i");
            jsonWriter.value(tokenOccurrence.getOperandNumber());
            if (endIndex <= closeIndex) {
                jsonWriter.endObject();
                increment();
            } else {
                jsonWriter.key("s");
                closeIndex = endIndex;
            }
        }

    }

}
