/* SctServer - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import java.io.IOException;
import java.util.Locale;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.json.JSONException;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerLocalisation;


/**
 *
 * @author Vincent Calame
 */
public class LangJsonProducer extends AbstractJsonProducer {

    private final Locale locale;
    private final int[] langIntegerArray;

    public LangJsonProducer(int langUi, int[] langIntegerArray) {
        if (langUi != LangInteger.UNDETERMINED) {
            this.locale = LangInteger.getLocale(langUi);
        } else {
            this.locale = Locale.getDefault();
        }
        this.langIntegerArray = langIntegerArray;
    }

    @Override
    public void writeJson(Appendable appendable) throws IOException {
        JSONWriter jsonWriter = new JSONWriter(appendable);
        try {
            jsonWriter.object();
            jsonWriter.key("langMap");
            jsonWriter.object();
            if (langIntegerArray != null) {
                int count = langIntegerArray.length;
                for (int i = 0; i < count; i++) {
                    int langInteger = langIntegerArray[i];
                    jsonWriter.key(LangInteger.toString(langInteger));
                    jsonWriter.value(LangIntegerLocalisation.getLangLibelle(langInteger, locale));
                }
            }
            jsonWriter.endObject();
            writeWarnings(jsonWriter);
            jsonWriter.endObject();
        } catch (JSONException je) {
            Throwable throwable = je.getCause();
            if (throwable == null) {
                throw new ShouldNotOccurException(je.getMessage(), je);
            } else {
                if (throwable instanceof IOException) {
                    throw ((IOException) throwable);
                } else {
                    throw new ShouldNotOccurException(throwable.getMessage(), (Exception) throwable);
                }
            }
        }
    }

}
