/* SctServer  - Copyright (c) 2011-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import java.io.IOException;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.json.JSONException;
import net.mapeadores.util.json.JSONWriter;
import net.scrutari.data.ThesaurusData;
import net.scrutari.datauri.ThesaurusURI;
import net.scrutari.db.ScrutariDB;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.tools.util.LibelleUtils;


/**
 *
 * @author Vincent Calame
 */
public class ThesaurusDataJsonProducer extends AbstractJsonProducer {

    private ScrutariDB scrutariDB;
    private int langUi;

    public ThesaurusDataJsonProducer(ScrutariDB scrutariDB, int langUi) {
        this.scrutariDB = scrutariDB;
        this.langUi = langUi;
    }

    public void writeJson(Appendable appendable) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        JSONWriter jsonWriter = new JSONWriter(appendable);
        try {
            jsonWriter.object();
            jsonWriter.key("thesaurusData");
            jsonWriter.object();
            int count = dataAccess.getThesaurusCount();
            jsonWriter.key("thesaurusCount");
            jsonWriter.value(count);
            if (count > 0) {
                jsonWriter.key("thesaurusArray");
                jsonWriter.array();
                for (int i = 0; i < count; i++) {
                    ThesaurusData thesaurusData = dataAccess.getThesaurusDataByIndex(i);
                    int code = thesaurusData.getCodeThesaurus();
                    jsonWriter.object();
                    jsonWriter.key("codethesaurus");
                    jsonWriter.value(code);
                    jsonWriter.key("codebase");
                    jsonWriter.value(thesaurusData.getCodeBase());
                    ThesaurusURI thesaurusURI = thesaurusData.getThesaurusURI();
                    String thesaurusName = thesaurusURI.getThesaurusName();
                    jsonWriter.key("thesaurusname");
                    jsonWriter.value(thesaurusName);
                    jsonWriter.key("intitules");
                    jsonWriter.object();
                    jsonWriter.key("thesaurus");
                    jsonWriter.value(LibelleUtils.getLibelleString(thesaurusData, langUi, thesaurusName));
                    jsonWriter.endObject();
                    jsonWriter.endObject();
                }
                jsonWriter.endArray();
            }
            jsonWriter.endObject();
            writeWarnings(jsonWriter);
            jsonWriter.endObject();
        } catch (JSONException je) {
            je.printStackTrace();
            Throwable throwable = je.getCause();
            if (throwable == null) {
                throw new ShouldNotOccurException(je.getMessage(), je);
            } else {
                if (throwable instanceof IOException) {
                    throw ((IOException) throwable);
                } else {
                    throw new ShouldNotOccurException(throwable.getMessage(), (Exception) throwable);
                }
            }
        }
        dataAccess.close();
    }

}
