/* SctServer - Copyright (c) 2008-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.simple.catalog;

import net.scrutari.datauri.BaseURI;
import net.scrutari.data.BaseData;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.ScrutariDB;
import net.scrutari.db.tools.util.LibelleUtils;
import fr.exemole.sctserver.request.simple.SimpleRequest;
import java.io.IOException;
import java.io.PrintWriter;


/**
 *
 * @author Vincent Calame
 */
public class BaseDataRequest extends SimpleRequest {

    private int langUi;
    private BaseData baseData;

    public BaseDataRequest(ScrutariDB scrutariDB, int langUi) {
        super(scrutariDB);
        this.langUi = langUi;
    }

    public BaseDataRequest(ScrutariDB scrutariDB, int langUi, BaseData baseData) {
        super(scrutariDB);
        this.langUi = langUi;
        this.baseData = baseData;
    }

    public void printResponse(PrintWriter pw) throws IOException {
        if (baseData != null) {
            printResponse(pw, baseData);
        } else {
            DataAccess dataAccess = scrutariDB.openDataAccess();
            for (int i = 0; i < dataAccess.getBaseCount(); i++) {
                BaseData currentBaseData = dataAccess.getBaseDataByIndex(i);
                printResponse(pw, currentBaseData);
            }
            dataAccess.close();
        }
    }

    private void printResponse(PrintWriter pw, BaseData currentBaseData) {
        int checkedLangInteger = currentBaseData.checkLangUi(langUi);
        BaseURI baseURI = currentBaseData.getBaseURI();
        String name = baseURI.getBaseName();
        pw.print("#b");
        pw.print('\n');
        pw.print("cdbdf=");
        pw.print(currentBaseData.getCodeBase());
        pw.print('\n');
        pw.print("auuid=");
        pw.print(baseURI.getAuthorityUUID());
        pw.print('\n');
        pw.print("nmbdf=");
        pw.print(baseURI.getBaseName());
        pw.print('\n');
        pw.print("cacrp=");
        for (int i = 0; i < currentBaseData.getCorpusCount(); i++) {
            if (i > 0) {
                pw.print(',');
            }
            pw.print(currentBaseData.getCodeCorpus(i));
        }
        pw.print('\n');
        pw.print("caths=");
        for (int i = 0; i < currentBaseData.getThesaurusCount(); i++) {
            if (i > 0) {
                pw.print(',');
            }
            pw.print(currentBaseData.getCodeThesaurus(i));
        }
        pw.print('\n');
        pw.print("bfico=");
        String baseIcon = currentBaseData.getBaseIcon(checkedLangInteger);
        if (baseIcon != null) {
            pw.print(baseIcon);
        }
        pw.print('\n');
        pw.print("insht");
        pw.print('=');
        pw.print(LibelleUtils.getLibelleString(currentBaseData, BaseData.INTITULE_SHORT, checkedLangInteger, name));
        pw.print('\n');
        pw.print("inlng");
        pw.print('=');
        pw.print(LibelleUtils.getLibelleString(currentBaseData, BaseData.INTITULE_LONG, checkedLangInteger, name));
        pw.print('\n');
    }

}
