/* SctServer - Copyright (c) 2008-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.simple.catalog;

import fr.exemole.sctserver.request.simple.SimpleRequest;
import java.io.IOException;
import java.io.PrintWriter;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.text.Libelle;
import net.scrutari.data.MotcleData;
import net.scrutari.data.ThesaurusData;
import net.scrutari.db.ScrutariDB;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.tools.util.DataUtils;
import net.scrutari.searchengine.api.result.FicheSearchResult;
import net.scrutari.searchengine.api.result.InLangOccurrenceList;
import net.scrutari.searchengine.api.result.MotcleSearchResultInfo;
import net.scrutari.searchengine.api.result.SearchTokenOccurrence;
import net.scrutari.searchengine.api.result.SearchTokenOccurrenceList;


/**
 *
 * @author Vincent Calame
 */
public class MotcleResultRequest extends SimpleRequest {

    private FicheSearchResult searchResult;
    private boolean decalageutf8;
    private int uiLangInteger;

    public MotcleResultRequest(ScrutariDB scrutariDB, FicheSearchResult searchResult) {
        super(scrutariDB);
        this.searchResult = searchResult;
    }

    public void setDecalageUTF8(boolean decalageutf8) {
        this.decalageutf8 = decalageutf8;
    }

    /**
     * Précise la langue de préférence
     */
    public void setUiLangInteger(int uiLangInteger) {
        this.uiLangInteger = uiLangInteger;
    }

    public void printResponse(PrintWriter pw) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        int count = searchResult.getMotcleCount();
        for (int i = 0; i < count; i++) {
            printMotcleResult(pw, searchResult.getMotcleResult(i), dataAccess);
        }
        dataAccess.close();
    }

    private void printMotcleResult(PrintWriter pw, MotcleSearchResultInfo motcleSearchResultInfo, DataAccess dataAccess) {
        int codemotcle = motcleSearchResultInfo.getCode();
        MotcleData motcleData = dataAccess.getMotcleDataByCode(codemotcle);
        int codethesaurus = motcleData.getCodeThesaurus();
        pw.print("#m");
        pw.print('\n');
        pw.print("cdmtc=");
        pw.print(codemotcle);
        pw.print('\n');
        pw.print("cdths=");
        pw.print(codethesaurus);
        pw.print('\n');
        ThesaurusData thesaurusData = dataAccess.getThesaurusDataByCode(codethesaurus);
        pw.print("cdbdf=");
        pw.print(thesaurusData.getCodeBase());
        pw.print('\n');
        String uilib = "";
        StringBuilder bufcodes = new StringBuilder();
        int libcount = motcleData.getLibelleCount();
        for (int i = 0; i < libcount; i++) {
            Libelle lib = motcleData.getLibelleByIndex(i);
            int langInteger = lib.getLangInteger();
            InLangOccurrenceList occurrenceList = motcleSearchResultInfo.getInLangOccurrenceListByLang(langInteger);
            if (occurrenceList != null) {
                if (bufcodes.length() > 0) {
                    bufcodes.append(',');
                }
                String langString = LangInteger.toString(langInteger);
                bufcodes.append(langString);
                if (decalageutf8) {
                    printWithDecalage(pw, lib, occurrenceList, langString);

                } else {
                    printWithoutDecalage(pw, lib, occurrenceList, langString);
                }
            } else if (langInteger == uiLangInteger) {
                uilib = lib.getLibelleString();
            }
        }
        pw.print("lglib=");
        pw.print(bufcodes.toString());
        pw.print('\n');
        pw.print("uilib=");
        pw.print(uilib);
        pw.print('\n');
    }

    private void printWithDecalage(PrintWriter pw, Libelle lib, SearchTokenOccurrenceList positionList, String langString) {
        pw.print("lib");
        pw.print(langString);
        pw.print('=');
        String libelle = lib.getLibelleString();
        int[] decalagelibelle = new int[libelle.length()];
        int decalage = 0;
        for (int j = 0; j < libelle.length(); j++) {
            char c = libelle.charAt(j);
            pw.print(c);
            decalage = decalage + DataUtils.getDecalageUTF8(c);
            decalagelibelle[j] = decalage;
        }
        pw.print('\n');
        for (int k = 0; k < positionList.getSearchTokenOccurrenceCount(); k++) {
            SearchTokenOccurrence tokenOccurrence = positionList.getSearchTokenOccurrence(k);
            int tokenIndex = tokenOccurrence.getOperandNumber() - 1;
            int positiondebut = tokenOccurrence.getBeginIndex();
            int decalagedebut = 0;
            if (positiondebut > 0) {
                decalagedebut = decalagelibelle[positiondebut - 1];
            }
            pw.print("p");
            pw.print(langString);
            if (tokenIndex < 10) {
                pw.print('0');
            }
            pw.print(tokenIndex);
            pw.print('=');
            pw.print(positiondebut + decalagedebut);
            pw.print('\n');
            int longueur = tokenOccurrence.getLength();
            longueur = longueur + decalagelibelle[positiondebut - 1 + longueur] - decalagedebut;
            pw.print("l");
            pw.print(langString);
            if (tokenIndex < 10) {
                pw.print('0');
            }
            pw.print(tokenIndex);
            pw.print('=');
            pw.print(longueur);
            pw.print('\n');
        }

    }

    private void printWithoutDecalage(PrintWriter pw, Libelle lib, SearchTokenOccurrenceList positionList, String langString) {
        pw.print("lib");
        pw.print(langString);
        pw.print('=');
        String libelle = lib.getLibelleString();
        pw.print(libelle);
        pw.print('\n');
        for (int k = 0; k < positionList.getSearchTokenOccurrenceCount(); k++) {
            SearchTokenOccurrence tokenOccurrence = positionList.getSearchTokenOccurrence(k);
            int tokenIndex = tokenOccurrence.getOperandNumber() -1;
            pw.print("p");
            pw.print(langString);
            if (tokenIndex < 10) {
                pw.print('0');
            }
            pw.print(tokenIndex);
            pw.print('=');
            pw.print(tokenOccurrence.getBeginIndex());
            pw.print('\n');
            pw.print("l");
            pw.print(langString);
            if (tokenIndex < 10) {
                pw.print('0');
            }
            pw.print(tokenIndex);
            pw.print('=');
            pw.print(tokenOccurrence.getLength());
            pw.print('\n');
        }
    }

}
