/* SctServer - Copyright (c) 2008-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.simple.catalog;

import net.scrutari.datauri.ThesaurusURI;
import net.scrutari.data.BaseData;
import net.scrutari.data.ThesaurusData;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.ScrutariDB;
import net.scrutari.db.tools.util.LibelleUtils;
import fr.exemole.sctserver.request.simple.SimpleRequest;
import java.io.IOException;
import java.io.PrintWriter;


/**
 *
 * @author Vincent Calame
 */
public class ThesaurusDataRequest extends SimpleRequest {

    private int langUi;
    private ThesaurusData thesaurusData;

    public ThesaurusDataRequest(ScrutariDB scrutariDB, int langUi) {
        super(scrutariDB);
        this.langUi = langUi;
    }

    public ThesaurusDataRequest(ScrutariDB scrutariDB, int langUi, ThesaurusData thesaurusData) {
        super(scrutariDB);
        this.langUi = langUi;
        this.thesaurusData = thesaurusData;
    }

    public void printResponse(PrintWriter pw) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        if (thesaurusData != null) {
            printThesaurus(pw, thesaurusData, dataAccess);
        } else {
            for (int i = 0; i < dataAccess.getThesaurusCount(); i++) {
                ThesaurusData currentThesaurusData = dataAccess.getThesaurusDataByIndex(i);
                printThesaurus(pw, currentThesaurusData, dataAccess);
            }
        }
        dataAccess.close();
    }

    private void printThesaurus(PrintWriter pw, ThesaurusData currentThesaurusData, DataAccess dataAccess) {
        BaseData currentBaseData = dataAccess.getBaseDataByCode(currentThesaurusData.getCodeBase());
        int checkedLangInteger = currentBaseData.checkLangUi(langUi);
        int codeths = currentThesaurusData.getCodeThesaurus();
        ThesaurusURI thesaurusURI = currentThesaurusData.getThesaurusURI();
        pw.print("#t");
        pw.print('\n');
        pw.print("cdths=");
        pw.print(codeths);
        pw.print('\n');
        pw.print("cdbdf=");
        pw.print(currentThesaurusData.getCodeBase());
        pw.print('\n');
        pw.print("nmths=");
        pw.print(thesaurusURI.getThesaurusName());
        pw.print('\n');
        pw.print("inths=");
        pw.print(LibelleUtils.getLibelleString(currentThesaurusData, checkedLangInteger, thesaurusURI.getThesaurusName()));
        ThesaurusData.ThesaurusLangData thesaurusLangInfo = currentThesaurusData.getLangDataByLang(checkedLangInteger);
        pw.print('\n');
    }

}
