/* SctServer - Copyright (c) 2005-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.uri;

import net.scrutari.datauri.BaseURI;
import net.scrutari.datauri.CorpusURI;
import net.scrutari.datauri.FicheURI;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.datauri.MotcleURI;
import net.scrutari.datauri.ThesaurusURI;
import net.scrutari.datauri.URIParseException;
import net.scrutari.datauri.URIParser;
import net.scrutari.data.BaseData;
import net.scrutari.data.CorpusData;
import net.scrutari.data.FicheData;
import net.scrutari.data.MotcleData;
import net.scrutari.db.ScrutariDB;
import net.scrutari.db.api.DataAccess;
import javax.servlet.http.HttpServletResponse;
import java.io.PrintWriter;
import java.io.IOException;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.text.Libelle;


/**
 *
 * @author Vincent Calame
 */
public class URIRequest {

    public final static short NOMOREAVAILABLE_TYPE = -3;
    public final static short MALFORMED_TYPE = -2;
    public final static short NOTFOUND_TYPE = -1;
    public final static short FICHE_TYPE = 1;
    public final static short CORPUS_TYPE = 2;
    public final static short THESAURUS_TYPE = 3;
    public final static short MOTCLE_TYPE = 4;
    public final static short BASE_TYPE = 5;
    private short type;
    private int code;
    private Object DataObject;
    private ScrutariDB scrutariDB;
    private String param_uri;

    public URIRequest(String param_uri, ScrutariDB scrutariDB) {
        this.scrutariDB = scrutariDB;
        this.param_uri = param_uri;
    }

    public void printResponse(HttpServletResponse reponse) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        init(dataAccess);
        reponse.setContentType("text/xml;charset=UTF-8");
        PrintWriter pw = reponse.getWriter();
        pw.print("<?xml version='1.0' encoding='UTF-8'?>");
        switch (type) {
            case NOMOREAVAILABLE_TYPE:
                pw.print("<noresult cause=\"noMoreAvailable\"/>");
                break;
            case MALFORMED_TYPE:
                pw.print("<noresult cause=\"malformedURI\"/>");
                break;
            case NOTFOUND_TYPE:
                pw.print("<noresult cause=\"notfoundURI\"/>");
                break;
            case FICHE_TYPE:
                printFicheURI(pw, (FicheData) DataObject);
                break;
            case CORPUS_TYPE:
                printCorpusURI(pw, (CorpusData) DataObject);
                break;
            case MOTCLE_TYPE:
                printMotcleURI(pw, (MotcleData) DataObject);
                break;
            case BASE_TYPE:
                printBaseURI(pw, (BaseData) DataObject);
                break;
        }
        dataAccess.close();
    }

    private void init(DataAccess dataAccess) {
        try {
            ScrutariDataURI scrutariURI = URIParser.parse(param_uri);
            if (scrutariURI instanceof FicheURI) {
                type = FICHE_TYPE;
                code = dataAccess.getCode(scrutariURI);
                if (code < 0) {
                    type = NOTFOUND_TYPE;
                } else {
                    DataObject = dataAccess.getFicheDataByCode(code);
                    if (DataObject == null) {
                        type = NOMOREAVAILABLE_TYPE;
                    }
                }
            } else if (scrutariURI instanceof CorpusURI) {
                type = CORPUS_TYPE;
                code = dataAccess.getCode(scrutariURI);
                if (code < 0) {
                    type = NOTFOUND_TYPE;
                } else {
                    DataObject = dataAccess.getCorpusDataByCode(code);
                    if (DataObject == null) {
                        type = NOMOREAVAILABLE_TYPE;
                    }
                }
            } else if (scrutariURI instanceof MotcleURI) {
                type = MOTCLE_TYPE;
                code = dataAccess.getCode(scrutariURI);
                if (code < 0) {
                    type = NOTFOUND_TYPE;
                } else {
                    DataObject = dataAccess.getMotcleDataByCode(code);
                    if (DataObject == null) {
                        type = NOMOREAVAILABLE_TYPE;
                    }
                }
            } else if (scrutariURI instanceof ThesaurusURI) {
                type = THESAURUS_TYPE;
                code = dataAccess.getCode(scrutariURI);
                if (code < 0) {
                    type = NOTFOUND_TYPE;
                } else {
                    DataObject = dataAccess.getThesaurusDataByCode(code);
                    if (DataObject == null) {
                        type = NOMOREAVAILABLE_TYPE;
                    }
                }
            } else if (scrutariURI instanceof BaseURI) {
                type = BASE_TYPE;
                code = dataAccess.getCode(scrutariURI);
                if (code < 0) {
                    type = NOTFOUND_TYPE;
                } else {
                    DataObject = dataAccess.getBaseDataByCode(code);
                    if (DataObject == null) {
                        type = NOMOREAVAILABLE_TYPE;
                    }
                }
            } else {
                type = NOTFOUND_TYPE;
            }
        } catch (URIParseException uriParseException) {
            type = MALFORMED_TYPE;
        }
    }

    private void printCorpusURI(PrintWriter pw, CorpusData corpusData) {
        pw.print("<corpus>");
        pw.print("<count-Data fiche-count=\"");
        pw.print(String.valueOf(corpusData.getFicheCount()));
        pw.print("\" complement-count=\"");
        pw.print(String.valueOf(corpusData.getComplementMaxNumber()));
        pw.print("\"/>");
        int numbermax = corpusData.getComplementMaxNumber();
        int count = corpusData.getLangDataCount();
        for (int i = 0; i < count; i++) {
            CorpusData.CorpusLangData langData = corpusData.getLangDataByIndex(i);
            pw.print("<lang-Data lang=\"");
            LangInteger.appendString(pw, langData.getLangInteger());
            pw.print("\">");
            printTag("fiche-count", String.valueOf(langData.getFicheCount()), pw, true);
            printTag("intitule-corpus", langData.getLibelleString(CorpusData.INTITULECORPUS_FIELDNUMBER), pw, true);
            printTag("intitule-fiche", langData.getLibelleString(CorpusData.INTITULEFICHE_FIELDNUMBER), pw, true);
            for (int k = 1; k <= numbermax; k++) {
                printTag("intitule-complement", langData.getLibelleString(k), pw, true);
            }
            pw.print("</lang-Data>");
        }
        pw.print("</corpus>");
    }

    private void printFicheURI(PrintWriter pw, FicheData ficheData) {
        pw.print("<fiche>");
        printTag("titre", ficheData.getTitre(), pw, true);
        printTag("soustitre", ficheData.getSoustitre(), pw, true);
        printTag("href", ficheData.getRawHref(), pw, true);
        int maxnumber = ficheData.getComplementMaxNumber();
        if (maxnumber > 0) {
            for (int i = 1; i <= maxnumber; i++) {
                printTag("complement", ficheData.getComplementByNumber(i), pw, true);
            }
        }
        pw.print("</fiche>");
    }

    private void printMotcleURI(PrintWriter pw, MotcleData motcleData) {
        pw.print("<motcle>");
        int count = motcleData.getLibelleCount();
        for (int i = 0; i < count; i++) {
            Libelle libelle = motcleData.getLibelleByIndex(i);
            pw.print("<lang-Data lang=\"");
            LangInteger.appendString(pw, libelle.getLangInteger());
            pw.print("\">");
            printTag("lib", libelle.getLibelleString(), pw, true);
            pw.print("</lang-Data>");
        }
        pw.print("</motcle>");
    }

    private void printTag(String tagname, String value, PrintWriter pw, boolean emptyValue) {
        if ((value == null) || (value.length() == 0)) {
            if (emptyValue) {
                pw.print("<");
                pw.print(tagname);
                pw.print("/>");
            }
        } else {
            pw.print("<");
            pw.print(tagname);
            pw.print(">");
            XMLUtils.printEscape(pw, value);
            pw.print("</");
            pw.print(tagname);
            pw.print(">");
        }
    }

    private void printBaseURI(PrintWriter pw, BaseData baseData) {
        pw.print("<base>");
        int count = baseData.getLangDataCount();
        for (int i = 0; i < count; i++) {
            BaseData.BaseLangData langData = baseData.getLangDataByIndex(i);
            pw.print("<lang-Data lang=\"");
            LangInteger.appendString(pw, langData.getLangInteger());
            pw.print("\">");
            printTag("intitule-short", langData.getLibelleString(BaseData.INTITULE_SHORT), pw, true);
            printTag("intitule-long", langData.getLibelleString(BaseData.INTITULE_LONG), pw, true);
            pw.print("</lang-Data>");
        }
        pw.print("</base>");
    }

}
