/* SctServer - Copyright (c) 2005-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.xml;

import java.util.ArrayList;
import java.util.List;
import net.scrutari.data.BaseData;
import net.scrutari.data.CorpusData;
import net.scrutari.data.ThesaurusData;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.tools.util.LibelleUtils;


/**
 *
 * @author Vincent Calame
 */
public class LangInfo {

    private int langInteger;
    private int ficheCount = 0;
    private List<IntituleInfo> intituleInfoList = new ArrayList<IntituleInfo>();

    public LangInfo(int langInteger, DataAccess dataAccess) {
        this.langInteger = langInteger;
        for (int i = 0; i < dataAccess.getBaseCount(); i++) {
            BaseData baseData = dataAccess.getBaseDataByIndex(i);
            createIntituleInfo(LibelleUtils.getLibelleString(baseData, BaseData.INTITULE_SHORT, langInteger, null), "short", "base", baseData.getCodeBase());
            createIntituleInfo(LibelleUtils.getLibelleString(baseData, BaseData.INTITULE_LONG, langInteger, null), "long", "base", baseData.getCodeBase());
        }
        for (int i = 0; i < dataAccess.getCorpusCount(); i++) {
            CorpusData corpusData = dataAccess.getCorpusDataByIndex(i);
            ficheCount = ficheCount + corpusData.getFicheCountByLang(langInteger);
            createIntituleInfo(LibelleUtils.getLibelleString(corpusData, CorpusData.INTITULECORPUS_FIELDNUMBER, langInteger, null), "corpus", "corpus", corpusData.getCodeCorpus());
            createIntituleInfo(LibelleUtils.getLibelleString(corpusData, CorpusData.INTITULEFICHE_FIELDNUMBER, langInteger, null), "fiche", "corpus", corpusData.getCodeCorpus());
            for (int num = 1; num <= corpusData.getComplementMaxNumber(); num++) {
                createIntituleInfo(LibelleUtils.getLibelleString(corpusData, num, langInteger, null), corpusData.getCodeCorpus(), num);
            }
        }
        for (int i = 0; i < dataAccess.getThesaurusCount(); i++) {
            ThesaurusData thesaurusData = dataAccess.getThesaurusDataByIndex(i);
            createIntituleInfo(LibelleUtils.getLibelleString(thesaurusData, langInteger, null), "thesaurus", "thesaurus", thesaurusData.getCodeThesaurus());
        }
    }

    public int getIntituleInfoCount() {
        return intituleInfoList.size();
    }

    public IntituleInfo getIntituleInfo(int index) {
        return intituleInfoList.get(index);
    }

    public int getFicheCount() {
        return ficheCount;
    }

    private void createIntituleInfo(String intitule, String name, String codename, int code) {
        if (intitule == null) {
            return;
        }
        intituleInfoList.add(new IntituleInfo(name, intitule, code, codename));
    }

    private void createIntituleInfo(String intitule, int code, int complementNumber) {
        if (intitule == null) {
            return;
        }
        intituleInfoList.add(new IntituleInfo("complement", intitule, code, "corpus", complementNumber));
    }


    public class IntituleInfo {

        String name;
        String intitule;
        String codename;
        int complementNumber = 0;
        int code;

        IntituleInfo(String name, String intitule, int code, String codename) {
            this.name = name;
            this.intitule = intitule;
            this.code = code;
            this.codename = codename;
        }

        IntituleInfo(String name, String intitule, int code, String codename, int complementNumber) {
            this.name = name;
            this.intitule = intitule;
            this.code = code;
            this.codename = codename;
            this.complementNumber = complementNumber;
        }

        public String getName() {
            return name;
        }

        public int getCode() {
            return code;
        }

        public String getCodeName() {
            return codename;
        }

        public String getIntitule() {
            return intitule;
        }

        public boolean hasComplementNumber() {
            return (complementNumber != 0);
        }

        public int getComplementNumber() {
            return complementNumber;
        }

    }

}
