/* SctServer - Copyright (c) 2005-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.xml;

import fr.exemole.sctserver.api.ScrutariSession;
import java.io.IOException;
import java.io.PrintWriter;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.xml.XMLUtils;
import net.scrutari.data.MotcleData;
import net.scrutari.db.ScrutariDB;
import net.scrutari.db.api.DataAccess;
import net.scrutari.searchengine.QId;
import net.scrutari.searchengine.api.result.FicheSearchResult;
import net.scrutari.searchengine.api.result.InLangOccurrenceList;
import net.scrutari.searchengine.api.result.MotcleSearchResultInfo;


/**
 * Classe produisant l'arbre XML des mots-clés.
 *
 * @author Vincent Calame
 */
public class MotsclesRequest {

    private final FicheSearchResult searchResult;
    private final ScrutariDB scrutariDB;

    public MotsclesRequest(QId qId, ScrutariSession scrutariSession) throws XmlRequestException {
        if (qId == null) {
            throw new XmlRequestException(XmlRequestException.CLIENT_SOURCE, XmlRequestException.MISSING_SEARCHID_PARAMETER);
        }
        this.searchResult = scrutariSession.getSearch(qId);
        if (searchResult == null) {
            throw new XmlRequestException(XmlRequestException.CLIENT_SOURCE, XmlRequestException.WRONG_SEARCHID_PARAMETER, qId.toString());
        }
        this.scrutariDB = scrutariSession.getScrutariDB();
    }

    /**
     * Écrit l'arbre XML des mots-clés comme réponse à une requête de servlet.
     */
    public void printResponse(HttpServletResponse reponse) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        int tabulations = 0;
        reponse.setContentType("text/xml;charset=UTF-8");
        PrintWriter pw = reponse.getWriter();
        pw.print("<?xml version='1.0' encoding='UTF-8' standalone='no' ?>");
        XMLUtils.printTabs(pw, tabulations);
        pw.print("<!DOCTYPE motscles SYSTEM \"dtd/motscles.dtd\">");
        XMLUtils.printTabs(pw, tabulations);
        pw.print("<motscles id=\"motscles\" search-id=\"");
        QId qId = searchResult.getFicheSearchSource().getQId();
        if (qId != null) {
            pw.print(qId.toString());
        }
        pw.print("\" all-motcle-count=\"");
        pw.print(String.valueOf(searchResult.getMotcleCount()));
        pw.print("\"");
        pw.print(">");
        int count = searchResult.getMotcleCount();
        for (int i = 0; i < count; i++) {
            printMotcleResult(pw, searchResult.getMotcleResult(i), tabulations + 1, dataAccess);
        }
        XMLUtils.printTabs(pw, tabulations);
        pw.print("</motscles>");
        dataAccess.close();
    }

    /*
     * Écrit les Datarmations liées à un mot-clé d'un résultat. Méthode appelée par printResponse.
     */
    private void printMotcleResult(PrintWriter pw, MotcleSearchResultInfo motcleResult, int tabulations, DataAccess dataAccess) {
        int code = motcleResult.getCode();
        MotcleData motcleData = dataAccess.getMotcleDataByCode(code);
        XMLUtils.printTabs(pw, tabulations);
        pw.print("<motcle id=\"");
        pw.print(String.valueOf(code));
        pw.print("\" motcle-code=\"");
        pw.print(String.valueOf(code));
        pw.print("\" thesaurus-code=\"");
        pw.print(String.valueOf(motcleData.getCodeThesaurus()));
        pw.print("\" fiche-count=\"");
        pw.print("0");
        pw.print("\" fiche-maximum=\"");
        pw.print("0");
        pw.print("\"");
        pw.print(">");
        int libcount = motcleData.getLibelleCount();
        for (int i = 0; i < libcount; i++) {
            Libelle libelle = motcleData.getLibelleByIndex(i);
            int langInteger = libelle.getLangInteger();
            String strg_libelle = libelle.getLibelleString();
            InLangOccurrenceList occurrenceList = motcleResult.getInLangOccurrenceListByLang(langInteger);
            String strg_lang = XmlRequestUtils.getLangString(langInteger);
            XMLUtils.printTabs(pw, tabulations + 1);
            pw.print("<lib id=\"");
            pw.print(String.valueOf(code));
            pw.print("_");
            pw.print(strg_lang);
            pw.print("\" lang=\"");
            pw.print(strg_lang);
            pw.print("\" role=\"");
            if (occurrenceList == null) {
                pw.print("0");
            } else {
                pw.print("1");
            }
            pw.print("\">");
            XmlRequestUtils.printPositionList(pw, strg_libelle, occurrenceList);
            XMLUtils.printTabs(pw, tabulations + 1);
            pw.print("</lib>");
        }
        XMLUtils.printTabs(pw, tabulations);
        pw.print("</motcle>");
    }

}
