/* SctServer - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.collect;

import fr.exemole.sctserver.api.collect.ScrutariInfo;
import fr.exemole.sctserver.api.collect.ScrutariInfoList;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.primitives.DateIntegerException;
import net.mapeadores.util.text.StringUtils;
import net.scrutari.datauri.BaseURI;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.datauri.URIParseException;
import net.scrutari.datauri.URIParser;


/**
 *
 * @author Vincent Calame
 */
public class ScrutariInfoStorage {

    private ScrutariInfoStorage() {
    }

    public static String toString(ScrutariInfoList scrutariInfoList) {
        StringBuilder buf = new StringBuilder();
        int count = scrutariInfoList.getScrutariInfoCount();
        for (int i = 0; i < count; i++) {
            ScrutariInfo scrutariInfo = scrutariInfoList.getScrutariInfo(i);
            buf.append(scrutariInfo.getKey());
            buf.append('\n');
            buf.append(scrutariInfo.getUrl().toString());
            buf.append('\n');
            short state = scrutariInfo.getState();
            buf.append(CollectUtils.stateToString(state));
            buf.append('\n');
            int lastUpdate = scrutariInfo.getLastUpdate();
            if (lastUpdate == DateInteger.NULL) {
                buf.append("NULL");
            } else {
                buf.append(DateInteger.toString(lastUpdate));
            }
            buf.append('\n');
            BaseURI baseURI = scrutariInfo.getBaseURI();
            if (baseURI == null) {
                buf.append("NULL");
            } else {
                buf.append(baseURI.toURI().toString());
                buf.append('\n');
            }
            buf.append('\n');
        }
        return buf.toString();
    }

    public static List<ScrutariInfo> fromString(String s) {
        String[] tokens = StringUtils.getLineTokens(s, StringUtils.EMPTY_INCLUDE);
        int length = tokens.length;
        Parser parser = new Parser();
        List<ScrutariInfo> resultList = new ArrayList<ScrutariInfo>();
        for (int i = 0; i < length; i++) {
            String ligne = tokens[i];
            if (ligne.length() == 0) {
                ScrutariInfo scrutariInfo = parser.flush();
                if (scrutariInfo != null) {
                    resultList.add(scrutariInfo);
                }
            } else {
                parser.addLine(ligne);
            }
        }
        ScrutariInfo scrutariInfo = parser.flush();
        if (scrutariInfo != null) {
            resultList.add(scrutariInfo);
        }
        return resultList;
    }


    private static class Parser {

        private String key = null;
        private URL url = null;
        private short state = ScrutariInfo.UNKNOWN_STATE;
        private int lastUpdate = DateInteger.NULL;
        private BaseURI baseURI = null;
        private boolean error = false;
        private int current = 0;

        private ScrutariInfo flush() {
            if ((error) || (url == null)) {
                reinit();
                return null;
            }
            InternalScrutariInfo info = new InternalScrutariInfo();
            info.key = key;
            info.url = url;
            info.state = state;
            info.lastUpdate = lastUpdate;
            info.baseURI = baseURI;
            reinit();
            return info;
        }

        private void addLine(String line) {
            if (error) {
                return;
            }
            switch (current) {
                case 0:
                    key = line;
                    break;
                case 1:
                    try {
                        url = new URL(line);
                    } catch (MalformedURLException mue) {
                        error = true;
                    }
                    break;
                case 2:
                    try {
                        state = CollectUtils.stateToShort(line);
                    } catch (IllegalArgumentException iae) {
                        error = true;
                    }
                    break;
                case 3:
                    if (!line.equals("NULL")) {
                        try {
                            lastUpdate = DateInteger.parse(line);
                        } catch (DateIntegerException die) {
                            error = true;
                        }
                    }
                    break;
                case 4:
                    if (!line.equals("NULL")) {
                        try {
                            ScrutariDataURI uri = URIParser.parse(line);
                            if (uri instanceof BaseURI) {
                                baseURI = (BaseURI) uri;
                            } else {
                                error = true;
                            }
                        } catch (URIParseException upe) {
                            error = true;
                        }
                        break;
                    }

            }
            current++;
        }

        private void reinit() {
            key = null;
            url = null;
            state = ScrutariInfo.UNKNOWN_STATE;
            lastUpdate = DateInteger.NULL;
            baseURI = null;
            error = false;
            current = 0;
        }

    }
    
    private static class InternalScrutariInfo implements ScrutariInfo {
        
        private URL url;
        private short state = ScrutariInfo.UNKNOWN_STATE;
        private int lastUpdate= DateInteger.NULL;
        private BaseURI baseURI;
        private String key;

        private InternalScrutariInfo() {

        }

        public String getKey() {
            return key;
        }

        public URL getUrl() {
            return url;
        }

        public short getState() {
            return state;
        }

        public int getLastUpdate() {
            return lastUpdate;
        }

        public BaseURI getBaseURI() {
            return baseURI;
        }

    }

}
