/* SctServer - Copyright (c) 2012-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.conf.dom;

import fr.exemole.sctserver.ErrorUtils;
import fr.exemole.sctserver.SctServer;
import java.text.ParseException;
import net.mapeadores.util.attr.AttributeDef;
import net.mapeadores.util.attr.AttributeDefBuilder;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.logging.ProcessMessageHandler;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.LibelleUtils;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.ElementHandler;
import net.scrutari.data.ScrutariAttributes;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public class AttributesDOMReader {

    private Builder builder;
    private ProcessMessageHandler errorHandler;

    public AttributesDOMReader(Builder builder, ProcessMessageHandler errorHandler) {
        this.builder = builder;
        this.errorHandler = errorHandler;
    }

    public void readAttributes(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("group")) {
                    readGroup(element);
                } else {
                    ErrorUtils.unknownTagWarning(errorHandler, tagName);
                }
            }
        }
    }

    private void readGroup(Element element_xml) {
        String name = element_xml.getAttribute("name");
        if (name.length() == 0) {
            ErrorUtils.emptyAttributeError(errorHandler, "group", "name");
            return;
        }
        boolean start = builder.startGroup(name);
        if (!start) {
            return;
        }
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("attr-def")) {
                    String localKey = element.getAttribute("key").trim();
                    if (localKey.length() == 0) {
                        ErrorUtils.emptyAttributeError(errorHandler, "attr-def", "key");
                        continue;
                    }
                    String nameSpace = element.getAttribute("ns").trim();
                    if (nameSpace.length() == 0) {
                        ErrorUtils.emptyAttributeError(errorHandler, "attr-def", "ns");
                        continue;
                    }
                    AttributeKey attributeKey;
                    try {
                        attributeKey = ScrutariAttributes.toAttributeKey(nameSpace, localKey);
                    } catch (ParseException pe) {
                        errorHandler.addMessage(SctServer.SEVERE_CATEGORY, "malformedAttributeKey", nameSpace + ":" + localKey);
                        continue;
                    }
                    AttributeDefHandler attributeDefHandler = new AttributeDefHandler(attributeKey);
                    DOMUtils.readChildren(element, attributeDefHandler);
                    builder.addAttributeDef(attributeDefHandler.toAttributeDef());
                } else {
                    ErrorUtils.unknownTagWarning(errorHandler, tagName);
                }
            }
        }
    }


    private class AttributeDefHandler implements ElementHandler {

        private AttributeKey attributeKey;
        private AttributeDefBuilder builder;

        private AttributeDefHandler(AttributeKey attributeKey) {
            this.attributeKey = attributeKey;
            AttributeDef defaultAttributeKey = ScrutariAttributes.getDefaultAttributeDef(attributeKey);
            if (defaultAttributeKey != null) {
                builder = AttributeDefBuilder.derive(defaultAttributeKey);
            } else {
                builder = new AttributeDefBuilder(attributeKey);
            }
        }

        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("lib")) {
                try {
                    Libelle libelle = LibelleUtils.readLib(element);
                    if (libelle != null) {
                        builder.addLibelle(libelle);
                    }
                } catch (LangIntegerException lie) {
                    errorHandler.addMessage(SctServer.SEVERE_CATEGORY, "libLangError", lie.getLangInteger());
                }
            } else if (tagName.equals("format")) {
                String type = element.getAttribute("type");
                if (ScrutariAttributes.isValidFormatType(type)) {
                    builder.setFormatType(type);
                } else {
                    errorHandler.addMessage(SctServer.WARNING_CATEGORY, "unknownFormatType", type);
                }
            }
        }

        public AttributeDef toAttributeDef() {
            return builder.toAttributeDef();
        }

    }


    public static interface Builder {

        public boolean startGroup(String name);

        public void addAttributeDef(AttributeDef attributeDef);

    }

}
