/* SctServer - Copyright (c) 2009-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.conf.dom;

import fr.exemole.sctserver.ErrorUtils;
import fr.exemole.sctserver.SctServer;
import net.mapeadores.util.conditions.ConditionUtils;
import net.mapeadores.util.conditions.TextTest;
import net.mapeadores.util.logging.ProcessMessageHandler;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public class SearchLogFilterDOMReader {

    private ProcessMessageHandler errorHandler;
    private Builder builder;

    public SearchLogFilterDOMReader(Builder builder, ProcessMessageHandler errorHandler) {
        this.errorHandler = errorHandler;
        this.builder = builder;
    }

    public void readSearchLogFilter(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("deny")) {
                    addDeny(element);
                } else {
                    ErrorUtils.unknownTagWarning(errorHandler, tagName);
                }
            }
        }
    }

    private void addDeny(Element element_xml) {
        TextTest ipCondition = ConditionUtils.parseTextTest(element_xml.getAttribute("ip"));
        TextTest siteCondition = ConditionUtils.parseTextTest(element_xml.getAttribute("site"));
        TextTest refCondition = ConditionUtils.parseTextTest(element_xml.getAttribute("ref"));
        if ((ipCondition == null) && (refCondition == null) && (siteCondition == null)) {
            errorHandler.addMessage(SctServer.WARNING_CATEGORY, "noAttribute", "deny");
            return;
        }
        builder.addDeny(ipCondition, siteCondition, refCondition);
    }


    public static interface Builder {

        public void addDeny(TextTest ipCondition, TextTest siteCondition, TextTest refCondition);

    }

}
