/* SctServer - Copyright (c) 2009-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.conf.dom;

import fr.exemole.sctserver.ErrorUtils;
import fr.exemole.sctserver.SctServer;
import java.net.MalformedURLException;
import java.net.URL;
import net.mapeadores.util.logging.ProcessMessageHandler;
import net.mapeadores.util.text.UrlKey;
import net.mapeadores.util.xml.XMLUtils;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public class SourcesDOMReader {

    private ProcessMessageHandler errorHandler;
    private Builder builder;

    public SourcesDOMReader(Builder builder, ProcessMessageHandler errorHandler) {
        this.errorHandler = errorHandler;
        this.builder = builder;
    }

    public void readSources(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("info-url")) {
                    addInfoUrl(element);
                } else {
                    ErrorUtils.unknownTagWarning(errorHandler, tagName);
                }
            }
        }
    }

    public void addInfoUrl(Element element_xml) {
        String urlString = XMLUtils.getData(element_xml);
        if (urlString.length() == 0) {
            ErrorUtils.emptyElementError(errorHandler, "info-url");
            return;
        }
        try {
            URL url = new URL(urlString);
            UrlKey urlKey = new UrlKey(url);
            boolean done = builder.addInfoUrl(urlKey);
            if (!done) {
                errorHandler.addMessage(SctServer.WARNING_CATEGORY, "doublonUrl", urlString + " (" + urlKey.getKeyString() + "),");
            }
        } catch (MalformedURLException mue) {
            errorHandler.addMessage(SctServer.SEVERE_CATEGORY, "malformedUrl", urlString + " (" + mue.getMessage() + ")");
        }
    }


    public static interface Builder {

        public boolean addInfoUrl(UrlKey urlKey);

    }

}
