/* SctServer - Copyright (c) 2011-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.conf.dom;

import fr.exemole.sctserver.ErrorUtils;
import fr.exemole.sctserver.SctServer;
import net.mapeadores.util.logging.ProcessMessageHandler;
import net.mapeadores.util.xml.XMLUtils;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.datauri.URIParseException;
import net.scrutari.datauri.URIParser;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public class UriCodesDOMReader {

    private ProcessMessageHandler errorHandler;
    private Builder builder;

    public UriCodesDOMReader(Builder builder, ProcessMessageHandler errorHandler) {
        this.errorHandler = errorHandler;
        this.builder = builder;
    }

    public void readUriCodes(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("uri")) {
                    addUri(element);
                } else {
                    ErrorUtils.unknownTagWarning(errorHandler, tagName);
                }
            }
        }
    }

    private void addUri(Element element_xml) {
        String codeString = element_xml.getAttribute("code");
        if (codeString.length() == 0) {
            ErrorUtils.emptyAttributeError(errorHandler, "uri", "code");
            return;
        }
        int code = 0;
        try {
            code = Integer.parseInt(codeString);
            if (code < 1) {
                notPositiveIntegerAttributeError("uri", "code", codeString);
            }
        } catch (NumberFormatException nfe) {
            notPositiveIntegerAttributeError("uri", "code", codeString);
        }
        String uriString = XMLUtils.getData(element_xml);
        if (uriString.length() == 0) {
            ErrorUtils.emptyElementError(errorHandler, "uri");
            return;
        }
        try {
            ScrutariDataURI scrutariDataURI = URIParser.parse(uriString);
            builder.addScrutariDataURI(code, scrutariDataURI);
        } catch (URIParseException uripe) {
            ErrorUtils.uriSyntaxError(errorHandler, uriString);
            return;
        }
    }

    private void notPositiveIntegerAttributeError(String tagName, String attributeName, String attributeValue) {
        errorHandler.addMessage(SctServer.SEVERE_CATEGORY, "notPositiveIntegerAttribute", tagName + "@" + attributeName + "=" + attributeValue);
    }


    public static interface Builder {

        public void addScrutariDataURI(int code, ScrutariDataURI scrutariDataURI);

    }

}
