/* SctServer - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.feed;

import net.scrutari.data.CorpusData;
import net.scrutari.db.tools.util.LibelleUtils;
import net.scrutari.searchengine.api.options.CorpusClasse;
import net.scrutari.searchengine.api.options.GlobalSearchOptions;


/**
 *
 * @author Vincent Calame
 */
public class CorpusWrapper {

    private final CorpusData corpusData;
    private final String corpusLibelleString;
    private final String[] complementLibelleString;
    private final String categoryTerm;
    private final String categoryLabel;

    public CorpusWrapper(CorpusData corpusData, String corpusLibelleString, String[] complementLibelleString, String categoryTerm, String categoryLabel) {
        this.corpusData = corpusData;
        this.corpusLibelleString = corpusLibelleString;
        this.complementLibelleString = complementLibelleString;
        this.categoryTerm = categoryTerm;
        this.categoryLabel = categoryLabel;
    }

    public String getCorpusLibelleString() {
        return corpusLibelleString;
    }

    public String getComplementLibelle(int num) {
        return complementLibelleString[num - 1];
    }

    public CorpusData getCorpusData() {
        return corpusData;
    }

    public String getCategoryTerm() {
        return categoryTerm;
    }

    public String getCategoryLabel() {
        return categoryLabel;
    }

    public static CorpusWrapper wrapCorpus(CorpusData corpusData, int langUi, GlobalSearchOptions globalSearchOptions) {
        String categoryTerm = null;
        String categoryLabel = null;
        int rank = globalSearchOptions.getCorpusClasseRank(corpusData.getCodeCorpus());
        if (rank != 0) {
            CorpusClasse corpusClasse = globalSearchOptions.getCorpusClasseByRank(rank);
            categoryTerm = corpusClasse.getName();
            categoryLabel = LibelleUtils.getLibelleString(corpusClasse, langUi, null);
        }
        String corpusLibelleString = LibelleUtils.getLibelleString(corpusData, CorpusData.INTITULECORPUS_FIELDNUMBER, langUi, corpusData.getCorpusURI().getCorpusName());
        int complementCount = corpusData.getComplementMaxNumber();
        String[] complementLibelleStringArray = new String[complementCount];
        for (int i = 0; i < complementCount; i++) {
            int num = i + 1;
            complementLibelleStringArray[i] = LibelleUtils.getLibelleString(corpusData, num, langUi, "comp_" + num);
        }
        return new CorpusWrapper(corpusData, corpusLibelleString, complementLibelleStringArray, categoryTerm, categoryLabel);
    }

}
