/* SctServer - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.feed;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import net.scrutari.data.FicheData;
import net.scrutari.db.ScrutariDBName;
import net.scrutari.db.api.codes.ElementCodeList;
import net.scrutari.searchengine.api.result.FicheSearchResult;
import net.scrutari.searchengine.api.result.FicheSearchResultGroup;
import net.scrutari.searchengine.api.result.FicheSearchResultInfo;


/**
 *
 * @author Vincent Calame
 */
public class FicheFeedList implements ElementCodeList {

    private final int langUi;
    private final ScrutariDBName scrutariDBName;
    private final List<FicheWrapper> ficheWrapperList = new ArrayList<FicheWrapper>();

    public FicheFeedList(int langUi, ScrutariDBName scrutariDBName) {
        this.langUi = langUi;
        this.scrutariDBName = scrutariDBName;
    }

    public int getFicheCount() {
        return ficheWrapperList.size();
    }

    public FicheWrapper getFicheWrapper(int i) {
        return ficheWrapperList.get(i);
    }

    public int getLangUi() {
        return langUi;
    }

    public ScrutariDBName getScrutariDBName() {
        return scrutariDBName;
    }

    @Override
    public int getCodeCount() {
        return ficheWrapperList.size();
    }

    @Override
    public int getCode(int i) {
        return ficheWrapperList.get(i).getFicheData().getCodeFiche();
    }

    @Override
    public int getSubsetCode(int i) {
        return ficheWrapperList.get(i).getCorpusWrapper().getCorpusData().getCodeCorpus();
    }

    void addFicheData(FicheData ficheData, CorpusWrapper corpusWrapper, BaseWrapper baseWrapper) {
        ficheWrapperList.add(FicheWrapper.wrapFiche(ficheData, corpusWrapper, baseWrapper));
    }

    void update(FicheSearchResult ficheSearchResult) {
        Set<Integer> codeSet = new HashSet<Integer>();
        int count = ficheSearchResult.getFicheSearchResultGroupCount();
        for (int i = 0; i < count; i++) {
            FicheSearchResultGroup ficheSearchResultGroup = ficheSearchResult.getFicheSearchResultGroup(i);
            int ficheCount = ficheSearchResultGroup.getFicheSearchResultInfoCount();
            for (int j = 0; j < ficheCount; j++) {
                FicheSearchResultInfo info = ficheSearchResultGroup.getFicheSearchResultInfo(j);
                codeSet.add(info.getCode());
            }
        }
        for (Iterator<FicheWrapper> it = ficheWrapperList.iterator(); it.hasNext();) {
            FicheWrapper ficheWrapper = it.next();
            if (!codeSet.contains(ficheWrapper.getFicheData().getCodeFiche())) {
                it.remove();
            }
        }
    }

}
