/* SctServer - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.xml.feed;

import fr.exemole.sctserver.api.ScrutariSession;
import fr.exemole.sctserver.api.feed.SctFeedConstants;
import fr.exemole.sctserver.api.feed.SctFeedInfo;
import fr.exemole.sctserver.api.feed.SctFeedOptions;
import java.io.IOException;
import java.util.Comparator;
import java.util.SortedMap;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;
import net.mapeadores.util.xml.AppendableXMLWriter;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XMLWriter;
import net.mapeadores.util.xml.ns.NameSpace;
import net.scrutari.data.BaseData;
import net.scrutari.data.CorpusData;
import net.scrutari.data.FicheData;
import net.scrutari.datauri.BaseURI;
import net.scrutari.datauri.CorpusURI;
import net.scrutari.datauri.FicheURI;
import net.scrutari.datauri.tree.BaseNode;
import net.scrutari.datauri.tree.CorpusNode;
import net.scrutari.datauri.tree.FicheNode;
import net.scrutari.datauri.tree.URITree;
import net.scrutari.db.ScrutariDBName;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.tools.util.BaseCheck;
import net.scrutari.searchengine.api.options.CorpusClasse;
import net.scrutari.searchengine.api.options.GlobalSearchOptions;


/**
 *
 * @author Vincent Calame
 */
class ClassesEntriesXMLPart extends AbstractEntriesXMLPart {

    ClassesEntriesXMLPart(AppendableXMLWriter xmlWriter, ScrutariSession scrutariSession) {
        super(xmlWriter, scrutariSession);
    }

    @Override
    public void appendEntries(String feedId, SctFeedInfo sctFeedInfo, DataAccess dataAccess, SctFeedOptions feedOptions) throws IOException {
        int scrutariDBNameCount = sctFeedInfo.getLastScrutariDBNameCount();
        for (int i = 0; i < scrutariDBNameCount; i++) {
            ScrutariDBName scrutariDBName = sctFeedInfo.getLastScrutariDBName(i);
            appendFeedEntry(feedId, scrutariDBName, dataAccess, feedOptions);
        }
    }

    private void appendFeedEntry(String feedId, ScrutariDBName date, DataAccess dataAccess, SctFeedOptions feedOptions) throws IOException {
        URITree uriTree = scrutariSession.getSctServer().getSctServerStorage().getURITree(date);
        if (uriTree == null) {
            return;
        }
        EntryContent entryContent = new EntryContent(feedOptions, scrutariSession.getGlobalSearchOptions());
        entryContent.checkUriTree(dataAccess, uriTree);
        int ficheAddCount = entryContent.getFicheAddCount();
        if (ficheAddCount == 0) {
            return;
        }
        openTag("entry");
        addSimpleElement("title", getEntryTitle(date, ficheAddCount, feedOptions));
        addSimpleElement("updated", date.toGmtIsoFormat());
        addSimpleElement("id", feedId + "/" + date.toString());
        startOpenTag("content");
        addAttribute("type", "xhtml");
        endOpenTag();
        startOpenTag("div");
        XMLUtils.appendNameSpaceAttribute(this, NameSpace.XHTML_NAMESPACE);
        endOpenTag();
        entryContent.addContent(this);
        closeTag("div");
        closeTag("content");
        closeTag("entry");
    }

    private String getEntryTitle(ScrutariDBName scrutariDBName, int ficheAddCount, SctFeedOptions feedOptions) {
        StringBuilder buf = new StringBuilder();
        if ((feedOptions.getOptionMask() & SctFeedConstants.TECHNICALTITLE_OPTION) != 0) {
            buf.append("s=");
            buf.append(scrutariSession.getSctServerName());
            buf.append(";d=");
            buf.append(scrutariDBName.toGmtIsoFormat());
            buf.append(";f=");
            buf.append(ficheAddCount);
            return buf.toString();
        } else {
            buf.append(scrutariSession.getSctServerName());
            buf.append(" + ");
            buf.append(ficheAddCount);
        }
        return buf.toString();
    }


    private static class EntryContent {

        private SctFeedOptions feedOptions;
        private int ficheAddCount = 0;
        private SortedMap<Integer, FicheGroup> ficheGroupByClasseMap;
        private FicheGroup uniqueFicheGroup;
        private GlobalSearchOptions globalSearchOptions;

        private EntryContent(SctFeedOptions feedOptions, GlobalSearchOptions globalSearchOptions) {
            this.feedOptions = feedOptions;
            this.globalSearchOptions = globalSearchOptions;
            int corpusClasseCount = globalSearchOptions.getCorpusClasseCount();
            if (corpusClasseCount == 0) {
                uniqueFicheGroup = new FicheGroup(null);
            } else {
                ficheGroupByClasseMap = new TreeMap<Integer, FicheGroup>(new InverseComparator());
                for (int i = 0; i < corpusClasseCount; i++) {
                    CorpusClasse corpusClasse = globalSearchOptions.getCorpusClasse(i);
                    ficheGroupByClasseMap.put(corpusClasse.getRank(), new FicheGroup(corpusClasse));
                }
            }
        }

        private int getFicheAddCount() {
            return ficheAddCount;
        }

        private void addContent(XMLWriter xmlWriter) throws IOException {
            if (uniqueFicheGroup != null) {
                uniqueFicheGroup.add(xmlWriter, feedOptions);
            } else {
                for (FicheGroup ficheGroup : ficheGroupByClasseMap.values()) {
                    ficheGroup.add(xmlWriter, feedOptions);
                }
            }
        }

        void checkUriTree(DataAccess dataAccess, URITree uriTree) {
            int workingLangInteger = feedOptions.getLangUi();
            int ct = 0;
            int baseCount = uriTree.getBaseCount();
            for (int i = 0; i < baseCount; i++) {
                BaseNode baseNode = uriTree.getBaseNode(i);
                BaseURI baseURI = baseNode.getBaseURI();
                BaseData baseData = (BaseData) dataAccess.getScrutariData(baseURI);
                if (baseData == null) {
                    continue;
                }
                BaseCheck baseCheck = BaseCheck.check(baseData, workingLangInteger);
                int corpusCount = baseNode.getCorpusCount();
                for (int j = 0; j < corpusCount; j++) {
                    CorpusNode corpusNode = baseNode.getCorpusNode(j);
                    CorpusURI corpusURI = corpusNode.getCorpusURI();
                    CorpusData corpusData = (CorpusData) dataAccess.getScrutariData(corpusURI);
                    if (corpusData == null) {
                        continue;
                    }
                    FicheGroup ficheGroup = getFicheGroup(corpusData.getCodeCorpus());
                    int ficheCount = corpusNode.getFicheCount();
                    for (int k = 0; k < ficheCount; k++) {
                        FicheNode ficheNode = corpusNode.getFicheNode(k);
                        FicheURI ficheURI = ficheNode.getFicheURI();
                        FicheData ficheData = (FicheData) dataAccess.getScrutariData(ficheURI);
                        if (ficheData == null) {
                            continue;
                        }
                        ficheGroup.addFicheData(ficheData, baseCheck);
                        ct++;
                    }
                }
            }
            this.ficheAddCount = ct;
        }

        private FicheGroup getFicheGroup(int codecorpus) {
            if (uniqueFicheGroup != null) {
                return uniqueFicheGroup;
            }
            int rank = globalSearchOptions.getCorpusClasseRank(codecorpus);
            return ficheGroupByClasseMap.get(rank);
        }

    }


    private static class FicheGroup {

        private CorpusClasse corpusClasse;
        private SortedSet<FicheDataWrapper> ficheDataWrapperSet;

        private FicheGroup(CorpusClasse corpusClasse) {
            this.corpusClasse = corpusClasse;
            ficheDataWrapperSet = new TreeSet<FicheDataWrapper>(new FicheDataWrapperComparator());
        }

        private void addFicheData(FicheData ficheData, BaseCheck baseCheck) {
            ficheDataWrapperSet.add(new FicheDataWrapper(ficheData, baseCheck));
        }

        public void add(XMLWriter xmlWriter, SctFeedOptions feedOptions) throws IOException {
            if (ficheDataWrapperSet.isEmpty()) {
                return;
            }
            if (corpusClasse != null) {
                xmlWriter.startOpenTag("p", true);
                xmlWriter.addAttribute("class", "SCT_FicheGroup");
                xmlWriter.endOpenTag(false);
                xmlWriter.addText(corpusClasse.getLibelleString(feedOptions.getLangUi(), true));
                xmlWriter.closeTag("p", false, false);
            }
            xmlWriter.startOpenTag("ul");
            xmlWriter.addAttribute("class", "SCT_UL_Fiche");
            xmlWriter.endOpenTag();
            for (FicheDataWrapper ficheDataWrapper : ficheDataWrapperSet) {
                ficheDataWrapper.add(xmlWriter, feedOptions);
            }
            xmlWriter.closeTag("ul");
        }

    }


    private static class FicheDataWrapper {

        private FicheData ficheData;
        private BaseCheck baseCheck;

        private FicheDataWrapper(FicheData ficheData, BaseCheck baseCheck) {
            this.ficheData = ficheData;
            this.baseCheck = baseCheck;

        }

        private FicheData getFicheData() {
            return ficheData;
        }

        private void add(XMLWriter xmlWriter, SctFeedOptions feedOptions) throws IOException {
            FeedXmlUtils.addFicheData(xmlWriter, ficheData, feedOptions, baseCheck);
        }

    }


    private static class FicheDataWrapperComparator implements Comparator<FicheDataWrapper> {

        private FicheDataWrapperComparator() {
        }

        @Override
        public int compare(FicheDataWrapper ficheDataWrapper1, FicheDataWrapper ficheDataWrapper2) {
            FicheData ficheData1 = ficheDataWrapper1.getFicheData();
            FicheData ficheData2 = ficheDataWrapper2.getFicheData();
            String titre1 = ficheData1.getTitre();
            String titre2 = ficheData2.getTitre();
            if (titre1 == null) {
                if (titre2 != null) {
                    return 1;
                }
                return compareCode(ficheData1, ficheData2);
            } else {
                if (titre2 == null) {
                    return -1;
                }
                int comp = titre1.compareToIgnoreCase(titre2);
                if (comp != 0) {
                    return comp;
                } else {
                    return compareCode(ficheData1, ficheData2);
                }
            }
        }

        private int compareCode(FicheData ficheData1, FicheData ficheData2) {
            int code1 = ficheData1.getCodeFiche();
            int code2 = ficheData2.getCodeFiche();
            if (code1 < code2) {
                return -1;
            }
            if (code1 > code2) {
                return 1;
            }
            return 0;
        }

    }


    private static class InverseComparator implements Comparator<Integer> {

        private InverseComparator() {
        }

        @Override
        public int compare(Integer itg1, Integer itg2) {
            return -itg1.compareTo(itg2);
        }

    }

}
