/* SctServer - Copyright (c) 2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.xml.feed;

import fr.exemole.sctserver.api.feed.SctFeedConstants;
import fr.exemole.sctserver.api.feed.SctFeedOptions;
import java.io.IOException;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.xml.XMLWriter;
import net.scrutari.data.FicheData;
import net.scrutari.data.MotcleData;
import net.scrutari.db.tools.util.BaseCheck;


/**
 *
 * @author Vincent Calame
 */
public final class FeedXmlUtils {

    private FeedXmlUtils() {
    }

    public static void addFicheData(XMLWriter xmlWriter, FicheData ficheData, SctFeedOptions feedOptions, BaseCheck baseCheck) throws IOException {
        int optionMask = feedOptions.getOptionMask();
        xmlWriter.openTag("li");
        xmlWriter.startOpenTag("p", true);
        xmlWriter.addAttribute("class", "SCT_Fiche_Titre");
        xmlWriter.endOpenTag(false);
        if ((optionMask & SctFeedConstants.ICON_OPTION) != 0) {
            String icon = ficheData.getFicheIcon();
            if (icon == null) {
                icon = baseCheck.getBaseIcon();
            }
            if (icon != null) {
                xmlWriter.startOpenTag("img", false);
                xmlWriter.addAttribute("src", icon);
                xmlWriter.addAttribute("class", "SCT_Fiche_Icon");
                xmlWriter.closeEmptyTag();
                xmlWriter.addText(" ");
            }
        }
        String href = ficheData.getHref(baseCheck.getCheckedLangUi());
        if (href != null) {
            xmlWriter.startOpenTag("a");
            xmlWriter.addAttribute("href", href);
            xmlWriter.endOpenTag(false);
        }
        String titre = ficheData.getTitre();
        if (titre != null) {
            xmlWriter.addText(titre);
        } else {
            xmlWriter.addText(ficheData.getFicheURI().getFicheId());
        }
        if (href != null) {
            xmlWriter.closeTag("a", false, false);
        }
        xmlWriter.closeTag("p", false, false);
        if ((optionMask & SctFeedConstants.SOUSTITRE_OPTION) != 0) {
            String soustitre = ficheData.getSoustitre();
            if (soustitre != null) {
                xmlWriter.startOpenTag("p", true);
                xmlWriter.addAttribute("class", "SCT_Fiche_Soustitre");
                xmlWriter.endOpenTag(false);
                xmlWriter.addText(soustitre);
                xmlWriter.closeTag("p", false, false);
            }
        }
        xmlWriter.closeTag("li");
    }

    public static void addMotcleData(XMLWriter xmlWriter, MotcleData motcleData, SctFeedOptions feedOptions) throws IOException {
        xmlWriter.openTag("li");
        xmlWriter.startOpenTag("p", true);
        xmlWriter.addAttribute("class", "SCT_Motcle");
        xmlWriter.endOpenTag(false);
        int libelleCount = motcleData.getLibelleCount();
        for (int l = 0; l < libelleCount; l++) {
            if (l > 0) {
                xmlWriter.addText(" / ");
            }
            Libelle libelle = motcleData.getLibelleByIndex(l);
            xmlWriter.addText(libelle.getLibelleString());
        }
        xmlWriter.closeTag("p", false, false);
        xmlWriter.closeTag("li");
    }

}
