/* @@@@@APPLICATION@@@@@ - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.directory.persistent;

import fr.exemole.sctserver.api.storage.QIdStorage;
import fr.exemole.sctserver.api.storage.SctStorageException;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import javax.xml.parsers.DocumentBuilder;
import net.mapeadores.util.xml.AppendableXMLWriter;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.XMLUtils;
import net.scrutari.db.ScrutariDBName;
import net.scrutari.db.api.DataAccess;
import net.scrutari.searchengine.QId;
import net.scrutari.searchengine.api.result.FicheSearchSource;
import net.scrutari.searchengine.tools.dom.FicheSearchSourceDOMReader;
import net.scrutari.searchengine.tools.operands.filters.OperandConverterParameters;
import net.scrutari.searchengine.tools.result.FicheSearchSourceBuilder;
import net.scrutari.searchengine.xml.FicheSearchSourceXMLPart;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;


/**
 *
 * @author Vincent Calame
 */
public class DirectoryQIdStorage implements QIdStorage {

    private final File qidDirectory;

    public DirectoryQIdStorage(File qidDirectory) {
        this.qidDirectory = qidDirectory;
    }

    @Override
    public FicheSearchSource getFicheSearchSource(OperandConverterParameters operandConverterParameters, QId qId) {
        File qIdFile = getQIdFile(qId, false);
        if (!qIdFile.exists()) {
            return null;
        }
        DocumentBuilder documentBuilder = DOMUtils.newDocumentBuilder();
        Document doc;
        try {
            doc = documentBuilder.parse(qIdFile);
        } catch (SAXException saxe) {
            return null;
        } catch (IOException saxe) {
            return null;
        }
        FicheSearchSourceBuilder builder = new FicheSearchSourceBuilder();
        builder.setQId(qId);
        FicheSearchSourceDOMReader reader = new FicheSearchSourceDOMReader(operandConverterParameters, builder);
        reader.readFicheSearchSource(doc.getDocumentElement());
        if (builder.isValid()) {
            return builder.toFicheSearchSource();
        } else {
            return null;
        }
    }

    @Override
    public void saveFicheSearchSource(DataAccess dataAccess, FicheSearchSource ficheSearchSource) {
        QId qId = ficheSearchSource.getQId();
        if (qId == null) {
            return;
        }
        File qIdFile = getQIdFile(qId, true);
        try {
            BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(qIdFile), "UTF-8"));
            AppendableXMLWriter xmlWriter = XMLUtils.getXMLWriter(writer);
            xmlWriter.appendXMLDeclaration();
            FicheSearchSourceXMLPart ficheSearchXMLPart = new FicheSearchSourceXMLPart(xmlWriter);
            ficheSearchXMLPart.appendFicheSearchSource(dataAccess, ficheSearchSource);
            writer.close();
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }

    private File getQIdFile(QId qId, boolean mkdirs) {
        ScrutariDBName scrutariDBName = qId.getScrutariDBName();
        int subId = qId.getSubId();
        String century = String.valueOf(subId / 100) + "00";
        String year = scrutariDBName.getYear();
        File dbDir = new File(qidDirectory, year + File.separator + scrutariDBName.getName() + File.separator + century);
        if (mkdirs) {
            dbDir.mkdirs();
        }
        return new File(dbDir, subId + ".xml");
    }

}
