/* UtilLib - Copyright (c) 2012-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.attr;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.util.text.CleanedString;


/**
 *
 * @author Vincent Calame
 */
public class AppendableAttributeHolderBuilder {

    private Map<AttributeKey, AttributeBuilder> attributeMap = new LinkedHashMap<AttributeKey, AttributeBuilder>();

    public AppendableAttributeHolderBuilder() {
    }

    public void addAttribute(AttributeKey attributeKey, List<CleanedString> valueList) {
        if (valueList.isEmpty()) {
            return;
        }
        AttributeBuilder buffer = attributeMap.get(attributeKey);
        if (buffer == null) {
            buffer = new AttributeBuilder(attributeKey);
            attributeMap.put(attributeKey, buffer);
        }
        for (CleanedString cs : valueList) {
            if (cs != null) {
                buffer.addValue(cs);
            }
        }
    }

    public void addAttribute(Attribute attribute) {
        AttributeKey attributeKey = attribute.getAttributeKey();
        AttributeBuilder buffer = attributeMap.get(attributeKey);
        if (buffer == null) {
            buffer = new AttributeBuilder(attributeKey);
            attributeMap.put(attributeKey, buffer);
        }
        int valueCount = attribute.getValueCount();
        for (int i = 0; i < valueCount; i++) {
            buffer.addValue(CleanedString.newInstance(attribute.getValue(i)));
        }
    }

    public AttributeHolder toAttributeHolder() {
        if (attributeMap.isEmpty()) {
            return AttributeUtils.EMPTY_ATTRIBUTEHOLDER;
        }
        List<Attribute> attributeList = new ArrayList<Attribute>();
        for (AttributeBuilder buffer : attributeMap.values()) {
            Attribute attribute = buffer.toAttribute();
            if (attribute != null) {
                attributeList.add(attribute);
            }
        }
        int size = attributeList.size();
        if (size == 0) {
            return AttributeUtils.EMPTY_ATTRIBUTEHOLDER;
        }
        if (size == 1) {
            return AttributeUtils.toSingletonAttributeHolder(attributeList.get(0));
        }
        Attribute[] array = attributeList.toArray(new Attribute[size]);
        return new ArrayAttributeHolder(array);
    }


    private static class ArrayAttributeHolder implements AttributeHolder {

        private Attribute[] attributeArray;

        private ArrayAttributeHolder(Attribute[] attributeArray) {
            this.attributeArray = attributeArray;
        }

        @Override
        public Attribute getAttributeByKey(AttributeKey key) {
            int length = attributeArray.length;
            for (int i = 0; i < length; i++) {
                Attribute attribute = attributeArray[i];
                if (attribute.getAttributeKey().equals(key)) {
                    return attribute;
                }
            }
            return null;
        }

        @Override
        public int getAttributeCount() {
            return attributeArray.length;
        }

        @Override
        public Attribute getAttributeByIndex(int i) {
            return attributeArray[i];
        }

    }

}
