/* UtilLib - Copyright (c) 2011-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.html;

import net.mapeadores.util.localisation.Message;
import net.mapeadores.util.localisation.MessageList;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.logging.LogUtils;


/**
 *
 * @author Vincent Calame
 */
public class HtmlPrinterUtils {

    private HtmlPrinterUtils() {
    }

    public static void printColon(HtmlPrinter htmlPrinter) {
        htmlPrinter.append("&nbsp;: ");
    }

    public static void printLibColon(HtmlPrinter htmlPrinter, String libKey) {
        htmlPrinter.localize(libKey);
        printColon(htmlPrinter);
    }

    public static void labelInput(HtmlPrinter htmlPrinter, HTMLAttributes inputAttributes, String labelString, HTMLAttributes labelAttributes) {
        htmlPrinter.startLabel(labelAttributes);
        htmlPrinter.appendInput(inputAttributes);
        if (labelString != null) {
            String label = htmlPrinter.getLocalization(labelString);
            if (label == null) {
                label = labelString;
            }
            htmlPrinter.escape(label);
        }
        htmlPrinter.endLabel();
    }

    public static void labelInput(HtmlPrinter htmlPrinter, HTMLAttributes inputAttributes, Message labelMessage, HTMLAttributes labelAttributes) {
        htmlPrinter.startLabel(labelAttributes);
        htmlPrinter.appendInput(inputAttributes);
        String label = htmlPrinter.getLocalization(labelMessage);
        if (label == null) {
            label = labelMessage.getMessageKey();
        }
        htmlPrinter.escape(label);
        htmlPrinter.endLabel();
    }

    public static void printLibCells(HtmlPrinter htmlPrinter, String libKey, boolean topAlign) {
        if (topAlign) {
            htmlPrinter.startCellWithTopAlign();
        } else {
            htmlPrinter.startCell();
        }
        htmlPrinter.localize(libKey);
        htmlPrinter.endCell();
        if (topAlign) {
            htmlPrinter.startCellWithTopAlign();
        } else {
            htmlPrinter.startCell();
        }
        htmlPrinter.append("&nbsp;:&nbsp;");
        htmlPrinter.endCell();
    }

    public static void printCommandMessage(HtmlPrinter htmlPrinter, CommandMessage commandMessage, HTMLAttributes doneAttributes, HTMLAttributes errorAttributes) {
        HTMLAttributes spanAttr = doneAttributes;
        if (commandMessage.isErrorMessage()) {
            spanAttr = errorAttributes;
        }
        htmlPrinter.startP();
        htmlPrinter.startSpan(spanAttr);
        htmlPrinter.localize(commandMessage);
        htmlPrinter.endSpan();
        if (commandMessage.hasMultiError()) {
            MessageList list = commandMessage.getMultiErrorList();
            htmlPrinter.startSpan(errorAttributes);
            htmlPrinter.startSmall();
            int count = list.getMessageCount();
            for (int i = 0; i < count; i++) {
                htmlPrinter.appendBr();
                htmlPrinter.localize(list.getMessage(i));
            }
            htmlPrinter.endSmall();
            htmlPrinter.endSpan();
        } else if (commandMessage.hasLog()) {
            htmlPrinter.appendBr();
            htmlPrinter.startPre(null);
            htmlPrinter.escape(LogUtils.toString(commandMessage.getProcessMessageLog()), true);
            htmlPrinter.endPre();
        }
        htmlPrinter.endP();
    }

    public static HTMLAttributes newFormAttributes(CommandBoxParameters commandBoxParameters) {
        if (commandBoxParameters == null) {
            return new HTMLAttributes();
        }
        HTMLAttributes attr = commandBoxParameters.getFormAttributes();
        if (attr != null) {
            return attr.copy();
        } else {
            return new HTMLAttributes();
        }
    }

    public static HTMLAttributes newFieldsetAttributes(CommandBoxParameters commandBoxParameters) {
        if (commandBoxParameters == null) {
            return new HTMLAttributes();
        }
        HTMLAttributes attr = commandBoxParameters.getFieldsetAttributes();
        if (attr != null) {
            return attr.copy();
        } else {
            return new HTMLAttributes();
        }
    }

    public static HTMLAttributes newLegendAttributes(CommandBoxParameters commandBoxParameters) {
        if (commandBoxParameters == null) {
            return new HTMLAttributes();
        }
        HTMLAttributes attr = commandBoxParameters.getLegendAttributes();
        if (attr != null) {
            return attr.copy();
        } else {
            return new HTMLAttributes();
        }
    }

    public static HTMLAttributes newCommandKeyDivAttributes(CommandBoxParameters commandBoxParameters) {
        if (commandBoxParameters == null) {
            return new HTMLAttributes();
        }
        HTMLAttributes attr = commandBoxParameters.getCommandKeyDivAttributes();
        if (attr != null) {
            return attr.copy();
        } else {
            return new HTMLAttributes();
        }
    }

    public static HTMLAttributes newSubmitDivAttributes(CommandBoxParameters commandBoxParameters) {
        if (commandBoxParameters == null) {
            return new HTMLAttributes();
        }
        HTMLAttributes attr = commandBoxParameters.getSubmitDivAttributes();
        if (attr != null) {
            return attr.copy();
        } else {
            return new HTMLAttributes();
        }
    }

}
