/* UtilLib - Copyright (c) 2008-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.html;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import net.mapeadores.util.localisation.ResourceLocalisation;


/**
 *
 * @author Vincent Calame
 */
public class UtilHtmlPrinter extends HtmlPrinter {

    private String htmlpage_title;
    private HTMLAttributes bodyHTMLAttributes;
    private ResourceLocalisation commandKeyLocalisation;
    private CommandBoxParameters commandBoxParameters;
    private boolean onCommandBox = false;
    private String currentCommandKey = null;
    private List<String> cssUrlList = new ArrayList<String>();
    private List<String> jsUrlList = new ArrayList<String>();
    private String iconPng = null;

    public UtilHtmlPrinter() {
    }

    public void setCommandKeyLocalisation(ResourceLocalisation commandKeyLocalisation) {
        this.commandKeyLocalisation = commandKeyLocalisation;
    }

    public ResourceLocalisation getCommandKeyLocalisation() {
        return commandKeyLocalisation;
    }

    public void setCommandBoxParameters(CommandBoxParameters commandBoxParameters) {
        this.commandBoxParameters = commandBoxParameters;
    }

    public CommandBoxParameters getCommandBoxParameters() {
        return commandBoxParameters;
    }

    public void setTitle(String title) {
        this.htmlpage_title = title;
    }

    public void localizeTitle(String key, Object value) {
        this.htmlpage_title = getLocalization(key, value);
        if (htmlpage_title == null) {
            htmlpage_title = key;
        }
    }

    public String getTitle() {
        return htmlpage_title;
    }

    public void setBodyHTMLAttributes(HTMLAttributes bodyHTMLAttributes) {
        this.bodyHTMLAttributes = bodyHTMLAttributes;
    }

    public void addCssUrl(String cssUrl) {
        cssUrlList.add(cssUrl);
    }

    public void addJsUrl(String jsUrl) {
        jsUrlList.add(jsUrl);
    }

    public void start(int langInteger) {
        appendHtmlDocType();
        startHtml(langInteger);
        startHead();
        startTitle();
        escape(htmlpage_title);
        endTitle();
        append("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">");
        breakLine();
        if (!jsUrlList.isEmpty()) {
            append("<meta http-equiv=\"Content-Script-Type\" content=\"text/javascript\">");
            breakLine();
        }
        for (String cssUrl : cssUrlList) {
            append("<link href=\"");
            append(cssUrl);
            append("\" rel=\"stylesheet\" type=\"text/css\">");
            breakLine();
        }
        if (iconPng != null) {
            append("<link href=\"");
            append(iconPng);
            append("\" rel=\"icon\" type=\"image/png\">");
            breakLine();
        }
        if (!jsUrlList.isEmpty()) {
            HTMLAttributes scriptAttributes = HA.type("text/javascript");
            int length = jsUrlList.size();
            for (int i = 0; i < length; i++) {
                scriptAttributes.src(jsUrlList.get(i));
                startScript(scriptAttributes);
                endScript();
            }
        }
        endHead();
        startBody(bodyHTMLAttributes);
    }

    public void startCommandBox(CommandBox commandBox) {
        if (onCommandBox) {
            throw new IllegalStateException("last CommandBox is not ended");
        }
        onCommandBox = true;
        HTMLAttributes formAttributes = HtmlPrinterUtils.newFormAttributes(commandBoxParameters);
        formAttributes.action(commandBox.getFormAction()).method(HtmlConstants.POST_METHOD);
        if (commandBox.isMultipart()) {
            formAttributes.enctype(HtmlConstants.MULTIPART_ENCTYPE);
        }
        startForm(formAttributes);
        appendHiddenInput("cmd", commandBox.getCommandString());
        String page = commandBox.getPage();
        if (page != null) {
            appendHiddenInput("page", page);
        }
        String errorPage = commandBox.getErrorPage();
        if (errorPage != null) {
            appendHiddenInput("page-err", errorPage);
        }
        Map<String, String> hiddenAttributesMap = commandBox.getHiddenAttributesMap();
        if (hiddenAttributesMap != null) {
            for (Map.Entry<String, String> entry : hiddenAttributesMap.entrySet()) {
                appendHiddenInput(entry.getKey(), entry.getValue());
            }
        }
        HTMLAttributes fieldsetAttributes = HtmlPrinterUtils.newFieldsetAttributes(commandBoxParameters);
        String commandFamily = commandBox.getCommandFamily();
        if (commandFamily != null) {
            fieldsetAttributes.classes("fs_" + commandFamily);
        } else {
            fieldsetAttributes.classes(null);
        }
        startFieldset(fieldsetAttributes);
        String commandKey = commandBox.getCommandKey();
        if (commandKey != null) {
            startLegend(HtmlPrinterUtils.newLegendAttributes(commandBoxParameters));
            escape(commandKeyLocalisation.toString(commandKey, null));
            endLegend();
            currentCommandKey = commandKey;
        }
    }

    public void endCommandBox(String submitKey) {
        if (!onCommandBox) {
            throw new IllegalStateException("no CommandBox started yet");
        }
        if (submitKey != null) {
            startDiv(HtmlPrinterUtils.newSubmitDivAttributes(commandBoxParameters));
            appendInput(HA.type(HtmlConstants.SUBMIT_TYPE).value(getLocalization(submitKey)));
            endDiv();
        }
        if (currentCommandKey != null) {
            startDiv(HtmlPrinterUtils.newCommandKeyDivAttributes(commandBoxParameters));
            append(currentCommandKey);
            endDiv();
        }
        endFieldset();
        endForm();
        currentCommandKey = null;
        onCommandBox = false;
    }

    public void end() {
        if (onCommandBox) {
            throw new IllegalStateException("last CommandBox is not ended");
        }
        endBody();
        endHtml();
    }

    @Deprecated
    public void printColon() {
        HtmlPrinterUtils.printColon(this);
    }

    @Deprecated
    public void printLibColon(String libKey) {
        HtmlPrinterUtils.printLibColon(this, libKey);
    }

    @Deprecated
    public void printLibCells(String libKey, boolean topAlign) {
        HtmlPrinterUtils.printLibCells(this, libKey, topAlign);
    }

    @Deprecated
    public void printLitteralLibCells(String value, boolean topAlign) {
        if (topAlign) {
            startCellWithTopAlign();
        } else {
            startCell();
        }
        escape(value);
        endCell();
        if (topAlign) {
            startCellWithTopAlign();
        } else {
            startCell();
        }
        append("&nbsp;:&nbsp;");
        endCell();
    }

    public void setIconPng(String iconPng) {
        this.iconPng = iconPng;
    }

}
