/* UtilLib - Copyright (c) 2006-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.localisation;

import java.text.DecimalFormatSymbols;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.text.DateFormatBundle;


/**
 *
 * @author Vincent Calame
 */
public final class LocaleBundle {

    private static final Map<Locale, LocaleBundle> bundleMap = new HashMap<Locale, LocaleBundle>();
    private Locale locale;
    private DecimalFormatSymbols symbols;
    private DateFormatBundle dateFormatBundle;

    private LocaleBundle(Locale locale) {
        this.locale = locale;
        symbols = new DecimalFormatSymbols(locale);
        dateFormatBundle = DateFormatBundle.getDateFormatBundle(locale);
    }

    public Locale getLocale() {
        return locale;
    }

    public DecimalFormatSymbols getDecimalFormatSymbols() {
        return symbols;
    }

    public DateFormatBundle getDateFormatBundle() {
        return dateFormatBundle;
    }

    public String getFileLenghtFormat() {
        return "0" + symbols.getDecimalSeparator() + "#";
    }

    public String getIdFormat() {
        return "#" + symbols.getGroupingSeparator() + "##0";
    }

    public static LocaleBundle getLocaleBundle(Locale locale) {
        LocaleBundle localeBundle = bundleMap.get(locale);
        if (localeBundle == null) {
            synchronized (bundleMap) {
                localeBundle = new LocaleBundle(locale);
                bundleMap.put(locale, localeBundle);
            }
        }
        return localeBundle;
    }

    public String getLangString() {
        return LangInteger.toString(LangInteger.fromLocale(locale));
    }

}
