/* UtilLib - Copyright (c) 2008-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.localisation;

import java.util.List;
import java.util.Locale;
import net.mapeadores.util.langinteger.LangInteger;


/**
 *
 * @author Vincent Calame
 */
public class LocalisationUtils {

    public final static MessageList EMPTY_MESSAGELIST = new EmptyMessageList();
    private final static int FRENCH = LangInteger.build("fr");

    private LocalisationUtils() {
    }

    public static String getKibiOctet(Locale locale) {
        return getKibiOctet(LangInteger.fromLocale(locale));
    }

    public static String getKibiOctet(int langInteger) {
        int langPart = LangInteger.getLangPart(langInteger);
        if (langPart == FRENCH) {
            return "Kio";
        } else {
            return "KiB";
        }
    }

    public static String getMebiOctet(Locale locale) {
        return getMebiOctet(LangInteger.fromLocale(locale));
    }

    public static String getMebiOctet(int langInteger) {
        int langPart = LangInteger.getLangPart(langInteger);
        if (langPart == FRENCH) {
            return "Mio";
        } else {
            return "MiB";
        }
    }

    public static Message toMessage(String messageKey) {
        if (messageKey == null) {
            throw new IllegalArgumentException("messageKey is null");
        }
        return new InternalMessage(messageKey, null);
    }

    public static Message toMessage(String messageKey, Object messageValue) {
        if (messageKey == null) {
            throw new IllegalArgumentException("messageKey is null");
        }
        return new InternalMessage(messageKey, messageValue);
    }

    public static MessageList toMessageList(List<Message> list) {
        if (list.isEmpty()) {
            return EMPTY_MESSAGELIST;
        }
        return new ArrayMessageList(list.toArray(new Message[list.size()]));
    }


    private static class EmptyMessageList implements MessageList {

        private EmptyMessageList() {
        }

        public int getMessageCount() {
            return 0;
        }

        public Message getMessage(int i) {
            throw new IndexOutOfBoundsException();
        }

    }


    private static class ArrayMessageList implements MessageList {

        private Message[] array;

        private ArrayMessageList(Message[] array) {
            this.array = array;
        }

        public int getMessageCount() {
            return array.length;
        }

        public Message getMessage(int i) {
            return array[i];
        }

    }


    private static class InternalMessage implements Message {

        private String messageKey;
        private Object messageValue;

        private InternalMessage(String messageKey, Object messageValue) {
            this.messageKey = messageKey;
            this.messageValue = messageValue;
        }

        public String getMessageKey() {
            return messageKey;
        }

        public Object getMessageValue() {
            return messageValue;
        }

    }

}
