/* UtilLib - Copyright (c) 2008-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.logging;

import net.mapeadores.util.localisation.Message;
import net.mapeadores.util.localisation.MessageList;


/**
 *
 * @author Vincent Calame
 */
public class CommandMessage implements Message {

    private String messageKey;
    private Object messageValue;
    private ProcessMessageLog processMessageLog;
    private MessageList multiErrorList;
    private boolean errorMessage = false;

    private CommandMessage(String messageKey, Object messageValue) {
        if (messageKey == null) {
            throw new IllegalArgumentException("messageKey is null");
        }
        this.messageKey = messageKey;
        this.messageValue = messageValue;
    }

    private void initIsErrorMessage(boolean errorMessage) {
        this.errorMessage = errorMessage;
    }

    private void initMultiErrorList(MessageList multiErrorList) {
        this.multiErrorList = multiErrorList;
    }

    private void initProcessMessageLog(ProcessMessageLog processMessageLog) {
        this.processMessageLog = processMessageLog;
    }

    public String getMessageKey() {
        return messageKey;
    }

    public Object getMessageValue() {
        return messageValue;
    }

    public boolean isErrorMessage() {
        return errorMessage;
    }

    public boolean hasMultiError() {
        return (multiErrorList != null);
    }

    public MessageList getMultiErrorList() {
        return multiErrorList;
    }

    public boolean hasLog() {
        return ((processMessageLog != null) && (processMessageLog.getLogGroupCount() > 0));
    }

    public ProcessMessageLog getProcessMessageLog() {
        return processMessageLog;
    }

    public static CommandMessage error(String messageKey, Object messageValue) {
        CommandMessage commandMessage = new CommandMessage(messageKey, messageValue);
        commandMessage.initIsErrorMessage(true);
        return commandMessage;
    }

    public static CommandMessage error(Message message) {
        CommandMessage commandMessage = new CommandMessage(message.getMessageKey(), message.getMessageValue());
        commandMessage.initIsErrorMessage(true);
        return commandMessage;
    }

    public static CommandMessage error(String messageKey, Object messageValue, MessageList multiErrorList) {
        CommandMessage commandMessage = new CommandMessage(messageKey, messageValue);
        commandMessage.initIsErrorMessage(true);
        commandMessage.initMultiErrorList(multiErrorList);
        return commandMessage;
    }

    public static CommandMessage done(String messageKey, Object messageValue) {
        CommandMessage commandMessage = new CommandMessage(messageKey, messageValue);
        return commandMessage;
    }

    public static CommandMessage done(Message message) {
        CommandMessage commandMessage = new CommandMessage(message.getMessageKey(), message.getMessageValue());
        return commandMessage;
    }

    public static CommandMessage done(String messageKey, Object messageValue, ProcessMessageLog processMessageLog) {
        CommandMessage commandMessage = new CommandMessage(messageKey, messageValue);
        commandMessage.initProcessMessageLog(processMessageLog);
        return commandMessage;
    }

}
