/* UtilLib - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.logging;

import java.util.ArrayList;
import java.util.List;


/**
 *
 * @author Vincent Calame
 */
public class ProcessMessageLogBuilder implements MultiProcessMessageHandler {

    private List<ProcessMessage> currentList = new ArrayList<ProcessMessage>();
    private String currentURI = null;
    private List<InternalLogGroup> groupList = new ArrayList<InternalLogGroup>();

    public ProcessMessageLogBuilder() {
    }

    @Override
    public void setCurrentURI(String uri) {
        if (currentList.isEmpty()) {
            currentURI = uri;
        } else {
            boolean flush = false;
            if (uri == null) {
                if (currentURI != null) {
                    flush = true;
                }
            } else if (currentURI == null) {
                flush = true;
            } else {
                flush = !uri.equals(currentURI);
            }
            if (flush) {
                flushGroup();
                currentURI = uri;
            }
        }
    }

    @Override
    public void addMessage(String category, String key, Object value) {
        currentList.add(LogUtils.newProcessMessage(category, key, value));
    }

    private void flushGroup() {
        if (!currentList.isEmpty()) {
            groupList.add(new InternalLogGroup(currentURI, LogUtils.toProcessMessageList(currentList)));
            currentList.clear();
        }
    }

    public ProcessMessageLog toProcessMessageLog() {
        flushGroup();
        return new InternalProcessMessageLog(groupList.toArray(new ProcessMessageLog.LogGroup[groupList.size()]));
    }


    private static class InternalLogGroup implements ProcessMessageLog.LogGroup {

        private String processURI;
        private ProcessMessageList processMessageList;

        private InternalLogGroup(String processURI, ProcessMessageList processMessageList) {
            this.processURI = processURI;
            this.processMessageList = processMessageList;
        }

        @Override
        public String getURI() {
            return processURI;
        }

        @Override
        public ProcessMessageList getList() {
            return processMessageList;
        }

    }


    private static class InternalProcessMessageLog implements ProcessMessageLog {

        private ProcessMessageLog.LogGroup[] array;

        private InternalProcessMessageLog(ProcessMessageLog.LogGroup[] array) {
            this.array = array;
        }

        @Override
        public int getLogGroupCount() {
            return array.length;
        }

        @Override
        public LogGroup getLogGroup(int i) {
            return array[i];
        }

    }

}
