/* UtilLib - Copyright (c) 2012-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.request;

import java.io.File;
import java.io.IOException;
import net.mapeadores.util.attr.AttributeHolder;
import net.mapeadores.util.attr.AttributeUtils;
import net.mapeadores.util.io.MimeTypeConstants;
import net.mapeadores.util.request.xml.RequestConfDOMReader;
import net.mapeadores.util.xml.DOMUtils;
import org.w3c.dom.Document;


/**
 *
 * @author Vincent Calame
 */
public class RequestUtils {

    public final static SupplementaryParameterList EMPTY_SUPPLEMENTARYPARAMETERLIST = new EmptySupplementaryParameterList();
    public final static RequestConf EMPTY_REQUESTCONF = new EmptyRequestConf();

    private RequestUtils() {
    }

    public static AppendableRequestMap merge(RequestMap requestMap, SupplementaryParameterList supplementaryParameterList) {
        AppendableRequestMap appendableRequestMap = new AppendableRequestMap(requestMap);
        appendableRequestMap.addSupplementaryParameterParameterList(supplementaryParameterList);
        return appendableRequestMap;
    }

    public static RequestConf parseRequestConf(File f) throws IOException {
        Document document = DOMUtils.readDocument(f);
        return parseRequestConf(document);
    }

    public static RequestConf parseRequestConf(Document document) {
        RequestConfBuilder requestConfBuilder = new RequestConfBuilder();
        RequestConfDOMReader reader = new RequestConfDOMReader(requestConfBuilder);
        reader.readConf(document.getDocumentElement());
        return requestConfBuilder.toRequestConf();
    }

    public static short getJsonType(RequestMap requestMap) {
        return parseJsonType(requestMap.getParameter(RequestConstants.JSONTYPE_PARAMETER));
    }

    public static boolean isTest(RequestMap requestMap) {
        String testString = requestMap.getParameter(RequestConstants.TEST_PARAMETER);
        if (testString == null) {
            return false;
        }
        testString = testString.toLowerCase();
        if ((testString.equals("1")) || (testString.equals("true"))) {
            return true;
        }
        return false;
    }

    public static boolean isJsonDefined(RequestMap requestMap) {
        String json = requestMap.getParameter(RequestConstants.JSON_PARAMETER);
        if (json != null) {
            return true;
        }
        String jsonType = requestMap.getParameter(RequestConstants.JSONTYPE_PARAMETER);
        if (jsonType != null) {
            return true;
        }
        String output = requestMap.getParameter(RequestConstants.OUTPUT_PARAMETER);
        if ((output != null) && (output.startsWith("json:"))) {
            return true;
        }
        return false;
    }

    public static String jsonTypeToContentType(short jsonType) {
        switch (jsonType) {
            case JsonType.JSON_TYPE:
                return MimeTypeConstants.JSON;
            case JsonType.JSON_IN_HTML_TYPE:
                return MimeTypeConstants.HTML + ";charset=UTF-8";
            case JsonType.JSON_P_TYPE:
                return MimeTypeConstants.JAVASCRIPT + ";charset=UTF-8";
            default:
                throw new IllegalArgumentException("Unknown type: " + jsonType);
        }
    }

    public static short parseJsonType(String s) {
        if (s == null) {
            return JsonType.JSON_TYPE;
        }
        if (s.equals(JsonType.JSON_STRING)) {
            return JsonType.JSON_TYPE;
        }
        if (s.equals(JsonType.JSON_IN_HTML_STRING)) {
            return JsonType.JSON_IN_HTML_TYPE;
        }
        if (s.equals(JsonType.JSON_P_STRING)) {
            return JsonType.JSON_P_TYPE;
        }
        return JsonType.JSON_TYPE;
    }


    private static class EmptyRequestConf implements RequestConf {

        private EmptyRequestConf() {
        }

        @Override
        public SupplementaryParameterList getSupplementaryParameterList() {
            return EMPTY_SUPPLEMENTARYPARAMETERLIST;
        }

        @Override
        public AttributeHolder getAttributes() {
            return AttributeUtils.EMPTY_ATTRIBUTEHOLDER;
        }

    }


    private static class EmptySupplementaryParameterList implements SupplementaryParameterList {

        private EmptySupplementaryParameterList() {
        }

        @Override
        public int getSupplementaryParameterCount() {
            return 0;
        }

        @Override
        public SupplementaryParameter getSupplementaryParameter(int i) {
            throw new IndexOutOfBoundsException();
        }

    }

}
