/* UtilLib - Copyright (c) 2007-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.sql;

import java.util.List;
import java.util.ArrayList;
import java.io.IOException;
import java.io.Writer;


/**
 *
 * @author Vincent Calame
 */
public class CreateTableWriter {

    public static final int PRIMARY_KEY = 1;
    private Writer writer;
    private boolean firstField = true;
    private List<String> primaryKeyList = new ArrayList<String>();

    public CreateTableWriter(Writer writer) {
        this.writer = writer;
    }

    public void startTable(String tableName, String options) throws IOException {
        firstField = true;
        primaryKeyList.clear();
        writer.write("CREATE TABLE");
        writeOptions(options);
        writer.write(" ");
        writer.write(tableName);
        writer.write(" (");
    }

    public void addIntegerField(String fieldName, int mask, String options) throws IOException {
        testFirstField();
        writer.write(fieldName);
        writer.write(" INTEGER");
        if ((mask & PRIMARY_KEY) != 0) {
            primaryKeyList.add(fieldName);
        }
        writeOptions(options);
    }

    public void addDateField(String fieldName, int mask, String options) throws IOException {
        testFirstField();
        writer.write(fieldName);
        writer.write(" DATE");
        if ((mask & PRIMARY_KEY) != 0) {
            primaryKeyList.add(fieldName);
        }
        writeOptions(options);
    }

    public void addDoubleField(String fieldName, int mask, String options) throws IOException {
        testFirstField();
        writer.write(fieldName);
        writer.write(" DOUBLE");
        if ((mask & PRIMARY_KEY) != 0) {
            primaryKeyList.add(fieldName);
        }
        writeOptions(options);
    }

    public void addTextField(String fieldName, int mask, String options) throws IOException {
        testFirstField();
        writer.write(fieldName);
        if ((mask & PRIMARY_KEY) != 0) {
            writer.write(" VARCHAR(255)");
            primaryKeyList.add(fieldName);
        } else {
            writer.write(" TEXT");
        }
        writeOptions(options);
    }

    private void writeOptions(String options) throws IOException {
        if ((options != null) && (options.length() > 0)) {
            writer.write(" ");
            writer.write(options);
        }
    }

    public void endTable() throws IOException {
        int size = primaryKeyList.size();
        if (size > 0) {
            writer.write(", PRIMARY KEY (");
            for (int i = 0; i < size; i++) {
                if (i > 0) {
                    writer.write(", ");
                }
                writer.write((String) primaryKeyList.get(i));
            }
            writer.write(")");
        }
        writer.write(")");
    }

    private void testFirstField() throws IOException {
        if (firstField) {
            firstField = false;
        } else {
            writer.write(", ");
        }
    }

}
