/* UtilLib - Copyright (c) 2005-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text;

import net.mapeadores.util.text.search.MultiSearchToken;
import net.mapeadores.util.text.search.SearchToken;
import net.mapeadores.util.text.search.SimpleSearchToken;


/**
 *
 * @author Vincent Calame
 */
public class SearchSequenceUtils {

    private SearchSequenceUtils() {
    }

    public static String toSQLCondition(String fieldName, SearchSequence searchSequence, short combinaisonType) {
        int tokenCount = searchSequence.getTokenCount();
        if (tokenCount == 0) {
            return "";
        }
        StringBuilder buf = new StringBuilder();
        buf.append('(');
        for (int i = 0; i < tokenCount; i++) {
            if (i > 0) {
                if (combinaisonType == TextConstants.AND) {
                    buf.append(" AND ");
                } else if (combinaisonType == TextConstants.OR) {
                    buf.append(" OR ");
                }
            }
            SearchToken token = searchSequence.getToken(i);
            if (token instanceof MultiSearchToken) {
                MultiSearchToken multiToken = (MultiSearchToken) token;
                int subCount = multiToken.getSubtokenCount();
                for (int j = 0; j < subCount; j++) {
                    if (j > 0) {
                        if (combinaisonType == TextConstants.AND) {
                            buf.append(" AND ");
                        } else if (combinaisonType == TextConstants.OR) {
                            buf.append(" OR ");
                        }
                    }
                    SimpleSearchToken simpleToken = multiToken.getSubtoken(j);
                    appendCondition(buf, fieldName, simpleToken.getTokenString(), simpleToken.getSearchType());
                }
            } else {
                SimpleSearchToken simpleToken = (SimpleSearchToken) token;
                appendCondition(buf, fieldName, simpleToken.getTokenString(), simpleToken.getSearchType());
            }
        }
        buf.append(')');
        return buf.toString();
    }

    private static void appendCondition(StringBuilder buf, String fieldName, String token, int type) {
        buf.append('(');
        buf.append(fieldName);
        switch (type) {
            case TextConstants.MATCHES:
                buf.append(" LIKE '");
                appendString(buf, token);
                buf.append("'");
                break;
            case TextConstants.CONTAINS:
                buf.append(" LIKE '%");
                appendString(buf, token);
                buf.append("%'");
                break;
            case TextConstants.STARTSWITH:
                buf.append(" LIKE '");
                appendString(buf, token);
                buf.append("%'");
                break;
            case TextConstants.ENDSWITH:
                buf.append(" LIKE '%");
                appendString(buf, token);
                buf.append("'");
                break;
        }
        buf.append(')');
    }

    private static void appendString(StringBuilder buf, String token) {
        int length = token.length();
        for (int i = 0; i < length; i++) {
            char carac = token.charAt(i);
            if (carac == '\'') {
                buf.append("''");
            } else {
                buf.append(carac);
            }
        }
    }

}
