/* UtilLib - Copyright (c) 2012-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text.gabarit;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StringReader;
import java.io.StringWriter;
import net.mapeadores.util.exceptions.ShouldNotOccurException;


/**
 *
 * @author Vincent Calame
 */
public final class GabaritUtils {

    private GabaritUtils() {
    }

    public static String resolve(String gabaritString, GabaritResolver gabaritResolver) {
        try {
            StringReader reader = new StringReader(gabaritString);
            StringWriter writer = new StringWriter();
            resolve(reader, gabaritResolver, writer);
            return writer.toString();
        } catch (IOException ioe) {
            throw new ShouldNotOccurException(ioe);
        }
    }

    public static String resolve(InputStream inputStream, String encoding, GabaritResolver gabaritResolver) throws IOException {
        StringWriter writer = new StringWriter();
        InputStreamReader inputStreamReader = new InputStreamReader(inputStream, encoding);
        resolve(inputStreamReader, gabaritResolver, writer);
        return writer.toString();
    }

    public String resolve(Reader gabaritReader, GabaritResolver gabaritResolver) throws IOException {
        StringWriter writer = new StringWriter();
        resolve(gabaritReader, gabaritResolver, writer);
        return writer.toString();
    }

    public static void resolve(String gabaritString, GabaritResolver gabaritResolver, Appendable appendable) throws IOException {
        StringReader reader = new StringReader(gabaritString);
        resolve(reader, gabaritResolver, appendable);
    }

    public static void resolve(InputStream inputStream, String encoding, GabaritResolver gabaritResolver, Appendable appendable) throws IOException {
        InputStreamReader inputStreamReader = new InputStreamReader(inputStream, encoding);
        resolve(inputStreamReader, gabaritResolver, appendable);
    }

    public static void resolve(Reader gabaritReader, GabaritResolver gabaritResolver, Appendable appendable) throws IOException {
        Gabarit gabarit = GabaritFactory.parse(gabaritReader);
        resolve(gabarit, gabaritResolver, appendable);
    }

    public static String resolve(Gabarit gabarit, GabaritResolver gabaritResolver) {
        try {
            StringWriter writer = new StringWriter();
            resolve(gabarit, gabaritResolver, writer);
            return writer.toString();
        } catch (IOException ioe) {
            throw new ShouldNotOccurException(ioe);
        }
    }

    public static void resolve(Gabarit gabarit, GabaritResolver gabaritResolver, Appendable appendable) throws IOException {
        int elementCount = gabarit.getGabaritElementCount();
        for (int i = 0; i < elementCount; i++) {
            Object element = gabarit.getGabaritElement(i);
            if (element instanceof String) {
                appendable.append((String) element);
            } else {
                GabaritInclusion gabaritInclusion = (GabaritInclusion) element;
                appendable.append(gabaritResolver.resolve(gabaritInclusion));
            }
        }
    }

}
