/* UtilLib - Copyright (c) 2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.xml;

import java.io.IOException;


/**
 *
 * @author Vincent Calame
 */
public abstract class FillControllerXMLPart implements XMLWriter {

    private XMLWriter xmlWriter;
    private boolean fillControled = false;

    public FillControllerXMLPart(XMLWriter xmlWriter) {
        this.xmlWriter = xmlWriter;
    }

    public XMLWriter getXMLWriter() {
        return xmlWriter;
    }

    public abstract void controlFill() throws IOException;

    public boolean isFillControled() {
        return fillControled;
    }

    public void addText(CharSequence charSequence) throws IOException {
        if (charSequence.length() > 0) {
            fillControlTest();
            xmlWriter.addText(charSequence);
        }
    }

    public void openTag(String name) throws IOException {
        fillControlTest();
        xmlWriter.openTag(name);
    }

    public void openTag(String name, boolean indentBefore, boolean indentIncrease) throws IOException {
        fillControlTest();
        xmlWriter.openTag(name, indentBefore, indentIncrease);
    }

    public void startOpenTag(String name) throws IOException {
        fillControlTest();
        xmlWriter.startOpenTag(name);
    }

    public void startOpenTag(String name, boolean indentBefore) throws IOException {
        fillControlTest();
        xmlWriter.startOpenTag(name, indentBefore);
    }

    public void closeTag(String name) throws IOException {
        xmlWriter.closeTag(name);
    }

    public void closeTag(String name, boolean indentBefore, boolean indentDecrease) throws IOException {
        xmlWriter.closeTag(name, indentBefore, indentDecrease);
    }

    public void endOpenTag() throws IOException {
        xmlWriter.endOpenTag();
    }

    public void endOpenTag(boolean increaseIndent) throws IOException {
        xmlWriter.endOpenTag(increaseIndent);
    }

    public void addAttribute(String name, String value) throws IOException {
        if ((value != null) && (value.length() > 0)) {
            fillControlTest();
            xmlWriter.addAttribute(name, value);
        }
    }

    public void addAttribute(String name, int value) throws IOException {
        fillControlTest();
        xmlWriter.addAttribute(name, value);
    }

    public void closeEmptyTag() throws IOException {
        xmlWriter.closeEmptyTag();
    }

    public void addSimpleElement(String tagname, String value) throws IOException {
        fillControlTest();
        xmlWriter.addSimpleElement(tagname, value);
    }

    public void addEmptyElement(String tagname) throws IOException {
        fillControlTest();
        xmlWriter.addEmptyElement(tagname);
    }

    public void addCData(CharSequence charSequence) throws IOException {
        if (charSequence.length() > 0) {
            fillControlTest();
            xmlWriter.addCData(charSequence);
        }
    }

    private void fillControlTest() throws IOException {
        if (!fillControled) {
            controlFill();
            fillControled = true;
        }
    }

}
