/* UtilLib - Copyright (c) 2008-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.xml.xsltpattern;

import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import net.mapeadores.util.exceptions.InternalResourceException;


/**
 *
 * @author Vincent Calame
 */
public class XSLTPatternBuilder {

    private StringBuilder buf = new StringBuilder();

    public XSLTPatternBuilder() {
    }

    @Override
    public String toString() {
        return buf.toString();
    }

    public boolean testPremier(boolean premier) {
        if (!premier) {
            appendSeparator();
        }
        return false;
    }

    public void appendSeparator() {
        buf.append(", ");
    }

    public void appendReader(Reader reader) throws IOException {
        int n;
        char[] buffer = new char[1004];
        while ((n = reader.read(buffer)) != -1) {
            for (int i = 0; i < n; i++) {
                append(buffer[i]);
            }
        }
    }

    public void appendInstruction(String name) {
        buf.append('{');
        buf.append(name);
        buf.append('}');
    }

    public void appendLibInstruction(String value) {
        buf.append("{lib=");
        buf.append(value);
        buf.append('}');
    }

    public void appendSimpleInstruction(String name, String value) {
        buf.append('{');
        buf.append(name);
        buf.append('=');
        buf.append(value);
        buf.append('}');
    }

    public void appendModeInstruction(String name, String mode, String className, String sep) {
        buf.append('{');
        buf.append(name);
        buf.append("; ");
        buf.append("mode=");
        buf.append(mode);
        if (className != null) {
            buf.append(",");
            buf.append(" class=");
            buf.append(className);
        }
        if (sep != null) {
            buf.append(", ");
            buf.append(" sep=");
            buf.append("\"'");
            buf.append(sep);
            buf.append("'\"");
        }
        buf.append('}');
    }

    public void append(char carac) {
        buf.append(carac);
    }

    public void append(String s) {
        buf.append(s);
    }

    public void newLine() {
        buf.append('\n');
    }

    public void endIf() {
        append("{FI}");
        newLine();
    }

    public void startIf(String clause) {
        append("{IF ");
        append(clause);
        append("}");
        newLine();
    }

    public void openHtmlSpan(String className) {
        openTag("span", className, false);
    }

    public void openOdtSpan(String styleName) {
        openOdtTag("text:span", styleName, false);
    }

    public void closeHtmlSpan() {
        closeTag("span", false);
    }

    public void closeOdtSpan() {
        closeTag("text:span", false);
    }

    public void openHtmlP(String className) {
        openTag("p", className, false);
    }

    public void openOdtP(String styleName) {
        openOdtTag("text:p", styleName, false);
    }

    public void closeOdtP() {
        closeTag("text:p", true);
    }

    public void closeHtmlP() {
        closeTag("p", true);
    }

    public void openHtmlUl(String className) {
        openTag("ul", className, true);
    }

    public void openOdtUl(String styleName) {
        openOdtTag("text:list", styleName, true);
    }

    public void closeOdtUl() {
        closeTag("text:list", true);
    }

    public void closeHtmlUl() {
        closeTag("ul", true);
    }

    public void openOdtH(int level) {
        String levelString = String.valueOf(level);
        append("<");
        append("text:h");
        append(" text:style-name=\"");
        append("Heading_20_");
        append(levelString);
        append("\"");
        append(" text:outline-level=\"");
        append(levelString);
        append("\"");
        append(">");
    }

    public void closeOdtH() {
        closeTag("text:h", true);
    }

    public void openTag(String name, String className, boolean newLine) {
        append("<");
        append(name);
        if (className != null) {
            append(" class=\"");
            append(className);
            append("\"");
        }
        append(">");
        if (newLine) {
            newLine();
        }
    }

    public void openOdtTag(String name, String styleName, boolean newLine) {
        append("<");
        append(name);
        if (styleName != null) {
            append(" text:style-name=\"");
            append(styleName);
            append("\"");
        }
        append(">");
        if (newLine) {
            newLine();
        }
    }

    public void closeTag(String name, boolean newLine) {
        append("</");
        append(name);
        append(">");
        if (newLine) {
            newLine();
        }
    }

    public void openDiv(String className) {
        openTag("div", className, true);
    }

    public void closeDiv() {
        closeTag("div", true);
    }

    public void appendApplyTemplates(String select, String mode) {
        append("{_XAT= .");
        if (mode != null) {
            append(", mode= ");
            append(mode);
        }
        append("}");
    }

    public void appendValueOf(String select) {
        append("{_XVO= ");
        append(select);
        append("}");
    }

    public void appendTemplate(String select) {
        newLine();
        newLine();
        append("{_XT= ");
        append(select);
        append("}");
        newLine();
        newLine();
    }

}
